SystemOrganization addCategory: #'Nebraska-Morphic-Collaborative'!
SystemOrganization addCategory: #'Nebraska-Morphic-Experimental'!
SystemOrganization addCategory: #'Nebraska-Morphic-Remote'!
SystemOrganization addCategory: #'Nebraska-Network-EToy Communications'!
SystemOrganization addCategory: #'Nebraska-Network-ObjectSocket'!
SystemOrganization addCategory: #'Nebraska-Audio Chat'!

Model subclass: #NebraskaServer
	instanceVariableNames: 'worldDepth world clients listenQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NebraskaServer commentStamp: '<historical>' prior: 0!
A Nebraska server has a private world and some collection of clients.  It associates a RemoteControlledHand for each client.  Redraw events in the world are broadcasted to all connected clients.  A Nebraska server can listen on a TCP/IP port and accept new clients. Current version has been modified so that the server serves the world in which it was launched. Other variations are certainly possible.

To start a server, execute the following code:
	NebraskaServerMorph serveWorld: World

To start a client, run the following in another image:
	NetworkTerminalMorph openAndConnectTo: 'servername'

Fill in your server's hostname for 'servername'. At this point, everything should be working!!

Before starting a server, you can tweak these:
BufferedCanvas enabled: false.
BufferedCanvas enabled: true.

At any time you can do these:
NebraskaDebug beginStats
NebraskaDebug showStats
NebraskaDebug showStats: #delays
NebraskaDebug showStats: #bigImage
NebraskaDebug showStats: #FormEncodeTimes
NebraskaDebug killStats

NOTE: if you want to have a local view of the server, you shouldn't use the TCP connections. The problem is that the server will occasionally do a #flush, and it won't work due to single threading. The better solution is to use a LoopBackStringSocket instead of a regular StringSocket, but there is no handy method for that right now....


!

!NebraskaServer class methodsFor: 'as yet unclassified' stamp: 'RAA 7/24/2000 12:06'!
defaultPort

	^9091! !

!NebraskaServer class methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 14:59'!
extremelyBigRectangle

	^(0@0 extent: 5000@5000)! !

!NebraskaServer class methodsFor: 'instance creation' stamp: 'mu 11/28/2003 19:38'!
newForWorld: aWorld

	^self basicNew initializeForWorld: aWorld! !

!NebraskaServer class methodsFor: 'instance creation' stamp: 'bf 7/1/2008 19:49'!
serveWorld: aWorld

	^self serveWorld: aWorld onPort: self defaultPorts! !

!NebraskaServer class methodsFor: 'instance creation' stamp: 'RAA 11/6/2000 17:22'!
serveWorld: aWorld onPort: aPortNumber

	| server |

	Utilities authorName.	"since we will need it later"

	server := self newForWorld: aWorld.
	server startListeningOnPort: aPortNumber.
	^server
	"server acceptNullConnection"		"server acceptPhonyConnection."
! !

!NebraskaServer methodsFor: 'networking' stamp: 'ls 3/25/2000 22:35'!
acceptNewConnections
	| connection |
	listenQueue ifNil: [ ^self ].
	[ clients size > 20 ifTrue: [ "too many connections!!" ^self ].
	  connection := listenQueue getConnectionOrNil.  
	  connection isNil ] 
	whileFalse: [
	  self addClientFromConnection: (StringSocket on: connection) ].! !

!NebraskaServer methodsFor: 'networking' stamp: 'RAA 7/22/2000 07:23'!
acceptNullConnection

	| twins |

	twins := LoopbackStringSocket newPair.
	self addClientFromConnection: twins first.
	(NullTerminalMorph new connection: twins second) openInWorld.
! !

!NebraskaServer methodsFor: 'networking' stamp: 'RAA 7/20/2000 12:57'!
acceptPhonyConnection

	| twins |

	twins := LoopbackStringSocket newPair.
	self addClientFromConnection: twins first.
	(NetworkTerminalMorph new connection: twins second) inspect "openInWorld".
! !

!NebraskaServer methodsFor: 'networking' stamp: 'RAA 7/20/2000 10:03'!
addClientFromConnection: connection
	| client |

	client := NebraskaClient onConnection: connection.
	clients add: client.
	client extent: world extent  depth: worldDepth.
	world addRemoteClient: client.
	self changed: #numClients.! !

!NebraskaServer methodsFor: 'networking' stamp: 'RAA 8/1/2000 00:01'!
backlog

	^clients inject: 0 into: [ :max :each | max max: each backlog]! !

!NebraskaServer methodsFor: 'accessing' stamp: 'ar 10/26/2000 14:23'!
clients
	^clients ifNil:[#()].! !

!NebraskaServer methodsFor: 'initialization' stamp: 'ar 10/26/2000 14:02'!
destroy
	self stopListening.
	clients do:[:each| each destroy].
	self breakDependents.! !

!NebraskaServer methodsFor: 'attributes' stamp: 'ls 4/11/2000 19:48'!
extent: newExtent  depth: newDepth
	"modify the extent and/or depth of the shared world"
	clients do: [ :client |
		client extent: newExtent depth: newDepth ].
	world extent: newExtent.

	worldDepth := newDepth.! !

!NebraskaServer methodsFor: 'initialization' stamp: 'ar 10/26/2000 14:20'!
initialize
	clients := IdentitySet new.
	self extent: 800@600 depth: 16.! !

!NebraskaServer methodsFor: 'initialization' stamp: 'ar 10/26/2000 13:49'!
initializeForWorld: aWorld

	world := aWorld.
	clients := IdentitySet new.
	self extent: world extent depth: Display depth.
	aWorld remoteServer: self.! !

!NebraskaServer methodsFor: 'attributes' stamp: 'ls 4/11/2000 18:41'!
numClients
	"return the number of connected clients"
	^clients size! !

!NebraskaServer methodsFor: 'networking' stamp: 'ls 3/25/2000 22:36'!
processIO
	self pruneDeadConnections.
	self acceptNewConnections.! !

!NebraskaServer methodsFor: 'networking' stamp: 'ar 10/26/2000 14:20'!
pruneDeadConnections
	| deadConnections |
	deadConnections := clients select: [ :client | client isConnected not ].
	deadConnections do: [ :client |
		world removeRemoteClient: client].

	deadConnections isEmpty ifTrue:[ ^self ].

	clients removeAll: deadConnections.
	self changed: #numClients.! !

!NebraskaServer methodsFor: 'attributes' stamp: 'ls 3/25/2000 23:13'!
sharedWorld
	^world! !

!NebraskaServer methodsFor: 'networking' stamp: 'ls 3/25/2000 22:31'!
startListeningOnPort: portNumber
	Socket initializeNetwork.
	self stopListening.
	listenQueue := ConnectionQueue portNumber: portNumber  queueLength: 5.! !

!NebraskaServer methodsFor: 'menus' stamp: 'RAA 7/31/2000 22:28'!
step

	self processIO.

	"savedWorld := Processor activeProcess world.
	Processor activeProcess setWorld: world."

	self flag: #bob.		"in this version, world is THE WORLD, so it steps itself"
	"world doOneCycle."

	"Processor activeProcess setWorld: savedWorld."

	clients do: [ :each | each canvas apply: [ :ignore | ]].	"for modes that need a little push"
! !

!NebraskaServer methodsFor: 'networking' stamp: 'ls 3/25/2000 22:32'!
stopListening
	listenQueue ifNil: [ ^self ].
	listenQueue destroy.
	listenQueue := nil.! !

EToyCommunicatorMorph subclass: #AudioChatGUI
	instanceVariableNames: 'mycodec myrecorder mytargetip myalert playOnArrival theConnectButton soundBlockNumber soundMessageID queueForMultipleSends transmitWhileRecording theTalkButton handsFreeTalking handsFreeTalkingFlashTime'
	classVariableNames: 'DebugLog LiveMessages NewAudioMessages PlayOnArrival'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/9/2000 16:12'!
debugLog: x
"
AudioChatGUI debugLog: nil
AudioChatGUI debugLog: OrderedCollection new
DebugLog LiveMessages NewAudioMessages PlayOnArrival 
"
	DebugLog := x.
! !

!AudioChatGUI class methodsFor: 'parts bin' stamp: 'md 8/10/2006 11:53'!
descriptionForPartsBin
	"Answer a description of the receiver for use in a parts bin"

	^ self partName: 	'Audio chat'
		categories:		#('Collaborative')
		documentation:	'A tool for talking to other Squeak users'
		sampleImageForm: (Form
	extent: 110@70
	depth: 8
	fromArray: #( 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193909241 4193845248 4193909241 4193845505 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 33159673 4193845248 4193909241 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843257 4193845248 4193908993 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 4193845248 4193845505 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 33095680 4193845505 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789619457 33095680 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789642629 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466465 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 2239817189 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857048960 2246173153 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676933 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3850756577 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789653477 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857024481 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789642725 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857013729 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3783321061 3842048257 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857048960 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3783648741 31843813 31843813 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3842106853 3857048965 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3783321061 31843813 3857048833 16901605 3842106625 31843813 3842106625 31843813 3842048257 3857049061 16901605 16843237 3857048960 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3783648741 31843813 3856990693 3856990693 16843237 3842106853 16843237 3842106853 31843813 31843585 3856990693 3842106853 3857048965 3789619457 16842752 4177592577 31580641 3777505320 673720360 673720360 685891880 673720360 673720360 673720545 3777505320 673720360 673720360 685892065 3783321061 31843813 3856990693 3856990693 3842106853 3842106853 3842106853 3842106853 16843009 31785445 3857049061 3842106853 3857048960 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3783648741 31843813 3856990693 3856990693 3842106853 3842106853 3842106853 3842106853 31843813 3856990693 3857049061 3842106853 3857048965 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3783321061 31843813 31785445 3856990693 3842106853 3842106853 3842106853 3842106853 31843813 31843585 3856990693 3842106853 3857048960 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3783648741 3842048257 3857048833 16901605 16843237 16843237 16843237 16843237 3842048257 3857049061 16901605 3856990693 3857048965 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3783321061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857048960 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3789642725 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857013729 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3789653477 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857024481 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3789676933 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3850756577 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3789677025 2239817189 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857048960 2246173153 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3789677025 3789642629 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466373 1535466465 3789677025 3789619457 16842752 4177592577 31580641 3777552865 3789677025 3789677025 685891880 3789677025 3789677025 3789629665 3777552865 3789677025 3789677025 685892065 3789677025 3789677025 3789677025 3789676928 2239792512 2239792512 2239792512 2239792512 2239792512 2239816161 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3777505320 673720360 673720360 685891880 673720360 673720360 673720545 3777505320 673720360 673720360 685892065 3789677025 3789677025 3789677025 3783613413 3857049061 3857049061 3857049061 3857049061 3857049061 3857013637 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 2246436325 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 2246173153 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676933 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3850756577 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 16843009 16843009 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789642725 3842048257 16901605 16901605 3857049061 3857049061 3857049061 3857049061 3857013729 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789619457 16843009 16843009 16843233 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789653477 3856990693 3856990693 3842106853 3857049061 3857049061 3857049061 3857049061 3857024481 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 16843009 1888776340 1888776340 1879113985 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789642725 3856990693 3856990693 3842106853 3842048257 3857048833 16901377 16901605 3857013729 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789676801 16880752 2490406000 2490406000 2490405889 16900577 3789677025 3789677025 3789677025 3789677025 3789677025 3789653477 3856990693 3856990693 3842106853 31843813 31843813 31843813 31843813 3857024481 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789619457 26505364 1888776340 1888776340 1888776340 16843233 3789677025 3789677025 3789677025 3789677025 3789677025 3789642725 3856990465 16901605 3842106853 3842048257 31843813 3842106625 3857049061 3857013729 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789619457 2490406000 2490406000 2490406000 2490406000 2483093985 3789677025 3789677025 3789677025 3789677025 3789677025 3789653477 3856990693 3857049061 3842106853 31843813 31843813 3842106625 3857049061 3857024481 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3774939540 1888776340 1888776340 1888776340 1888776340 1888747777 3789677025 3789677025 3789677025 3789677025 3789677025 3789642725 3856990693 3857049061 3842106853 31843813 31843813 3856990693 3857049061 3857013729 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3774939504 2490406000 2490406000 2490406000 2490406000 2490368257 3789677025 3789677025 3789677025 3789677025 3789677025 3789653477 3842048257 3857049061 16843237 3842048257 3842106853 3856990693 3857049061 3857024481 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3774939393 3789677025 16871572 1888776340 1895825407 1888776340 1888776340 1888776193 31580641 3789677025 3789677025 3789677025 3789677025 3789642725 3857049061 3857049061 3857049061 3857049061 3857049061 3842106853 3857049061 3857013729 3789677025 3789677025 3789619457 16842752 4177592577 31580641 31580641 31580641 16880752 2490406000 4285568112 4285568112 2490406000 2490405889 31580641 3789677025 3789677025 3789677025 3789677025 3789676928 3857049061 3857049061 3857049061 3857049061 3857049061 3842106853 3857049061 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 31580641 31580641 16871572 1888776340 4287918228 4287918228 1888776340 1888776193 31580641 3789677025 3789677025 3789677025 3789677025 3789676933 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3850756577 3789677025 3789677025 3789619457 16842752 4177592577 31580641 31580641 31580641 16880752 2490406000 4285568112 4285568112 2490406000 2490405889 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 2246436325 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 2246173153 3789677025 3789677025 3789619457 16842752 4177592577 31580641 31580641 31580641 16871572 1888776340 4287918228 4287918228 1888776340 1888776193 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3783613413 3857049061 3857049061 3857049061 3857049061 3857049061 3857013637 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 31580641 31580641 16880752 2490406000 4285568112 4285568112 2490406000 2490405889 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 2239792512 2239792512 2239792512 2239792512 2239792512 2239816161 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 31580641 31580641 16871572 1888776340 4287918228 4287918228 1888776340 1888776193 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 1535466373 1535466373 1535466373 1535466373 1535466373 1541530081 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3774939393 3789677025 16880752 2490406000 2499805183 2490406000 2490406000 2490405889 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789653376 3857049061 3857049061 3857049061 3857049061 3857049061 3850405345 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 16871572 1888776340 1888776340 1888776340 1888776340 1888776193 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3783648741 3857049061 3857049061 3857049061 3857049061 3857049061 3857048965 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3774939504 2490406000 2490406000 2490406000 2490406000 2490368257 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 2246436325 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 2246173153 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3774939540 1888776340 1888776340 1888776340 1888776340 1888747777 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 1541793253 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 1541530081 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789619457 2490406000 2490406000 2490406000 2490406000 2483093985 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 3856990465 16843237 3857049061 3857048833 31843813 31843813 3857049061 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789619457 26505364 1888776340 1888776340 1888776340 16843233 3789677025 3789677025 3789677025 3789677025 3789677025 3789676933 3856990693 31785445 3857049061 3857049061 31843585 31843813 3857049061 3850756577 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789676801 16880752 2490406000 2490406000 2490405889 16900577 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 3857049061 31843813 3842048257 3857049061 31843813 31843585 31843813 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 16843009 1888776340 1888776340 1879113985 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789676933 3857049061 31843813 31843813 31843813 31843813 31785445 3857049061 3850756577 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789619457 16843009 16843009 16843233 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 3857049061 31843813 3842048257 31843813 31843813 16901605 3857049061 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 16843009 16843009 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676933 3857049061 31843813 31843813 31843813 31843813 31785445 3857049061 3850756577 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 3857049061 31843813 31843813 31843813 31843813 31843585 3857049061 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676933 3857048833 16901605 3842048257 3842106625 16901377 16901377 31843813 3850756577 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789676928 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 3850428897 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 1541793253 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 1541530081 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 2246436325 3857049061 3857049061 3857049061 3857049061 3857049061 3857049061 2246173153 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3783648741 3857049061 3857049061 3857049061 3857049061 3857049061 3857048965 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789653376 3857049061 3857049061 3857049061 3857049061 3857049061 3850405345 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 1535466373 1535466373 1535466373 1535466373 1535466373 1541530081 3789677025 3789677025 3789677025 3789619457 16842752 4177592577 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789619457 16842752 4193845505 31580641 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789677025 3789619457 33095680 4193845505 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 33095680 4193908993 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 4193845248 4193909241 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843257 4193845248 4193909241 4193845505 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 16843009 33159673 4193845248)
	offset: 0@0)! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/11/2000 11:54'!
handleNewAudioChat2From: dataStream sentBy: senderName ipAddress: ipAddressString

	| newSound seqSound compressed |

	compressed := self newCompressedSoundFrom: dataStream.
	newSound := compressed asSound.
"-------an experiment to try
newSound adjustVolumeTo: 7.0 overMSecs: 10
--------"
DebugLog ifNotNil: [
	DebugLog add: {compressed. newSound}.
].
	LiveMessages ifNil: [LiveMessages := Dictionary new].
	seqSound := LiveMessages at: ipAddressString ifAbsentPut: [SequentialSound new].
	seqSound isPlaying ifTrue: [
		seqSound
			add: newSound;
			pruneFinishedSounds.
	] ifFalse: [
		seqSound
			initialize;
			add: newSound.
	].
	seqSound isPlaying ifFalse: [seqSound play].! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/9/2000 19:28'!
handleNewAudioChatFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	| compressed |

	compressed := self newCompressedSoundFrom: dataStream.
DebugLog ifNotNil: [
	DebugLog add: {compressed}.
].

	self newAudioMessages nextPut: compressed.
	self playOnArrival ifTrue: [self playNextAudioMessage].
	
! !

!AudioChatGUI class methodsFor: 'class initialization' stamp: 'RAA 8/5/2000 19:22'!
initialize

	EToyIncomingMessage
		forType: EToyIncomingMessage typeAudioChat 
		send: #handleNewAudioChatFrom:sentBy:ipAddress: 
		to: self.

	EToyIncomingMessage
		forType: EToyIncomingMessage typeAudioChatContinuous
		send: #handleNewAudioChat2From:sentBy:ipAddress: 
		to: self.


! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 12:16'!
newAudioMessages

	^NewAudioMessages ifNil: [NewAudioMessages := SharedQueue new].! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/9/2000 19:28'!
newCompressedSoundFrom: dataStream

	| samplingRate |

	samplingRate := (dataStream upTo: 0 asCharacter) asNumber.
	^CompressedSoundData new 
		withEToySound: dataStream upToEnd
		samplingRate: samplingRate.
! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 13:01'!
numberOfNewMessages

	^self newAudioMessages size! !

!AudioChatGUI class methodsFor: 'creation' stamp: 'RAA 8/4/2000 14:06'!
openAsMorph

	AudioChatGUI new openInWorld.	"old syswindow version in #start"

! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/6/2000 14:23'!
playNextAudioMessage

	(self newAudioMessages nextOrNil ifNil: [^self]) asSound play.! !

!AudioChatGUI class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 13:14'!
playOnArrival

	^PlayOnArrival ifNil: [PlayOnArrival := false]! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/2/2000 16:33'!
buttonColor 

	^Color lightBrown! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'aoy 2/17/2003 01:01'!
changeTalkButtonLabel
	| bText |
	self transmitWhileRecording.
	handsFreeTalking 
		ifTrue: 
			[theTalkButton
				labelUp: 'Talk';
				labelDown: 'Release';
				label: 'Talk'.
			bText := 'Click once to begin a message. Click again to end the message.']
		ifFalse: 
			[theTalkButton
				labelUp: 'Talk';
				labelDown: (transmitWhileRecording 
							ifTrue: ['TALKING']
							ifFalse: ['RECORDING']);
				label: 'Talk'.
			bText := 'Press and hold to record a message.'].
	bText := transmitWhileRecording 
		ifTrue: [bText , ' The message will be sent while you are speaking.']
		ifFalse: [bText , ' The message will be sent when you are finished.'].
	theTalkButton setBalloonText: bText! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'ar 8/6/2009 20:38'!
connect

	mytargetip := UIManager default 
		request: 'Connect to?' 
		initialAnswer: (mytargetip ifNil: ['']).
	mytargetip := NetNameResolver stringFromAddress: (
		(NetNameResolver addressFromString: mytargetip) ifNil: [^mytargetip := '']
	)
! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/2/2000 16:36'!
connectButton
	
	^SimpleButtonMorph new
		label: 'Connect';
		color: self buttonColor;
		target: self;
		actWhen: #buttonUp;
		actionSelector: #connect;
		setBalloonText: 'Press to connect to another audio chat user.'

! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'RAA 8/4/2000 13:09'!
currentConnectionStateString

	^'?'
! !

!AudioChatGUI methodsFor: 'user interface' stamp: 'TBP 3/5/2000 16:22'!
defaultBackgroundColor
	"In a better design, this would be handled by preferences."
	^Color yellow."r: 1.0 g: 0.7 b: 0.8"! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:36'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 4! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:24'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color yellow! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'RAA 8/6/2000 18:27'!
getChoice: aSymbol

	aSymbol == #playOnArrival ifTrue: [^self class playOnArrival].
	aSymbol == #transmitWhileRecording ifTrue: [^self transmitWhileRecording].
	aSymbol == #handsFreeTalking ifTrue: [^self handsFreeTalking].

! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/6/2000 18:25'!
handsFreeTalking

	^handsFreeTalking ifNil: [handsFreeTalking := false].! !

!AudioChatGUI methodsFor: 'user interface' stamp: 'TBP 3/5/2000 16:02'!
initialExtent
	"Nice and small--that was the idea.
	It shouldn't take up much screen real estate."
	^200@100! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'dgd 2/14/2003 19:48'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	transmitWhileRecording := false.
	handsFreeTalking := false.
	mycodec := GSMCodec new.
	myrecorder := ChatNotes new.
	mytargetip := ''.
	
	self start2.
	self changeTalkButtonLabel! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/4/2000 14:26'!
ipAddress: aString

	mytargetip := aString! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/12/2000 16:22'!
messageWaitingAlertIndicator

	| messageCounter |
	myalert := AlertMorph new socketOwner: self.
	messageCounter := UpdatingStringMorph on: self selector: #objectsInQueue.
	myalert addMorph: messageCounter.
	messageCounter contents: '0'; color: Color white.
	messageCounter align: messageCounter center with: myalert center.
	myalert setBalloonText: 'New messages indicator. This will flash and show the number of messages when there are messages that you haven''t listened to. You can click here to play the next message.'.

	myalert on: #mouseUp send: #playNextMessage to: self.
	^myalert! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'RAA 8/4/2000 13:01'!
objectsInQueue

	^self class numberOfNewMessages! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/2/2000 16:34'!
playButton

	^SimpleButtonMorph new
		label: 'Play';
		color: self buttonColor;
		target: self;
		actWhen: #buttonUp;
		actionSelector: #playNextMessage;
		setBalloonText: 'Play the next new message.'

! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'RAA 8/4/2000 12:26'!
playNextMessage

	self class playNextAudioMessage.
! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/6/2000 09:47'!
record

	queueForMultipleSends := nil.
	myrecorder record.! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/2/2000 16:37'!
recordAndStopButton

	^ChatButtonMorph new
		labelUp: 'Record';
		labelDown: 'RECORDING';
		label: 'Record';
		color: self buttonColor;
		target: self;
		actionUpSelector: #stop;
		actionDownSelector: #record;
		setBalloonText: 'Press and hold to record a message. It will be sent when you release the mouse.'
! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'RAA 8/4/2000 14:59'!
removeConnectButton

	theConnectButton ifNotNil: [
		theConnectButton delete.
		theConnectButton := nil.
	].! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/12/2000 15:01'!
samplingRateForTransmission

	^11025		"try to cut down on amount of data sent for live chats"! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/13/2000 11:44'!
send

	| null rawSound aSampledSound |

	mytargetip isEmpty ifTrue: [
		^self inform: 'You must connect with someone first.'.
	].
	rawSound := myrecorder recorder recordedSound ifNil: [^self].
	aSampledSound := rawSound asSampledSound.
"Smalltalk at: #Q3 put: {rawSound. rawSound asSampledSound. aCompressedSound}."
	self transmitWhileRecording ifTrue: [
		self sendOneOfMany: rawSound asSampledSound.
		queueForMultipleSends ifNotNil: [queueForMultipleSends nextPut: nil].
		queueForMultipleSends := nil.
		^self
	].

	null := String with: 0 asCharacter.
	EToyPeerToPeer new 
		sendSomeData: {
			EToyIncomingMessage typeAudioChat,null. 
			Preferences defaultAuthorName,null.
			aSampledSound originalSamplingRate asInteger printString,null.
			(mycodec compressSound: aSampledSound) channels first.
		}
		to: mytargetip
		for: self.

! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/12/2000 14:34'!
sendAnyCompletedSounds

	| soundsSoFar firstCompleteSound |

	myrecorder isRecording ifFalse: [^self].
	mytargetip isEmpty ifTrue: [^self].
	soundsSoFar := myrecorder recorder recordedSound ifNil: [^self].
	firstCompleteSound := soundsSoFar removeFirstCompleteSoundOrNil ifNil: [^self].
	self sendOneOfMany: firstCompleteSound.! !

!AudioChatGUI methodsFor: 'sending' stamp: 'nice 12/27/2009 03:10'!
sendOneOfMany: aSampledSound

	| null message aCompressedSound t ratio resultBuf maxVal |

	self samplingRateForTransmission = aSampledSound originalSamplingRate ifTrue: [
		aCompressedSound := mycodec compressSound: aSampledSound.
	] ifFalse: [
		t := [ | oldSamples val newCount fromIndex |
			ratio := aSampledSound originalSamplingRate // self samplingRateForTransmission.
			oldSamples := aSampledSound samples.
			newCount := oldSamples monoSampleCount // ratio.
			resultBuf := SoundBuffer newMonoSampleCount: newCount.
			fromIndex := 1.
			maxVal := 0.
			1 to: newCount do: [ :i |
				maxVal := maxVal max: (val := oldSamples at: fromIndex).
				resultBuf at: i put: val.
				fromIndex := fromIndex + ratio.
			].
		] timeToRun.
		NebraskaDebug at: #soundReductionTime add: {t. maxVal}.
		maxVal < 400 ifTrue: [
			NebraskaDebug at: #soundReductionTime add: {'---dropped---'}.
			^self
		].		"awfully quiet"
		aCompressedSound := mycodec compressSound: (
			SampledSound new 
				setSamples: resultBuf 
				samplingRate: aSampledSound originalSamplingRate // ratio
		).
	].

	null := String with: 0 asCharacter.
	message := {
		EToyIncomingMessage typeAudioChatContinuous,null. 
		Preferences defaultAuthorName,null.
		aCompressedSound samplingRate asInteger printString,null.
		aCompressedSound channels first.
	}.
	queueForMultipleSends ifNil: [
		queueForMultipleSends := EToyPeerToPeer new 
			sendSomeData: message
			to: mytargetip
			for: self
			multiple: true.
	] ifNotNil: [
		queueForMultipleSends nextPut: message
	].

! !

!AudioChatGUI methodsFor: 'stepping and presenter' stamp: 'RAA 8/4/2000 14:05'!
start

	| myUpdatingText playButton myOpenConnectionButton myStopButton window  |
"
--- old system window version ---
"
	Socket initializeNetwork.
	myrecorder initialize.

	window := (SystemWindow labelled: 'iSCREAM') model: self.

	myalert := AlertMorph new.
	myalert socketOwner: self.
	window addMorph: myalert frame: (0.35@0.4 corner: 0.5@0.7).

	(playButton := self playButton) center: 200@300.
	window addMorph: playButton frame: (0.5@0.4 corner: 1.0@0.7).

	(myOpenConnectionButton := self connectButton) center: 250@300.
	window addMorph: myOpenConnectionButton frame: (0.5@0 corner: 1.0@0.4).

	(myStopButton := self recordAndStopButton) center: 300@300.
	window addMorph: myStopButton frame: (0.5@0.7 corner: 1.0@1.0).

	myUpdatingText := UpdatingStringMorph on: self selector: #objectsInQueue.
	window addMorph: myUpdatingText frame: (0.41@0.75 corner: 0.45@0.95).

	"myUserList init."! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/12/2000 16:25'!
start2

	Socket initializeNetwork.
	myrecorder initialize.

	self addARow: {
		self inAColumn: {
			(
				self inARow: {
					self inAColumn: {self toggleForSendWhileTalking}.
					self inAColumn: {self toggleForHandsFreeTalking}.
					self inAColumn: {self toggleForPlayOnArrival}.
				}
			) hResizing: #shrinkWrap.
			self inARow: {
				self talkBacklogIndicator.
				self messageWaitingAlertIndicator.
			}.
		}.
		self inAColumn: {
			theConnectButton := self connectButton.
			self playButton.
			theTalkButton := self talkButton.
		}.
	}.
! !

!AudioChatGUI methodsFor: 'stepping and presenter' stamp: 'RAA 8/12/2000 18:11'!
step

	| now |
	super step.
	self transmitWhileRecording ifTrue: [self sendAnyCompletedSounds].
	self handsFreeTalking & myrecorder isRecording ifTrue: [
		now := Time millisecondClockValue.
		((handsFreeTalkingFlashTime ifNil: [0]) - now) abs > 200 ifTrue: [
			theTalkButton color: (
				theTalkButton color = self buttonColor 
						ifTrue: [Color white] 
						ifFalse: [self buttonColor]
			).
			handsFreeTalkingFlashTime := now.
		].
	].
	self class playOnArrival ifTrue: [self playNextMessage].

	"myrecorder ifNotNil: [
		myrecorder recorder samplingRate printString ,'   ',
		SoundPlayer samplingRate printString,'    '

		displayAt: 0@0
	]."! !

!AudioChatGUI methodsFor: 'testing' stamp: 'RAA 8/12/2000 18:09'!
stepTime

	myrecorder ifNil: [^200].
	myrecorder isRecording ifFalse: [^200].
	^20! !

!AudioChatGUI methodsFor: 'testing' stamp: 'RAA 8/2/2000 07:47'!
stepTimeIn: aSystemWindow

	^self stepTime
! !

!AudioChatGUI methodsFor: 'stepping and presenter' stamp: 'Tbp 4/11/2000 16:49'!
stop

	myrecorder stop.
	self send.! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/12/2000 16:18'!
talkBacklog

	^(queueForMultipleSends ifNil: [^0]) size // 2! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/12/2000 16:24'!
talkBacklogIndicator

	^(UpdatingStringMorph on: self selector: #talkBacklog)
		setBalloonText: 'Approximate number of seconds of delay in your messages getting to the other end.'! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/7/2000 06:52'!
talkButton

	^ChatButtonMorph new
		labelUp: 'xxx';
		labelDown: 'xxx';
		label: 'xxx';
		color: self buttonColor;
		target: self;
		actionUpSelector: #talkButtonUp;
		actionDownSelector: #talkButtonDown;
		setBalloonText: 'xxx'
! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/9/2000 18:05'!
talkButtonDown

	EToyListenerMorph confirmListening.
	self handsFreeTalking ifFalse: [^self record].
	theTalkButton label: 'Release'.
! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/9/2000 18:13'!
talkButtonUp

	theTalkButton recolor: self buttonColor.
	self handsFreeTalking ifFalse: [^self stop].
	myrecorder isRecording ifTrue: [
		theTalkButton label: 'Talk'.
		^self stop.
	].
	self record.
	theTalkButton label: 'TALKING'.


! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'RAA 8/7/2000 06:52'!
toggleChoice: aSymbol

	aSymbol == #playOnArrival ifTrue: [
		^PlayOnArrival := self class playOnArrival not
	].
	aSymbol == #transmitWhileRecording ifTrue: [
		transmitWhileRecording := self transmitWhileRecording not.
		self changeTalkButtonLabel.
		^transmitWhileRecording
	].
	aSymbol == #handsFreeTalking ifTrue: [
		handsFreeTalking := self handsFreeTalking not.
		self changeTalkButtonLabel.
		^handsFreeTalking
	].


! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/12/2000 16:14'!
toggleForHandsFreeTalking

	^self
		simpleToggleButtonFor: self 
		attribute: #handsFreeTalking 
		help: 'Whether you want to talk without holding the mouse down.'! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/12/2000 16:15'!
toggleForPlayOnArrival

	^self
		simpleToggleButtonFor: self 
		attribute: #playOnArrival 
		help: 'Whether you want to play messages automatically on arrival.'! !

!AudioChatGUI methodsFor: 'initialization' stamp: 'RAA 8/12/2000 16:14'!
toggleForSendWhileTalking

	^self
		simpleToggleButtonFor: self 
		attribute: #transmitWhileRecording 
		help: 'Whether you want to send messages while recording.'! !

!AudioChatGUI methodsFor: 'sending' stamp: 'RAA 8/6/2000 13:08'!
transmitWhileRecording

	^transmitWhileRecording ifNil: [transmitWhileRecording := false]! !

EToyCommunicatorMorph subclass: #EToyChatOrBadgeMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Experimental'!

EToyChatOrBadgeMorph subclass: #EToyChatMorph
	instanceVariableNames: 'listener receivingPane myForm recipientForm acceptOnCR sendingPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!EToyChatMorph commentStamp: '<historical>' prior: 0!
EToyChatMorph new open setIPAddress: '1.2.3.4'

"
EToyChatMorph represents a chat session with another person. Type your message in the top text pane and press cmd-S.
"!

!EToyChatMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 14:51'!
chatFrom: ipAddress name: senderName text: text

	| chatWindow |

	chatWindow := self 
		chatWindowForIP: ipAddress 
		name: senderName 
		picture: (EToySenderMorph pictureForIPAddress: ipAddress) 
		inWorld: self currentWorld.
	chatWindow
		chatFrom: ipAddress 
		name: senderName 
		text: text
! !

!EToyChatMorph class methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:45'!
chatWindowForIP: ipAddress name: senderName picture: aForm inWorld: aWorld

	| makeANewOne aSenderBadge existing |

	existing _ self instanceForIP: ipAddress inWorld: aWorld.
	existing ifNotNil: [^existing].
	makeANewOne _ [
		self new
			recipientForm: aForm; 
			open; 
			setIPAddress: ipAddress
	].
	EToyCommunicatorMorph playArrivalSound.
	self doChatsInternalToBadge ifTrue: [
		aSenderBadge _ EToySenderMorph instanceForIP: ipAddress inWorld: aWorld.
		aSenderBadge ifNotNil: [
			aSenderBadge startChat: false.
			^aSenderBadge 
				findDeepSubmorphThat: [ :x | x isKindOf: EToyChatMorph] 
				ifAbsent: makeANewOne
		].
		aSenderBadge _ EToySenderMorph instanceForIP: ipAddress.
		aSenderBadge ifNotNil: [
			aSenderBadge _ aSenderBadge veryDeepCopy.
			aSenderBadge 
				killExistingChat;
				openInWorld: aWorld;
				startChat: false.
			^aSenderBadge 
				findDeepSubmorphThat: [ :x | x isKindOf: EToyChatMorph] 
				ifAbsent: makeANewOne
		].
		(aSenderBadge _ EToySenderMorph new)
			userName: senderName 
			userPicture: aForm
			userEmail: 'unknown'  translated
			userIPAddress: ipAddress;
			position: 200@200;
			openInWorld: aWorld;
			startChat: false.
		^aSenderBadge 
			findDeepSubmorphThat: [ :x | x isKindOf: EToyChatMorph] 
			ifAbsent: makeANewOne
	].
	^makeANewOne value.

! !

!EToyChatMorph class methodsFor: 'parts bin' stamp: 'sw 7/29/2007 01:58'!
descriptionForPartsBin

	^ self partName: 	'Text chat' translatedNoop
		categories:		#()
		documentation:	'A tool for sending messages to other Squeak users' translatedNoop! !

!EToyChatMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 17:12'!
doChatsInternalToBadge

	^true! !

!EToyChatMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/16/2000 12:48'!
instanceForIP: ipAddress inWorld: aWorld

	^self allInstances detect: [ :x | 
		x world == aWorld and: [x ipAddress = ipAddress]
	] ifNone: [nil]

! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/5/2000 14:14'!
acceptTo: someText forMorph: aMorph

	| betterText |

	betterText := self improveText: someText forMorph: aMorph.
	self 
		transmitStreamedObject: (betterText eToyStreamedRepresentationNotifying: self) 
		to: self ipAddress.
	aMorph setText: '' asText.
	self appendMessage: 
		self startOfMessageFromMe,
		' - ',
		betterText,
		String cr.

	^true! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/10/2000 22:34'!
appendMessage: aText

	receivingPane appendTextEtoy: aText.! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'ar 12/17/2001 02:18'!
chatFrom: ipAddress name: senderName text: text

	| initialText attrib |

	recipientForm ifNil: [
		initialText := senderName asText allBold.
	] ifNotNil: [
		attrib := TextAnchor new anchoredMorph: recipientForm "asMorph".
		initialText := (String value: 1) asText.
		initialText addAttribute: attrib from: 1 to: 1.
	].
	self appendMessage: initialText,' - ',text,String cr.
	EToyCommunicatorMorph playArrivalSound.


! !

!EToyChatMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:34'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ self standardBorderColor! !

!EToyChatMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:36'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 8! !

!EToyChatMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:07'!
defaultBounds
"answer the default bounds for the receiver"
	^ 400 @ 100 extent: 200 @ 150! !

!EToyChatMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:25'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color paleYellow! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 10:37'!
getChoice: aSymbol
	
	aSymbol == #acceptOnCR ifTrue: [^acceptOnCR ifNil: [true]].
	^false.
! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'nice 12/26/2009 10:03'!
improveText: someText forMorph: aMorph

	| betterText conversions fontForAll |

	fontForAll := aMorph eToyGetMainFont.
	betterText := someText veryDeepCopy.
	conversions := OrderedCollection new.
	betterText runs withStartStopAndValueDo: [:start :stop :attributes |
		attributes do: [:att |
			(att isMemberOf: TextFontChange) ifTrue: [
				conversions add: {att. start. stop}
			]
		]
	].
	conversions do: [ :old |
		| newAttr |
		betterText removeAttribute: old first from: old second to: old third.
		newAttr := TextFontReference toFont: (fontForAll fontAt: old first fontNumber).
		newAttr fontNumber: old first fontNumber.
		betterText addAttribute: newAttr from: old second to: old third.
	].
	^betterText! !

!EToyChatMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 19:36'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	acceptOnCR := true.
	self listDirection: #topToBottom;
		 layoutInset: 0;
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 rubberBandCells: false;
		 minWidth: 200;
		 minHeight: 200;
		 rebuild ! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 16:14'!
insetTheScrollbars

	self allMorphsDo: [ :each | 
		(each isKindOf: PluggableTextMorph) ifTrue: [each retractable: false]
	].! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/10/2000 06:09'!
ipAddress
	
	^(fields at: #ipAddress) contents! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 11:30'!
open
	
	^self openIn: self currentWorld! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/3/2000 07:40'!
openIn: aWorld

	"open an a chat window"

	aWorld ifNil: [^self].
	self 
		position: 400@100;
		extent:  200@150;
		openInWorld: aWorld.! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'KR 5/10/2008 15:08'!
rebuild
	| r1 r2 |

	r1 _ self addARow: {
		self simpleToggleButtonFor: self attribute: #acceptOnCR help: 'Send with Return?' translated.
		self inAColumn: {StringMorph new contents: 'Your message to:' translated; font: Preferences standardMenuFont; lock}.
		self textEntryFieldNamed: #ipAddress with: ''
					help: 'IP address for chat partner' translated.
	}.
	recipientForm ifNotNil: [
		r1 addMorphBack: recipientForm asMorph lock
	].
	sendingPane _ PluggableTextMorph
				on: self
				text: nil
				accept: #acceptTo:forMorph:.
	sendingPane hResizing: #spaceFill; vResizing: #spaceFill.
	sendingPane font: Preferences standardMenuFont.
	self
		addMorphBack: sendingPane.
	r2 _ self addARow: {self inAColumn: {StringMorph new contents: 'Replies' translated; font: Preferences standardMenuFont; lock}}.
	receivingPane _ PluggableTextMorph
				on: self
				text: nil
				accept: nil.
	receivingPane font: Preferences standardMenuFont.
	receivingPane hResizing: #spaceFill; vResizing: #spaceFill.
	self
		addMorphBack: receivingPane.
	receivingPane spaceFillWeight: 3.
	{r1. r2} do: [ :each |
		each
			vResizing: #shrinkWrap; minHeight: 18;
			color: Color veryLightGray.
	].
	sendingPane acceptOnCR: (acceptOnCR ifNil: [acceptOnCR _ true])! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 08:50'!
recipientForm: aForm

	recipientForm := aForm.
	recipientForm ifNotNil: [recipientForm := recipientForm scaledToSize: 20@20].! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 16:43'!
reportError: aString

	receivingPane appendTextEtoy: (aString asText addAttribute: TextColor red), String cr.! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/10/2000 05:39'!
setIPAddress: aString
	
	(fields at: #ipAddress) contents: aString! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 08:04'!
standardBorderColor

	^Color darkGray! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'ar 12/17/2001 02:18'!
startOfMessageFromMe

	myForm ifNil: [
		myForm := EToySenderMorph pictureForIPAddress: NetNameResolver localAddressString.
		myForm ifNotNil: [
			myForm := myForm scaledToSize: 20@20
		].
	].
	myForm ifNil: [
		^(Preferences defaultAuthorName asText allBold addAttribute: TextColor blue)
	].
	^(String value: 1) asText
		addAttribute: (TextAnchor new anchoredMorph: myForm);
		yourself

! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 10:41'!
toggleChoice: aSymbol
	
	aSymbol == #acceptOnCR ifTrue: [
		acceptOnCR := (acceptOnCR ifNil: [true]) not.
		sendingPane ifNotNil: [sendingPane acceptOnCR: acceptOnCR].
		^self
	].

! !

!EToyChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 11:58'!
transmittedObjectCategory

	^EToyIncomingMessage typeKeyboardChat! !

EToyChatMorph subclass: #EToyMultiChatMorph
	instanceVariableNames: 'targetIPAddresses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!EToyMultiChatMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 08:53'!
chatWindowForIP: ipAddress name: senderName picture: aForm inWorld: aWorld

	^self allInstances 
		detect: [ :x | x world == aWorld] 
		ifNone: [
			EToyCommunicatorMorph playArrivalSound.
			self new open
		].

! !

!EToyMultiChatMorph class methodsFor: 'parts bin' stamp: 'tak 7/25/2007 00:05'!
descriptionForPartsBin

	^ self partName: 	'Text chat+' translatedNoop
		categories:		#()
		documentation:	'A tool for sending messages to several Squeak users at once' translatedNoop
		sampleImageForm: (Form
	extent: 25@25
	depth: 16
	fromArray: #( 1177640695 1593270007 1593270007 1593270007 1593270007 1593270007 1593270007 1593270007 1593270007 1593270007 1593270007 1593270007 1593245696 1593263665 1593270007 1593270007 1593270007 1177634353 1177628012 1177628012 1177640695 1593270007 1593270007 1593278463 2147450879 1316159488 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593274233 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1731723264 1593257324 762064236 762064236 762064236 762064236 762057894 762057894 762064236 762064236 762064236 762064236 762064236 1177616384 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593278459 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 2147188731 1870200832 1593274233 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1870229369 1731723264)
	offset: 0@0)! !

!EToyMultiChatMorph methodsFor: 'layout' stamp: 'ar 10/5/2000 19:24'!
acceptDroppingMorph: morphToDrop event: evt

	(morphToDrop isKindOf: EToySenderMorph) ifFalse: [
		^morphToDrop rejectDropMorphEvent: evt.
	].
	self eToyRejectDropMorph: morphToDrop event: evt.		"we don't really want it"
	self updateIPAddressField: targetIPAddresses,{morphToDrop ipAddress}.

! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/5/2000 14:15'!
acceptTo: someText forMorph: aMorph

	| streamedMessage betterText |

	betterText := self improveText: someText forMorph: aMorph.
	streamedMessage := {targetIPAddresses. betterText} eToyStreamedRepresentationNotifying: self.
	targetIPAddresses do: [ :each |
		self 
			transmitStreamedObject: streamedMessage
			to: each.
	].
	aMorph setText: '' asText.
	self appendMessage: 
		self startOfMessageFromMe,
		' - ',
		betterText,
		String cr.

	^true! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 09:01'!
chatFrom: ipAddress name: senderName text: textPackage

	super chatFrom: ipAddress name: senderName text: textPackage second.
	self updateIPAddressField: (
		targetIPAddresses,textPackage first,{ipAddress} 
			copyWithout: NetNameResolver localAddressString
	).
! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:54'!
editEvent: anEvent for: aMorph

	| answer initialText aFillInTheBlankMorph |

	(aMorph bounds containsPoint: anEvent cursorPoint) ifFalse: [^self].
	initialText _ String streamContents: [ :strm |
		targetIPAddresses do: [ :each | strm nextPutAll: each; cr].
	].
	aFillInTheBlankMorph _ FillInTheBlankMorph new
		setQuery: 'Who are you chatting with?' translated
		initialAnswer: initialText
		answerHeight: 250
		acceptOnCR: false.
	aFillInTheBlankMorph responseUponCancel: nil.
	self world addMorph: aFillInTheBlankMorph centeredNear: anEvent cursorPoint.
	answer _ aFillInTheBlankMorph getUserResponse.
	answer ifNil: [^self].
	self updateIPAddressField: (answer findTokens: ' ',String cr).

! !

!EToyMultiChatMorph methodsFor: 'initialization' stamp: 'RAA 8/17/2000 08:57'!
initialize

	targetIPAddresses := OrderedCollection new.
	super initialize.
	bounds := 0@0 extent: 350@350.! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:54'!
rebuild
	| r1 r2 |

	r1 _ self addARow: {
		self simpleToggleButtonFor: self attribute: #acceptOnCR help: 'Send with Return?' translated.
		self inAColumn: {StringMorph new contents: 'Multi chat with:' translated; lock}.
		self textEntryFieldNamed: #ipAddress with: ''
					help: 'Click to edit participant list' translated.
	}.
	sendingPane _ PluggableTextMorph
				on: self
				text: nil
				accept: #acceptTo:forMorph:.
	sendingPane hResizing: #spaceFill; vResizing: #spaceFill.
	self
		addMorphBack: sendingPane.
	r2 _ self addARow: {self inAColumn: {StringMorph new contents: 'Replies' translated; lock}}.
	receivingPane _ PluggableTextMorph
				on: self
				text: nil
				accept: nil.
	receivingPane hResizing: #spaceFill; vResizing: #spaceFill.
	self
		addMorphBack: receivingPane.
	receivingPane spaceFillWeight: 3.
	{r1. r2} do: [ :each |
		each
			vResizing: #shrinkWrap; minHeight: 18;
			color: Color veryLightGray.
	].
	self updateIPAddressField: targetIPAddresses.
	sendingPane acceptOnCR: (acceptOnCR ifNil: [acceptOnCR _ true]).! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 08:53'!
standardBorderColor

	^Color veryLightGray! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 07:44'!
transmittedObjectCategory

	^EToyIncomingMessage typeMultiChat! !

!EToyMultiChatMorph methodsFor: 'as yet unclassified' stamp: 'nice 8/19/2010 14:29'!
updateIPAddressField: newAddresses
	
	targetIPAddresses := (
		newAddresses copyWithout: NetNameResolver localAddressString
	) asSet asArray sort.

	(fields at: #ipAddress) contents: targetIPAddresses size printString,' people'.! !

!EToyMultiChatMorph methodsFor: 'dropping/grabbing' stamp: 'RAA 8/17/2000 09:04'!
wantsDroppedMorph: aMorph event: evt

	(aMorph isKindOf: EToySenderMorph) ifFalse: [^false].
	(bounds containsPoint: evt cursorPoint) ifFalse: [^false].
	^true.! !

!EToyChatOrBadgeMorph class methodsFor: 'new-morph participation' stamp: 'RAA 8/3/2000 07:51'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ self ~~ EToyChatOrBadgeMorph! !

EToyChatOrBadgeMorph subclass: #EToySenderMorph
	instanceVariableNames: 'userPicture'
	classVariableNames: 'DEBUG'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!EToySenderMorph commentStamp: '<historical>' prior: 0!
EToySenderMorph
	new
	userName: 'Bob Arning' 
	userPicture: nil 
	userEmail: 'arning@charm.net' 
	userIPAddress: '1.2.3.4';
	position: 200@200;
	open
"
EToySenderMorph represents another person to whom you wish to send things. Drop a morph on an EToySenderMorph and a copy of that morph is sent to the person represented. Currently only peer-to-peer communications are supported, but other options are planned.
"!

!EToySenderMorph class methodsFor: 'parts bin' stamp: 'tak 7/25/2007 00:05'!
descriptionForPartsBin

	^ self partName: 	'Badge' translatedNoop
		categories:		#()
		documentation:	'A tool for collaborating with other Squeak users' translatedNoop
		sampleImageForm: (Form
	extent: 66@72
	depth: 16
	fromArray: #( 7175 1545042975 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082429975 470220800 470252575 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082413575 1545042975 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082429975 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 1582767304 1032871511 2134867775 2134842568 2134867775 2134867775 2134867775 1032879935 2134867775 2134867775 2134867775 2134867775 1582792511 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134842568 1032871511 1032863120 1582775696 1032871511 2134867775 2134867775 1032871511 2134842568 1032863120 482885008 1032879935 482901823 482885008 1032879935 1032863120 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134850960 2134850960 1032879935 1032863120 2134850960 2134867775 2134859351 482876616 2134850960 2134867775 1032879935 1032879935 1032879935 1032879935 1032879935 1032863120 1582792511 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 1582767304 1032871511 1032863120 1582775696 1032871511 2134867775 2134842568 1582767304 1582767304 1582792511 482893399 482893399 482893399 482893399 482893399 1032863120 1582792511 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 1032863120 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 65537 65537 65537 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 65537 1032863120 1032863120 98111 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 81296 2134867775 2134867775 1032847361 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134850960 1032879935 2134867775 2134867775 2134835201 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 65537 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 81296 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 65537 98111 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 65537 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 65537 81296 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 65537 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134850960 81296 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134850960 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134850960 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134835201 98111 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134850960 1032879935 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 2147450879 2147450879 1039171583 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 1039171583 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141142512 1039154672 1593270007 1039163127 1593278463 1593261552 2147442423 1039154672 2147433968 1039154672 1593270007 1039163127 1593278463 1593261552 2147442423 1593270007 1593261552 2147450879 2147442423 1593270007 2147442423 1039171583 484990711 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141150967 1039163127 1039171583 1039163127 1039171583 1039171583 1039154672 1039154672 1039163127 1039163127 1039171583 1039163127 1039171583 484982256 1039146216 1593270007 484982256 1039171583 2147425512 1593261552 2147425512 1039154672 1039171583 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 1039154672 1593278463 1039171583 1039171583 1039171583 1039154672 1039146216 1593278463 1039154672 1593278463 1039171583 1039171583 1039171583 1593261552 2147450879 1039171583 1593270007 2147433968 2147433968 2147433968 2147442423 1039171583 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 1593270007 2147450879 1039163127 1039163127 1593261552 2147442423 1039163127 2147450879 1593270007 2147450879 1039163127 1039163127 1593261552 2147433968 1593278463 1593261552 2147442423 2147433968 1593261552 1593270007 1039171583 2147442423 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2141159231 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450783 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141150967 1039171583 1593261552 2147442423 1039171583 2147450879 2147442423 2147450879 1593278463 1593261552 2147450879 2147442423 1039171583 2147442423 2147450879 2147442423 1039171583 2147442423 2147450783 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141150967 2147433968 1039171583 1039154672 2147433968 2147450879 1593261552 2147442423 1039171583 1593278463 1039171583 2147433968 2147433968 1593261552 2147450879 2147442423 2147433968 1593261552 2147450783 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 1039171583 1039171583 1039154672 1039154672 2147450879 2147433968 2147425512 484990711 2147433968 1593278463 2147433968 1039154672 2147433968 2147450879 2147450879 1039163127 484973800 1593278367 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141142512 2147442423 1039171583 1039171583 1593270007 1593278463 2147433968 2147450879 1039171583 2147450879 1039163127 2147450879 1593270007 2147433968 2147442423 2147450879 2147433968 2147433968 2147450783 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141142512 1039163127 1593261552 2147442423 1593278463 1593278463 1593261552 2147450879 1039163127 1593261552 2147442423 2147442423 1593278463 1593261552 2147442423 2147442423 1039171583 2147433968 1593278367 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2141159423 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450783 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134861595 1391951679 2134867775 2134867775 2134856439 1729855295 2134867775 2134867775 1729849115 2134867775 2134867775 2134861595 1729855295 2134867775 2134867775 1729843959 1391951679 2134867775 2134867775 2134856439 1729855295 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 1326930843 1879001943 1729855295 2134861595 1398112251 1738035990 2134867775 2134855446 1536646039 1326874431 2134867775 1387357800 1718112945 2134867775 2134856463 1736736736 2145407816 1729855295 2134861595 1398243327 1738232599 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 1032863120 1032879935 2134867775 1032863120 1032879935 2134856439 1879011327 1879011327 1264025407 2134856533 2147188731 2147188731 1391951679 1391947770 1878683642 1878676215 2134856439 2120646246 2120646246 1391951679 1391951840 2145419232 2145419232 1397260095 2134856535 2147450879 2147450879 1391951679 2134867775 2082438175 2082438175 2134867775 2134842568 1507359 1514696 2134842568 48235488 48241864 2134854487 1391932912 904949759 1879003895 1391951867 484908263 1039040507 1398112063 1263890426 904753146 1878674261 2134856331 2120629539 1025736294 1384873791 1397260256 2145402336 2145419232 2145407735 1391951871 1039154672 1039171583 1398243135 2134867775 2082438175 2082438175 2134867775 2134835216 2031647 2031632 2134835680 65012704 65012192 2134854487 904949759 1879011327 1879003895 1391951867 2147171822 2147188731 1398112063 1263890426 904753146 1878674261 2134856331 2120646246 1025736294 1384873791 1397260256 1591754208 2145419232 2145407735 1391951871 1039163127 2147450879 1398243135 2134867775 2082438175 2082438175 2134867775 2134835216 2031647 2031632 2134835680 65012704 65012192 2134854487 904949759 1879011327 1879003895 1391951867 2147171822 2147188731 1398112063 1263890426 904753146 1878674261 2134856331 2120629539 2120646246 1384873791 1397260256 484449504 1591771104 2145407735 1391951871 2147442423 1593278463 1398243135 2134867775 2082438175 2082438175 2134867775 2134842568 1507359 1514696 2134842568 48235488 48241864 2134854487 1391932912 904949759 1879003895 1391951867 1593056487 2147188731 1398112063 1263890426 1391685626 1878674261 2134856331 2120637892 2120646246 1384873791 1397251808 484466400 484474848 2145407735 1391951871 484982256 1593278463 1398243135 2134867775 2082438175 2082438175 2134867775 2134867775 1032863120 1032879935 2134867775 1032863120 1032879935 2134855447 1879011327 1879011327 1536911131 1729849240 2147188731 2147188731 1393983295 1326870522 1878683642 1878675222 2134856369 2120646246 2120646246 1387364159 1393524704 2145419232 2145419232 1736730395 1729849243 2147450879 2147450879 1394048831 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134861595 1536913407 1879011327 1326939967 2134856470 2147188731 2147182488 1729855295 1729846167 1878683642 1536648987 2134861595 1718124134 2120640104 1729855295 1729849220 2145419232 2145419232 1393524543 2134856471 2147450879 2147444635 1729855295 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 1729842967 1264014071 2134867775 2134867775 1391940437 1393977115 2134867775 2134861595 1326862102 1729855295 2134867775 1729843889 1387357979 2134867775 2134861595 1393513288 1397248759 2134867775 2134867775 1391940439 1394042651 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 2082438175 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082438175 1545042975 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2134867775 2082429975 470252575 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082413575 7175 1545042975 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082438175 2082429975 470220800)
	offset: 0@0)! !

!EToySenderMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/16/2000 12:52'!
instanceForIP: ipAddress

	^self allInstances detect: [ :x | 
		x ipAddress = ipAddress
	] ifNone: [nil]
! !

!EToySenderMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/16/2000 12:50'!
instanceForIP: ipAddress inWorld: aWorld

	^self allInstances detect: [ :x | 
		x world == aWorld and: [x ipAddress = ipAddress]
	] ifNone: [nil]
! !

!EToySenderMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 15:42'!
nameForIPAddress: ipString

	| senderMorphs |

	senderMorphs := EToySenderMorph allInstances select: [ :x | 
		x userName notNil and: [x ipAddress = ipString]
	].
	senderMorphs isEmpty ifTrue: [^nil].
	^senderMorphs first userName

! !

!EToySenderMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 14:47'!
pictureForIPAddress: ipString

	| senderMorphs |

	senderMorphs := EToySenderMorph allInstances select: [ :x | 
		x userPicture notNil and: [x ipAddress = ipString]
	].
	senderMorphs isEmpty ifTrue: [^nil].
	^senderMorphs first userPicture

! !

!EToySenderMorph methodsFor: 'dropping/grabbing' stamp: 'ar 10/5/2000 19:58'!
aboutToBeGrabbedBy: aHand

	| aFridge |
	super aboutToBeGrabbedBy: aHand.
	aFridge := self ownerThatIsA: EToyFridgeMorph.
	aFridge ifNil: [^self].
	aFridge noteRemovalOf: self.! !

!EToySenderMorph methodsFor: 'layout' stamp: 'RAA 3/7/2001 22:31'!
acceptDroppingMorph: morphToDrop event: evt

	| myCopy outData |

	(morphToDrop isKindOf: NewHandleMorph) ifTrue: [			"don't send these"
		^morphToDrop rejectDropMorphEvent: evt.
	].
	self eToyRejectDropMorph: morphToDrop event: evt.		"we don't really want it"

	"7 mar 2001 - remove #veryDeepCopy"
	myCopy := morphToDrop.	"gradient fills require doing this second"
	myCopy setProperty: #positionInOriginatingWorld toValue: morphToDrop position.
	self stopFlashing.

	outData := myCopy eToyStreamedRepresentationNotifying: self.
	self resetIndicator: #working.
	self transmitStreamedObject: outData to: self ipAddress.

! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:56'!
checkOnAFriend

	| gateKeeperEntry caption choices resp |

	gateKeeperEntry _ EToyGateKeeperMorph entryForIPAddress: self ipAddress.
	caption _ 
'Last name: ' translated ,gateKeeperEntry latestUserName,
'\Last message in: ' translated ,gateKeeperEntry lastIncomingMessageTimeString,
'\Last status check at: ' translated ,gateKeeperEntry lastTimeCheckedString,
'\Last status in: ' translated ,gateKeeperEntry statusReplyReceivedString.
	choices _ 'Get his status now\Send my status now' translated.
	resp _ (PopUpMenu labels: choices withCRs) startUpWithCaption: caption withCRs.
	resp = 1 ifTrue: [
		gateKeeperEntry lastTimeChecked: Time totalSeconds.
		self sendStatusCheck.
	].
	resp = 2 ifTrue: [
		self sendStatusReply.
	].
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/10/2000 12:25'!
currentBadgeVersion

	"enables on-the-fly updating of older morphs"
	^10! !

!EToySenderMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:34'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ Color magenta! !

!EToySenderMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:37'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 4! !

!EToySenderMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:26'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color lightMagenta! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/17/2000 12:30'!
establishDropZone: aMorph

	self setProperty: #specialDropZone toValue: aMorph.
	aMorph 
		on: #mouseEnterDragging send: #mouseEnteredDZ to: self;
		on: #mouseLeaveDragging send: #mouseLeftDZ to: self;
		on: #mouseLeave send: #mouseLeftDZ to: self.
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'nk 6/12/2004 09:23'!
fixOldVersion

	| uName uForm uEmail uIP |
	uName := self userName.
	uForm := userPicture ifNil: [
		(self 
		findDeepSubmorphThat: [ :x | (x isKindOf: ImageMorph) or: [x isSketchMorph]] 
		ifAbsent: [self halt]) form.
	].
	uEmail := (fields at: #emailAddress) contents.
	uIP := self ipAddress.
	self
		userName: uName 
		userPicture: (uForm scaledToSize: 61@53)
		userEmail: uEmail 
		userIPAddress: uIP
! !

!EToySenderMorph methodsFor: 'initialization' stamp: 'dgd 2/17/2003 19:58'!
initialize
	"initialize the state of the receiver"
	Socket initializeNetwork.
	"we may want our IP address"
	Preferences defaultAuthorName.
	"seems like a good place to insure we have a name"
	super initialize.
	""
	self listDirection: #topToBottom;
		 layoutInset: 4;
		 setProperty: #normalBorderColor toValue: self borderColor;
		 setProperty: #flashingColors toValue: {Color red. Color yellow}! !

!EToySenderMorph methodsFor: 'parts bin' stamp: 'RAA 8/20/2001 13:08'!
initializeToStandAlone

	super initializeToStandAlone.
	self installModelIn: ActiveWorld.
! !

!EToySenderMorph methodsFor: 'debug and other' stamp: 'RAA 8/31/2000 18:31'!
installModelIn: myWorld

	"if we get this far and nothing exists, make it up"

	userPicture ifNotNil: [^self].
	self
		userName: Preferences defaultAuthorName 
		userPicture: nil 
		userEmail: 'who@where.net' 
		userIPAddress: NetNameResolver localAddressString
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 15:58'!
ipAddress

	^(fields at: #ipAddress) contents! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/17/2000 16:20'!
ipAddress: aString

	^(fields at: #ipAddress) contents: aString! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'ar 11/10/2000 14:23'!
killExistingChat

	| oldOne |
	self rubberBandCells: true. "disable growing"
	(oldOne := self valueOfProperty: #embeddedChatHolder) ifNotNil: [
		oldOne delete.
		self removeProperty: #embeddedChatHolder
	].

	(oldOne := self valueOfProperty: #embeddedAudioChatHolder) ifNotNil: [
		oldOne delete.
		self removeProperty: #embeddedAudioChatHolder
	].

! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 18:05'!
mouseEnteredDZ

	| dz |
	dz := self valueOfProperty: #specialDropZone ifAbsent: [^self].
	dz color: Color blue.! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 18:05'!
mouseLeftDZ

	| dz |
	dz := self valueOfProperty: #specialDropZone ifAbsent: [^self].
	dz color: Color transparent.! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 11:52'!
sendStatusCheck

	| null |
	null := String with: 0 asCharacter.
	EToyPeerToPeer new 
		sendSomeData: {
			EToyIncomingMessage typeStatusRequest,null. 
			Preferences defaultAuthorName,null.
		}
		to: self ipAddress
		for: self.
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 11:53'!
sendStatusReply

	| null |
	null := String with: 0 asCharacter.
	EToyPeerToPeer new 
		sendSomeData: {
			EToyIncomingMessage typeStatusReply,null. 
			Preferences defaultAuthorName,null.
			((EToyGateKeeperMorph acceptableTypesFor: self ipAddress) 
				eToyStreamedRepresentationNotifying: self).
		}
		to: self ipAddress
		for: self.
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 14:22'!
startAudioChat

	self startAudioChat: true
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'aoy 2/15/2003 20:59'!
startAudioChat: toggleMode 
	| chat r |
	(self valueOfProperty: #embeddedAudioChatHolder) ifNotNil: 
			[toggleMode ifFalse: [^self].
			^self killExistingChat].
	chat := AudioChatGUI new ipAddress: self ipAddress.
	(self ownerThatIsA: EToyFridgeMorph) isNil 
		ifTrue: 
			[chat
				removeConnectButton;
				vResizing: #shrinkWrap;
				hResizing: #shrinkWrap;
				borderWidth: 2.	"we already know the connectee"
			r := (self addARow: { 
								chat}) vResizing: #shrinkWrap.
			self world startSteppingSubmorphsOf: chat.
			self setProperty: #embeddedAudioChatHolder toValue: r.
			self
				hResizing: #shrinkWrap;
				vResizing: #shrinkWrap]
		ifFalse: 
			[chat openInWorld: self world]! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 17:14'!
startChat

	self startChat: true
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'ar 11/10/2000 15:10'!
startChat: toggleMode

	| chat r |

	(self valueOfProperty: #embeddedChatHolder) ifNotNil: [
		toggleMode ifFalse: [^self].
		^self killExistingChat
	].
	(EToyChatMorph doChatsInternalToBadge and: 
				[(self ownerThatIsA: EToyFridgeMorph) isNil]) ifTrue: [
		chat := EToyChatMorph basicNew
			recipientForm: userPicture; 
			initialize;
			setIPAddress: self ipAddress.
		chat
			vResizing: #spaceFill;
			hResizing: #spaceFill;
			borderWidth: 2;
			insetTheScrollbars.
		r := (self addARow: {chat}) vResizing: #spaceFill.
		self rubberBandCells: false. "enable growing"
		self height: 350. "an estimated guess for allowing shrinking as well as growing"
		self world startSteppingSubmorphsOf: chat.
		self setProperty: #embeddedChatHolder toValue: r.
	] ifFalse: [
		chat := EToyChatMorph 
			chatWindowForIP: self ipAddress
			name: self userName 
			picture: userPicture 
			inWorld: self world.
		chat owner addMorphFront: chat.
	]
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'bf 7/3/2008 20:36'!
startNebraskaClient

	| newMorph |
	[
		[
			newMorph _ NetworkTerminalMorph connectTo: (self ipAddress copyUpTo: $:). "FIXME: get real port of Nebraska Server"
			WorldState addDeferredUIMessage: [newMorph openInStyle: #scaled] fixTemps.
		]
			on: Error
			do: [ :ex |
				WorldState addDeferredUIMessage: [
					self inform: 'No connection to: ' translated. self ipAddress,' (',ex printString,')'
				] fixTemps
			].
	] fork
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'ar 10/24/2000 14:04'!
startTelemorphic

	self world 
		connectRemoteUserWithName: self userName 
		picture: (userPicture ifNotNil: [userPicture scaledToSize: 16@20]) 
		andIPAddress: self ipAddress
! !

!EToySenderMorph methodsFor: 'stepping and presenter' stamp: 'RAA 8/10/2000 12:24'!
step

	(self valueOfProperty: #currentBadgeVersion) = self currentBadgeVersion ifFalse: [
		self setProperty: #currentBadgeVersion toValue: self currentBadgeVersion.
		self fixOldVersion.
		Preferences defaultAuthorName.		"seems like a good place to insure we have a name"
	].
	super step.! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/13/2000 11:12'!
tellAFriend

	self world project tellAFriend: (fields at: #emailAddress) contents
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 16:15'!
transmitStreamedObject: outData

	self transmitStreamedObject: outData to: self ipAddress

! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 12:00'!
transmittedObjectCategory

	^EToyIncomingMessage typeMorph! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/20/2001 13:03'!
userName

	^ (self 
		findDeepSubmorphThat: [ :x | x isKindOf: StringMorph] 
		ifAbsent: [^nil]) contents
! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'yo 9/15/2008 10:14'!
userName: aString userPicture: aFormOrNil userEmail: emailString userIPAddress: ipString

	| dropZoneRow |

	self setProperty: #currentBadgeVersion toValue: self currentBadgeVersion.
	userPicture _ aFormOrNil ifNil: [
		(TextStyle default fontOfSize: 26) emphasized: 1; characterFormAt: $?
	].
	userPicture _ userPicture scaledToSize: 61@53.
	self killExistingChat.
	self removeAllMorphs.
	self useRoundedCorners.
	self 
		addARow: {
			self inAColumn: {(StringMorph contents: aString) lock}
		}.
	dropZoneRow _ self
		addARow: {
			self inAColumn: {userPicture asMorph lock}
		}.
	self establishDropZone: dropZoneRow.
	self
		addARow: {
			self textEntryFieldNamed: #emailAddress with: emailString
					help: 'Email address for this person'
		};
		addARow: {
			self textEntryFieldNamed: #ipAddress with: ipString
					help: 'IP address for this person'
		};
		addARow: {
			self indicatorFieldNamed: #working color: Color blue help: 'working' translated.
			self indicatorFieldNamed: #communicating color: Color green help: 'sending' translated.
			self buttonNamed: 'C' action: #startChat color: Color paleBlue 
								help: 'Open a written chat with this person' translated.
			self buttonNamed: 'T' action: #startTelemorphic color: Color paleYellow 
								help: 'Start telemorphic with this person' translated.
			self buttonNamed: '!!' action: #tellAFriend color: Color paleGreen 
								help: 'Tell this person about the current project' translated.
			self buttonNamed: '?' action: #checkOnAFriend color: Color lightBrown 
								help: 'See if this person is available' translated.
			"self buttonNamed: 'A' action: #startAudioChat color: Color yellow 
								help: 'Open an audio chat with this person' translated."
			self buttonNamed: 'S' action: #startNebraskaClient color: Color white 
								help: 'See this person''s world (if he allows that)' translated.
		}.
	! !

!EToySenderMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 16:06'!
userPicture

	^userPicture! !

!EToySenderMorph methodsFor: 'dropping/grabbing' stamp: 'RAA 7/14/2000 18:02'!
wantsDroppedMorph: aMorph event: evt

	| dz |
	dz := self valueOfProperty: #specialDropZone ifAbsent: [^false].
	(dz bounds containsPoint: (evt cursorPoint)) ifFalse: [^false].
	^true.! !

!EToyCommunicatorMorph methodsFor: '*nebraska-*nebraska-Morphic-Collaborative' stamp: 'nice 12/27/2009 03:10'!
addGateKeeperMorphs

	| list currentTime choices |

	self setProperty: #gateKeeperCounterValue toValue: EToyGateKeeperMorph updateCounter.
	choices := #(
		(60 'm' 'in the last minute')
		(3600 'h' 'in the last hour')
		(86400 'd' 'in the last day')
	).
	currentTime := Time totalSeconds.
	list := EToyGateKeeperMorph knownIPAddresses.
	list do: [ :each | | age row |
		age := each timeBetweenLastAccessAnd: currentTime.
		age := choices
			detect: [ :x | age <= x first]
			ifNone: [{0. '-'. (age // 86400) printString,'days ago'}].
		row := self addARow:
		(EToyIncomingMessage allTypes collect: [ :type |
				self toggleButtonFor: each attribute: type]
		),
		{

			(self inAColumn: {
				(StringMorph contents: age second) lock.
			}) layoutInset: 2; hResizing: #shrinkWrap; setBalloonText: 'Last attempt was ',age third.

			(self inAColumn: {
				(StringMorph contents: each ipAddress) lock.
			}) layoutInset: 2; hResizing: #shrinkWrap.

			(self inAColumn: {
				(StringMorph contents: each latestUserName) lock.
			}) layoutInset: 2.
		}.
		row
			color: (Color r: 0.6 g: 0.8 b: 1.0);
			borderWidth: 1;
			borderColor: #raised;
			vResizing: #spaceFill;
			"on: #mouseUp send: #mouseUp:in: to: self;"
			setBalloonText: each fullInfoString
	].! !

!EToyCommunicatorMorph methodsFor: '*nebraska-*nebraska-Morphic-Collaborative' stamp: 'mir 10/12/2000 14:55'!
transmitStreamedObject: outData as: objectCategory to: anIPAddress

	EToyPeerToPeer transmitStreamedObject: outData as: objectCategory to: anIPAddress for: self! !

!EToyCommunicatorMorph methodsFor: '*nebraska-*nebraska-Morphic-Collaborative' stamp: 'mir 10/10/2000 12:47'!
transmitStreamedObject: outData to: anIPAddress

	self transmitStreamedObject: outData as: self transmittedObjectCategory to: anIPAddress
! !

EToyCommunicatorMorph subclass: #EToyFridgeMorph
	instanceVariableNames: 'recipients incomingRow recipientRow updateCounter groupMode'
	classVariableNames: 'FridgeRecipients NewItems TheFridgeForm UpdateCounter'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!EToyFridgeMorph commentStamp: '<historical>' prior: 0!
EToyFridgeMorph new openInWorld!

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/13/2000 12:19'!
addRecipient: aSenderMorph

	self fridgeRecipients do: [ :each |
		aSenderMorph ipAddress = each ipAddress ifTrue: [^self]
	].
	self fridgeRecipients add: aSenderMorph.
	UpdateCounter := self updateCounter + 1
! !

!EToyFridgeMorph class methodsFor: 'parts bin' stamp: 'sw 7/29/2007 01:58'!
descriptionForPartsBin

	^ self partName: 	'Fridge' translatedNoop
		categories:		#()
		documentation:	'A tool for sending objects to other Squeak users' translatedNoop! !

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 12:09'!
fridgeForm

	| fridgeFileName |

	fridgeFileName := 'fridge.form'.
	TheFridgeForm ifNotNil: [^TheFridgeForm].
	(FileDirectory default fileExists: fridgeFileName) ifFalse: [^nil].
	^TheFridgeForm := Form fromFileNamed: fridgeFileName.! !

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 19:23'!
fridgeRecipients

	^FridgeRecipients ifNil: [FridgeRecipients := OrderedCollection new]! !

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 19:19'!
newItems

	^NewItems ifNil: [NewItems := OrderedCollection new]! !

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/16/2000 13:55'!
newItem: newMorph

	| theFridge fridgeWorld trialRect |

	theFridge := Project named: 'Fridge'.
	theFridge ifNil: [^self newItems add: newMorph].
	fridgeWorld := theFridge world.
	trialRect := fridgeWorld randomBoundsFor: newMorph.
	fridgeWorld 
		addMorphFront: (newMorph position: trialRect topLeft);
		startSteppingSubmorphsOf: newMorph
! !

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 11:04'!
removeRecipientWithIPAddress: ipString

	FridgeRecipients := self fridgeRecipients reject: [ :each |
		ipString = each ipAddress
	].
	UpdateCounter := self updateCounter + 1
! !

!EToyFridgeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/12/2000 19:22'!
updateCounter

	^UpdateCounter ifNil: [0]! !

!EToyFridgeMorph methodsFor: 'layout' stamp: 'RAA 3/7/2001 22:31'!
acceptDroppingMorph: morphToDrop event: evt

	| outData |

	(morphToDrop isKindOf: NewHandleMorph) ifTrue: [		"don't send these"
		^morphToDrop rejectDropMorphEvent: evt
	].
	self eToyRejectDropMorph: morphToDrop event: evt.		"we will keep a copy"
	(morphToDrop isKindOf: EToySenderMorph) ifTrue: [
		self class addRecipient: morphToDrop.
		^self rebuild
	].
	self stopFlashing.
	"7 mar 2001 - remove #veryDeepCopy"
	outData := morphToDrop eToyStreamedRepresentationNotifying: self.
	self resetIndicator: #working.
	self class fridgeRecipients do: [ :each |
		self transmitStreamedObject: outData to: each ipAddress
	].

! !

!EToyFridgeMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:34'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ #raised! !

!EToyFridgeMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:37'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 4! !

!EToyFridgeMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:25'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color paleRed! !

!EToyFridgeMorph methodsFor: 'drawing' stamp: 'RAA 7/14/2000 12:07'!
drawOn: aCanvas

	| f cache |
	f := self class fridgeForm ifNil: [^super drawOn: aCanvas].
	cache := Form extent: bounds extent depth: aCanvas depth.
	f
		displayInterpolatedIn: cache boundingBox truncated
		on: cache.
	cache replaceColor: Color black withColor: Color transparent.
	aCanvas 
		translucentImage: cache
		at: bounds origin.
! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/13/2000 12:47'!
getChoice: aString

	aString = 'group' ifTrue: [^groupMode ifNil: [true]].! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:48'!
groupToggleButton

	^(self inAColumn: {
		(EtoyUpdatingThreePhaseButtonMorph checkBox)
			target: self;
			actionSelector: #toggleChoice:;
			arguments: {'group'};
			getSelector: #getChoice:;
			setBalloonText: 'Changes between group mode and individuals' translated;
			step
	}) hResizing: #shrinkWrap
! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/15/2000 09:21'!
handlesMouseDown: globalEvt

	| localCursorPoint |
	localCursorPoint := self globalPointToLocal: globalEvt cursorPoint.
	groupMode ifFalse: [
		self allMorphsDo: [ :each |
			(each isKindOf: EToySenderMorph) ifTrue: [
				(each bounds containsPoint: localCursorPoint) ifTrue: [^false].
			].
		].
	].
	^true! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:51'!
handlesMouseOverDragging: globalEvt

	^true! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:47'!
handlesMouseOver: globalEvt

	^true! !

!EToyFridgeMorph methodsFor: 'initialization' stamp: 'dgd 2/17/2003 19:58'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	groupMode := true.
	self listDirection: #topToBottom;
		 layoutInset: 10;
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 setProperty: #normalBorderColor toValue: self borderColor;
		 setProperty: #flashingColors toValue: {Color red. Color yellow};
		 rebuild! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:55'!
mouseDown: localEvt

	self addMouseActionIndicatorsWidth: 15 color: (Color blue alpha: 0.7).
! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:51'!
mouseEnterDragging: evt

	^self mouseEnterEither: evt
! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/17/2000 09:51'!
mouseEnterEither: evt

	evt hand hasSubmorphs ifFalse: [
		^self addMouseActionIndicatorsWidth: 10 color: (Color blue alpha: 0.3).
	].
	(evt hand firstSubmorph isKindOf: EToySenderMorph) ifTrue: [
		^self addMouseActionIndicatorsWidth: 10 color: (Color magenta alpha: 0.3).
	].
	self addMouseActionIndicatorsWidth: 10 color: (Color green alpha: 0.3).

! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:51'!
mouseEnter: evt

	^self mouseEnterEither: evt
! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:54'!
mouseLeaveDragging: evt

	^self mouseLeaveEither: evt
! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/17/2000 09:53'!
mouseLeaveEither: evt

	self deleteAnyMouseActionIndicators.

! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:53'!
mouseLeave: evt

	^self mouseLeaveEither: evt
! !

!EToyFridgeMorph methodsFor: 'event handling' stamp: 'RAA 7/17/2000 09:55'!
mouseUp: localEvt

	(self containsPoint: localEvt cursorPoint) ifFalse: [^self].
	Project enterIfThereOrFind: 'Fridge'! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/14/2000 11:05'!
noteRemovalOf: aSenderMorph

	self class removeRecipientWithIPAddress: aSenderMorph ipAddress! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:48'!
rebuild

	| row filler fudge people maxPerRow insetY |

	updateCounter _ self class updateCounter.
	self removeAllMorphs.
	(self addARow: {
		filler _ Morph new color: Color transparent; extent: 4@4.
	}) vResizing: #shrinkWrap.
	self addARow: {
		(StringMorph contents: 'the Fridge' translated) lock.
		self groupToggleButton.
	}.
	row _ self addARow: {}.
	people _ self class fridgeRecipients.
	maxPerRow _ people size < 7 ifTrue: [2] ifFalse: [3].	
		"how big can this get before we need a different approach?"
	people do: [ :each |
		row submorphCount >= maxPerRow ifTrue: [row _ self addARow: {}].
		row addMorphBack: (
			groupMode ifTrue: [
				(each userPicture scaledToSize: 35@35) asMorph lock
			] ifFalse: [
				each veryDeepCopy killExistingChat
			]
		)
	].
	fullBounds _ nil.
	self fullBounds.
	"htsBefore _ submorphs collect: [ :each | each height]."

	fudge _ 20.
	insetY _ self layoutInset.
	insetY isPoint ifTrue: [insetY _ insetY y].
	filler extent: 
		4 @ (self height - filler height * 0.37 - insetY - borderWidth - fudge) truncated.

	"self fixLayout.
	htsAfter _ submorphs collect: [ :each | each height].
	{htsBefore. htsAfter} explore."

! !

!EToyFridgeMorph methodsFor: 'stepping and presenter' stamp: 'RAA 7/13/2000 12:31'!
step

	super step.
	updateCounter = self class updateCounter ifFalse: [self rebuild].
! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/13/2000 12:50'!
toggleChoice: aString

	updateCounter := nil.		"force rebuild"
	aString = 'group' ifTrue: [^groupMode := (groupMode ifNil: [true]) not].
! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 11:50'!
transmittedObjectCategory

	^EToyIncomingMessage typeFridge! !

!EToyFridgeMorph methodsFor: 'as yet unclassified' stamp: 'nb 6/17/2003 12:25'!
trulyFlashIndicator: aSymbol

	| state |

	state := (self 
		valueOfProperty: #fridgeFlashingState
		ifAbsent: [false]) not.
	self setProperty: #fridgeFlashingState toValue: state.

	self 
		addMouseActionIndicatorsWidth: 15 
		color: (Color green alpha: (state ifTrue: [0.3] ifFalse: [0.7])). Beeper beep.
	"self world displayWorldSafely."! !

!EToyFridgeMorph methodsFor: 'dropping/grabbing' stamp: 'RAA 7/12/2000 15:53'!
wantsDroppedMorph: aMorph event: evt

	^true! !

EToyCommunicatorMorph subclass: #EToyGateKeeperMorph
	instanceVariableNames: 'counter'
	classVariableNames: 'KnownIPAddresses UpdateCounter'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Experimental'!

!EToyGateKeeperMorph commentStamp: '<historical>' prior: 0!
EToyGateKeeperMorph new open

"
I am used to control the types of connections a user is willing to allow.
"!

!EToyGateKeeperMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 12:43'!
acceptableTypesFor: ipAddressString

	^(self knownIPAddresses at: ipAddressString ifAbsent: [^#()]) acceptableTypes! !

!EToyGateKeeperMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 8/8/2000 11:06'!
acceptRequest: requestType from: senderName at: ipAddressString

	| entry |

	UpdateCounter := self updateCounter + 1.
	entry := self entryForIPAddress: ipAddressString.
	senderName isEmpty ifFalse: [entry latestUserName: senderName].
	^entry requestAccessOfType: requestType! !

!EToyGateKeeperMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 2/21/2001 11:57'!
entryForIPAddress: ipAddressString

	| known entry |

	UpdateCounter := self updateCounter + 1.
	known := self knownIPAddresses.
	entry := known at: ipAddressString ifAbsentPut: [
		entry := EToyGateKeeperEntry new.
		entry ipAddress: ipAddressString.
		entry
	].
	^entry! !

!EToyGateKeeperMorph class methodsFor: 'new-morph participation' stamp: 'RAA 8/3/2000 07:48'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ false! !

!EToyGateKeeperMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 08:42'!
knownIPAddresses

	^KnownIPAddresses ifNil: [KnownIPAddresses := Dictionary new]! !

!EToyGateKeeperMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 09:28'!
updateCounter

	^UpdateCounter ifNil: [UpdateCounter := 0]! !

!EToyGateKeeperMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:34'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ #raised! !

!EToyGateKeeperMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:37'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 4! !

!EToyGateKeeperMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:25'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color lightGray! !

!EToyGateKeeperMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 19:42'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	self listDirection: #topToBottom;
		 layoutInset: 4;
		 hResizing: #spaceFill;
		 vResizing: #spaceFill;
		 useRoundedCorners;
		 rebuild ! !

!EToyGateKeeperMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 09:29'!
open

	self rebuild.
	self openInWorld.! !

!EToyGateKeeperMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/16/2000 13:27'!
rebuild

	self removeAllMorphs.
	self addGateKeeperMorphs.
! !

!EToyGateKeeperMorph methodsFor: 'stepping and presenter' stamp: 'RAA 7/16/2000 13:28'!
step

	(self valueOfProperty: #gateKeeperCounterValue) = 
			EToyGateKeeperMorph updateCounter ifTrue: [^self].
	self rebuild.
! !

EToyCommunicatorMorph subclass: #EToyListenerMorph
	instanceVariableNames: 'listener updateCounter'
	classVariableNames: 'GlobalIncomingQueue GlobalListener QueueSemaphore UpdateCounter WasListeningAtShutdown'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!EToyListenerMorph commentStamp: '<historical>' prior: 0!
EToyListenerMorph new open
EToyListenerMorph startListening.
EToyListenerMorph stopListening.

"
EToyListenerMorph listens for messgaes from other EToy communicators. You need one of these open to receive messages from elsewhere.
- Received Morphs are shown in a list. Items can be grabbed (a copy) or deleted.
- Chat messages are sent to an appropriate EToyChatMorph (created if necessary)
"

!

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 18:24'!
addToGlobalIncomingQueue: aMorphTuple

	self critical: [
		self globalIncomingQueue add: aMorphTuple.
		self bumpUpdateCounter.
	].! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 18:23'!
bumpUpdateCounter

	UpdateCounter := (UpdateCounter ifNil: [0]) + 1.
! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'mir 6/17/2007 20:05'!
commResultDeferred: anArrayOfAssociations

	| m ipAddress aDictionary |

	"to be run as part of the UI process in case user interaction is required"

	aDictionary _ Dictionary new.
	anArrayOfAssociations do: [ :each | aDictionary add: each].
	
	aDictionary at: #commFlash ifPresent: [ :ignore | ^self].
	m _ aDictionary at: #message ifAbsent: [^self].
	m = 'OK' ifFalse: [^self].
	ipAddress _ aDictionary at: #ipAddress.

	EToyIncomingMessage new 
		incomingMessgage: (ReadStream on: (aDictionary at: #data)) 
		fromIPAddress: ipAddress

	! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 11/6/2000 11:48'!
commResult: anArrayOfAssociations

	WorldState addDeferredUIMessage: [self commResultDeferred: anArrayOfAssociations].! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:49'!
confirmListening

	self isListening ifFalse: [
		(self confirm: 'You currently are not listening and will not hear a reply.
Shall I start listening for you?' translated) ifTrue: [
			self startListening
		].
	].
! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 14:11'!
critical: aBlock

	QueueSemaphore ifNil: [QueueSemaphore := Semaphore forMutualExclusion].
	^QueueSemaphore critical: aBlock
! !

!EToyListenerMorph class methodsFor: 'parts bin' stamp: 'bf 7/26/2012 09:46'!
descriptionForPartsBin
	^ self partName: 	'Listener' translatedNoop
		categories:		#()
		documentation:	'A tool for receiving things from other Squeak users' translatedNoop! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 14:04'!
ensureListenerInCurrentWorld

	| w |
	w := self currentWorld.
	EToyListenerMorph allInstances 
		detect: [ :each | each world == w]
		ifNone: [EToyListenerMorph new open]! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 12:46'!
flashIndicator: ignoredForNow! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 14:12'!
globalIncomingQueue

	^GlobalIncomingQueue ifNil: [GlobalIncomingQueue := OrderedCollection new].! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 14:13'!
globalIncomingQueueCopy

	^self critical: [self globalIncomingQueue copy].
! !

!EToyListenerMorph class methodsFor: 'class initialization' stamp: 'RAA 7/25/2000 16:28'!
initialize
"
EToyListenerMorph initialize
"
	
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.
! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 8/9/2000 17:56'!
isListening

	^GlobalListener notNil
! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 5/1/2002 19:29'!
makeListeningToggleNew: activeMode

	| background c baseExtent bgExtent botCent factor len endPts base |

	factor := 2.
	bgExtent := (50@25) * factor.
	baseExtent := (15@15) * factor.
	background := Form extent: bgExtent depth: 8.
	botCent := background boundingBox bottomCenter.
	c := background getCanvas.
"c fillColor: Color white."
	base :=  (botCent - (baseExtent // 2)) extent: baseExtent.
	c
		fillOval: base
		color: Color black 
		borderWidth: 0 
		borderColor: Color black.
	activeMode ifTrue: [
		len := background boundingBox height - 15.
		endPts := {botCent - (len@len). botCent - (len negated@len)}.
		endPts do: [ :each |
			c line: botCent to: each width: 2 color: Color black.
		].
		endPts do: [ :each |
			#(4 8 12) do: [ :offset |
				c frameOval: (each - offset corner: each + offset) color: Color red
			].
		].
	].
"background asMorph openInWorld."
	^background


	! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/19/2000 18:59'!
makeListeningToggle: withEars

	| background c capExtent bgExtent earExtent earDeltaX earDeltaY botCent factor parts |

	factor := 2.
	bgExtent := (50@25) * factor.
	capExtent := (30@30) * factor.
	earExtent := (15@15) * factor.
	earDeltaX := capExtent x // 2.
	earDeltaY := capExtent y // 2.
	background := Form extent: bgExtent depth: 8.
	botCent := background boundingBox bottomCenter.
	c := background getCanvas.
	"c fillColor: Color white."
	parts := {
		(botCent - (capExtent // 2)) extent: capExtent.
	}.
	withEars ifTrue: [
		parts := parts , {
			(botCent - (earDeltaX @ earDeltaY) - (earExtent // 2)) extent: earExtent.
			(botCent - (earDeltaX negated @ earDeltaY) - (earExtent // 2)) extent: earExtent.
		} 
	].
	parts do: [ :each |
		c
			fillOval: each
			color: Color black 
			borderWidth: 0 
			borderColor: Color black.
	].
	^background

"=====
	f2 := Form extent: 30@15 depth: 8.
	background displayInterpolatedOn: f2.
	f2 replaceColor: Color white withColor: Color transparent.
	^f2
====="


	! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 18:24'!
removeFromGlobalIncomingQueue: theActualObject

	self critical: [
		GlobalIncomingQueue := self globalIncomingQueue reject: [ :each | 
			each second == theActualObject
		].
		self bumpUpdateCounter.
	].! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 12:47'!
resetIndicator: ignoredForNow! !

!EToyListenerMorph class methodsFor: 'system startup' stamp: 'RAA 7/25/2000 16:26'!
shutDown: quitting

	WasListeningAtShutdown := GlobalListener notNil.
	self stopListening.
! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 18:24'!
startListening

	self stopListening.
	GlobalListener := EToyPeerToPeer new awaitDataFor: self.
	self bumpUpdateCounter.

! !

!EToyListenerMorph class methodsFor: 'system startup' stamp: 'RAA 7/25/2000 16:27'!
startUp: resuming

	WasListeningAtShutdown == true ifTrue: [
		self startListening.
	].
! !

!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'sw 7/3/2001 21:54'!
stopListening
	GlobalListener ifNotNil:
		[GlobalListener stopListening.
		GlobalListener := nil.
		self bumpUpdateCounter]

	"EToyListenerMorph stopListening"! !

!EToyListenerMorph class methodsFor: 'class initialization' stamp: 'ads 7/18/2003 09:07'!
unload
	Smalltalk removeFromStartUpList: self.
	Smalltalk removeFromShutDownList: self.
! !

!EToyListenerMorph methodsFor: 'as yet unclassified' stamp: 'RAA 5/1/2002 17:57'!
addNewObject: newObject thumbForm: aForm sentBy: senderName ipAddress: ipAddressString

	| thumb row |

	thumb := aForm asMorph.
	thumb setProperty: #depictedObject toValue: newObject.
	row := self addARow: {
		thumb. 
		self inAColumn: {
			StringMorph new contents: senderName; lock.
			StringMorph new contents: ipAddressString; lock.
		}
	}.
	true ifTrue: [	"simpler protocol"
		row on: #mouseUp send: #mouseUpEvent:for: to: self.
	] ifFalse: [
		row on: #mouseDown send: #mouseDownEvent:for: to: self.
	].

! !

!EToyListenerMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:34'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ Color blue! !

!EToyListenerMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:37'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 4! !

!EToyListenerMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:25'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color lightBlue! !

!EToyListenerMorph methodsFor: 'submorphs-add/remove' stamp: 'RAA 7/15/2000 10:44'!
delete

	listener ifNotNil: [listener stopListening. listener := nil].	
					"for old instances that were locally listening"
	super delete.! !

!EToyListenerMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 19:44'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	self listDirection: #topToBottom;
		 layoutInset: 4;
		 rebuild ! !

!EToyListenerMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:50'!
mouseDownEvent: event for: aMorph 
	| menu selection depictedObject |
	depictedObject := aMorph firstSubmorph valueOfProperty: #depictedObject.
	menu := CustomMenu new.
	menu
		add: 'Grab' translated action: [event hand attachMorph: depictedObject veryDeepCopy];
		add: 'Delete' translated
			action: 
				[self class removeFromGlobalIncomingQueue: depictedObject.
				self rebuild].
	selection := menu build startUpCenteredWithCaption: 'Morph from ' translated
						, (aMorph submorphs second) firstSubmorph contents.
	selection ifNil: [^self].
	selection value! !

!EToyListenerMorph methodsFor: 'as yet unclassified' stamp: 'RAA 5/1/2002 17:58'!
mouseUpEvent: event for: aMorph

	| depictedObject |

	depictedObject := aMorph firstSubmorph valueOfProperty: #depictedObject.
	event hand attachMorph: depictedObject.
	self class removeFromGlobalIncomingQueue: depictedObject.
	self rebuild.
! !

!EToyListenerMorph methodsFor: 'as yet unclassified' stamp: 'KR 3/25/2008 17:51'!
rebuild

	| earMorph |
	updateCounter _ UpdateCounter.
	self removeAllMorphs.
	self addGateKeeperMorphs.
	GlobalListener ifNil: [
		earMorph _ (self class makeListeningToggleNew: false) asMorph.
		earMorph setBalloonText: 'Click to START listening for messages' translated.
		earMorph on: #mouseUp send: #startListening to: self.
	] ifNotNil: [
		earMorph _ (self class makeListeningToggleNew: true) asMorph.
		earMorph setBalloonText: 'Click to STOP listening for messages' translated.
		earMorph on: #mouseUp send: #stopListening to: self.
	].
	self addARow: {self inAColumn: {earMorph}}.
	self
		addARow: {
			self inAColumn: {(StringMorph contents: 'Incoming communications' translated ) lock}.
			self indicatorFieldNamed: #working color: Color blue help: 'working' translated.
			self indicatorFieldNamed: #communicating color: Color green help: 'receiving' translated.
		}.
	"{thumbForm. newObject. senderName. ipAddressString}"
	self class globalIncomingQueueCopy do: [ :each |
		self
			addNewObject: each second 
			thumbForm: each first 
			sentBy: each third 
			ipAddress: each fourth.
	].! !

!EToyListenerMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 18:23'!
startListening

	self class startListening! !

!EToyListenerMorph methodsFor: 'stepping and presenter' stamp: 'RAA 7/31/2000 12:49'!
step

	| needRebuild |
	super step.
	needRebuild := false.
	(self valueOfProperty: #gateKeeperCounterValue) = 
			EToyGateKeeperMorph updateCounter ifFalse: [needRebuild := true].
	updateCounter = UpdateCounter ifFalse: [
		needRebuild := true.
	].
	needRebuild ifTrue: [self rebuild].
! !

!EToyListenerMorph methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 18:23'!
stopListening

	self class stopListening! !

EToyCommunicatorMorph subclass: #EToyMorphsWelcomeMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Collaborative'!

!EToyMorphsWelcomeMorph commentStamp: '<historical>' prior: 0!
EToyMorphsWelcomeMorph new openInWorld!

!EToyMorphsWelcomeMorph class methodsFor: 'parts bin' stamp: 'tak 7/25/2007 00:05'!
descriptionForPartsBin

	^ self partName: 	'Welcome' translatedNoop
		categories:		#()
		documentation:	'A sign that you accept morphs dropped directly into your world' translatedNoop! !

!EToyMorphsWelcomeMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 7/16/2000 13:44'!
morphsWelcomeInWorld: aWorld

	^self allInstances anySatisfy: [ :each | each world == aWorld]! !

!EToyMorphsWelcomeMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:25'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color yellow! !

!EToyMorphsWelcomeMorph methodsFor: 'initialization' stamp: 'KR 3/25/2008 17:51'!
initialize
	"initialize the state of the receiver"
	| earMorph |
	super initialize.
	""
	
	self layoutInset: 8 @ 8.
	"earMorph _ (EToyListenerMorph makeListeningToggle: true)  
	asMorph."
	earMorph _ TextMorph new contents: 'Morphs
welcome
here';
				 fontName: Preferences standardEToysFont familyName size: 18;
				 centered;
				 lock.
	self addARow: {earMorph}.
	self setBalloonText: 'My presence in this world means received morphs may appear automatically' translated! !

SimpleButtonMorph subclass: #ChatButtonMorph
	instanceVariableNames: 'actionDownSelector actionUpSelector labelDown labelUp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Audio Chat'!

!ChatButtonMorph methodsFor: 'accessing' stamp: 'Tbp 4/11/2000 16:25'!
actionDownSelector: aSymbolOrString

	(nil = aSymbolOrString or:
	['nil' = aSymbolOrString or:
	[aSymbolOrString isEmpty]])
		ifTrue: [^actionDownSelector := nil].

	actionDownSelector := aSymbolOrString asSymbol.! !

!ChatButtonMorph methodsFor: 'accessing' stamp: 'Tbp 4/11/2000 16:27'!
actionUpSelector: aSymbolOrString


	(nil = aSymbolOrString or:
	 ['nil' = aSymbolOrString or:
	 [aSymbolOrString isEmpty]])
		ifTrue: [^ actionUpSelector := nil].

	actionUpSelector := aSymbolOrString asSymbol.! !

!ChatButtonMorph methodsFor: 'events' stamp: 'dgd 2/22/2003 18:40'!
doButtonDownAction
	(target notNil and: [actionDownSelector notNil]) 
		ifTrue: [Cursor normal showWhile: [target perform: actionDownSelector]]! !

!ChatButtonMorph methodsFor: 'events' stamp: 'dgd 2/22/2003 18:40'!
doButtonUpAction
	(target notNil and: [actionUpSelector notNil]) 
		ifTrue: [Cursor normal showWhile: [target perform: actionUpSelector]]! !

!ChatButtonMorph methodsFor: 'accessing' stamp: 'Tbp 4/11/2000 16:31'!
labelDown: aString

	labelDown := aString.! !

!ChatButtonMorph methodsFor: 'accessing' stamp: 'Tbp 4/11/2000 16:32'!
labelUp: aString

	labelUp := aString! !

!ChatButtonMorph methodsFor: 'event handling' stamp: 'ar 6/4/2001 00:40'!
mouseDown: evt

	oldColor := self fillStyle.
	self label: labelDown.
	self doButtonDownAction.

! !

!ChatButtonMorph methodsFor: 'event handling' stamp: 'RAA 8/6/2000 18:37'!
mouseUp: evt

	"if oldColor nil, it signals that mouse had not gone DOWN inside me, e.g. because of a cmd-drag; in this case we want to avoid triggering the action!!"

	oldColor ifNil: [^self].
	self color: oldColor.
	(self containsPoint: evt cursorPoint) ifTrue: [
		self label: labelUp.
		self doButtonUpAction.
	].
! !

!InfiniteForm methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 8/28/2000 10:27'!
encodeForRemoteCanvas

	^patternForm encodeForRemoteCanvas
! !

MorphicModel subclass: #EToyGateKeeperEntry
	instanceVariableNames: 'ipAddress accessAttempts lastTimes acceptableTypes latestUserName attempsDenied lastRequests'
	classVariableNames: 'KnownIPAddresses'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Experimental'!

!EToyGateKeeperEntry class methodsFor: 'new-morph participation' stamp: 'RAA 8/3/2000 07:48'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ false! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 12:42'!
acceptableTypes

	^acceptableTypes! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 14:11'!
dateAndTimeStringFrom: totalSeconds

	| dateAndTime |
	dateAndTime := Time dateAndTimeFromSeconds: totalSeconds.
	^dateAndTime first printString,' ',dateAndTime second printString! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 13:51'!
fullInfoString

	^self latestUserName,
		' at ',
		ipAddress ,
		' attempts: ',
		accessAttempts printString,
		'/',
		attempsDenied printString,
		' last: ',
		(self lastIncomingMessageTimeString)
	 
"acceptableTypes"

 ! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 12:19'!
getChoice: aString

	^acceptableTypes includes: aString! !

!EToyGateKeeperEntry methodsFor: 'initialization' stamp: 'RAA 8/4/2000 11:49'!
initialize

	self flag: #bob.		"need to decide better initial types"

	super initialize.
	ipAddress := '???'.
	accessAttempts := attempsDenied := 0.
	lastRequests := OrderedCollection new.
	acceptableTypes := Set withAll: EToyIncomingMessage allTypes.

 ! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 09:33'!
ipAddress

	^ipAddress! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 09:18'!
ipAddress: aString

	ipAddress := aString! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 13:37'!
lastIncomingMessageTimeString

	lastRequests isEmpty ifTrue: [^'never'].
	^self dateAndTimeStringFrom: lastRequests first first
! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 13:56'!
lastTimeChecked

	^self valueOfProperty: #lastTimeChecked
! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 14:22'!
lastTimeCheckedString

	| statusTime |
	statusTime := self valueOfProperty: #lastTimeChecked ifAbsent: [^'none'].
	^(self dateAndTimeStringFrom: statusTime)! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 13:57'!
lastTimeChecked: aDateAndTimeInSeconds

	self setProperty: #lastTimeChecked toValue: aDateAndTimeInSeconds
! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 10:49'!
latestUserName

	^latestUserName ifNil: ['???']! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 10:46'!
latestUserName: aString

	latestUserName := aString! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 14:09'!
requestAccessOfType: aString

	| ok |

	accessAttempts := accessAttempts + 1.
	lastRequests addFirst: {Time totalSeconds. aString}.
	lastRequests size > 10 ifTrue: [
		lastRequests := lastRequests copyFrom: 1 to: 10.
	].
	ok := (acceptableTypes includes: aString) or: [acceptableTypes includes: 'all'].
	ok ifFalse: [attempsDenied := attempsDenied + 1].
	^ok! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 8/1/2000 14:16'!
statusReplyReceivedString

	| statusTime |
	statusTime := self valueOfProperty: #lastStatusReplyTime ifAbsent: [^'none'].
	^(self dateAndTimeStringFrom: statusTime),' accepts:
', (self valueOfProperty: #lastStatusReply) asArray printString! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 14:10'!
statusReplyReceived: anArray

	self setProperty: #lastStatusReplyTime toValue: Time totalSeconds.
	self setProperty: #lastStatusReply toValue: anArray.! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 09:38'!
timeBetweenLastAccessAnd: currentTime

	lastRequests isEmpty ifTrue: [^0].
	^currentTime - lastRequests first first
! !

!EToyGateKeeperEntry methodsFor: 'as yet unclassified' stamp: 'RAA 7/11/2000 10:39'!
toggleChoice: aString

	(acceptableTypes includes: aString) ifTrue: [
		acceptableTypes remove: aString ifAbsent: []
	] ifFalse: [
		acceptableTypes add: aString
	].! !

EmbeddedWorldBorderMorph subclass: #NetworkTerminalBorderMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NetworkTerminalBorderMorph methodsFor: 'boxes' stamp: 'RAA 8/15/2000 12:21'!
boxesAndColorsAndSelectors

	^#()! !

!NetworkTerminalBorderMorph methodsFor: 'initialization' stamp: 'RAA 12/14/2000 14:12'!
initialize

	super initialize.
	self setBalloonText: nil.		"'I am a view on another Squeak'."
	self layoutInset: 0.
! !

!NetworkTerminalBorderMorph methodsFor: 'as yet unclassified' stamp: 'ar 11/2/2000 21:24'!
toggleFullView
	"Toggle the full view for network terminal"
	| fullExtent priorExtent |
	fullExtent := self worldIEnclose extent + (2 * self borderWidth).
	priorExtent := self valueOfProperty: #priorExtent.
	priorExtent ifNil:[
		self setProperty: #priorExtent toValue: self extent.
		self extent: fullExtent.
		self position: self position + self borderWidth asPoint negated.
	] ifNotNil:[
		self removeProperty: #priorExtent.
		self extent: priorExtent.
		self position: (self position max: 0@0).
	].! !

TestCase subclass: #ArbitraryObjectSocketTestCase
	instanceVariableNames: 'socket1 socket2 end1 end2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Network-ObjectSocket'!

!ArbitraryObjectSocketTestCase methodsFor: 'setup' stamp: 'ul 10/21/2014 17:10'!
setUp
	"it would be nice to have an in-image loopback socket, so that the tests do not need the underlying platform's sockets to behave nicely"
	socket1 := Socket newTCP.
	socket2 := Socket newTCP.
	
	socket1 listenOn: 9999.
	socket2 connectTo: (NetNameResolver localHostAddress) port: socket1 port.

	socket1 waitForConnectionFor: 60.	
	socket2 waitForConnectionFor: 60.
	
	end1 := ArbitraryObjectSocket on: socket1.
	end2 := ArbitraryObjectSocket on: socket2.
	! !

!ArbitraryObjectSocketTestCase methodsFor: 'running' stamp: 'ul 10/23/2009 09:03'!
tearDown

	socket1 ifNotNil: [ socket1 destroy ].
	socket2 ifNotNil: [ socket2 destroy ]! !

!ArbitraryObjectSocketTestCase methodsFor: 'testing' stamp: 'ul 10/23/2009 09:13'!
testBasics
	end1 nextPut: 'hello'.
	end1 nextPut: 42.
	end1 nextPut: 3@5.
	end1 processIO.  "hopefully one call is enough...."
	10 milliSeconds asDelay wait. "data sent, wait a few milliseconds"
	end2 processIO.  "hopefully one call is enough...."
	self should: [ end2 next = 'hello' ].
	self should: [ end2 next = 42 ].
	self should: [ end2 next = (3@5) ].
	! !

TestCase subclass: #StringSocketTestCase
	instanceVariableNames: 'socket1 socket2 end1 end2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Network-ObjectSocket'!

!StringSocketTestCase methodsFor: 'running' stamp: 'ul 10/21/2014 17:10'!
setUp
	"it would be nice to have an in-image loopback socket, so that the tests do not need the underlying platform's sockets to behave nicely"
	socket1 := Socket newTCP.
	socket2 := Socket newTCP.
	
	socket1 listenOn: 9999.
	socket2 connectTo: (NetNameResolver localHostAddress) port: socket1 port.

	socket1 waitForConnectionFor: 60.	
	socket2 waitForConnectionFor: 60.
	
	end1 := StringSocket on: socket1.
	end2 := StringSocket on: socket2.
	! !

!StringSocketTestCase methodsFor: 'running' stamp: 'ls 8/4/2004 15:22'!
tearDown
	end1 destroy.
	end2 destroy.
	! !

!StringSocketTestCase methodsFor: 'running' stamp: 'ar 4/5/2010 22:08'!
testBasics
	end1 nextPut: #().
	end1 nextPut: #('').
	end1 nextPut: #('hello' 'world').
	end1 processIO.

	"The wait makes this test a bit more robust because network delivery 
	is asynchronous and may not have happened when we call end2 processIO. 
	This causes the test to fail occasionally for no apparent reason."
	(Delay forMilliseconds: 100) wait.

	end2 processIO.
	self should: [ end2 next = #() ].
	self should: [ end2 next = #('') ].
	self should: [ end2 next = #('hello' 'world') ].
	! !

!StringSocketTestCase methodsFor: 'running' stamp: 'ul 10/23/2009 09:14'!
testBogusInput1
	| negString |
	negString := String new: 4.
	negString putInteger32: -10 at: 1.
	socket1 sendData: negString.
	10 milliSeconds asDelay wait. "data sent, wait a few milliseconds"
	end2 processIO.
	
	self should: [ end2 isConnected not ].
	! !

!StringSocketTestCase methodsFor: 'running' stamp: 'ul 10/23/2009 09:14'!
testBogusInput2
	| bogoString |
	bogoString := String new: 8.
	bogoString putInteger32: 2 at: 1.
	bogoString putInteger32: -10 at: 5.
	socket1 sendData: bogoString.
	10 milliSeconds asDelay wait. "data sent, wait a few milliseconds"
	end2 processIO.
	
	self should: [ end2 isConnected not ].
	! !

!TTCFont methodsFor: '*nebraska-file in/out' stamp: 'ul 5/16/2011 01:11'!
encodedForRemoteCanvas

	^String streamContents: [ :stream |
		stream
			nextPutAll: self familyName;
			space;
			print: self pointSize;
			space;
			print: self emphasis ]! !

Object subclass: #CanvasDecoder
	instanceVariableNames: 'drawingCanvas clipRect transform connection fonts'
	classVariableNames: 'CachedForms DecodeTable'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!CanvasDecoder commentStamp: '<historical>' prior: 0!
Decodes commands encoded by MREncoder, and draws them onto a canvas.!

!CanvasDecoder class methodsFor: 'instance creation' stamp: 'nk 7/30/2004 21:51'!
connection: aConnection 
	^(self new)
		connection: aConnection;
		yourself! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'RAA 7/25/2000 13:06'!
decodeColor: string
	| rgb a rgb1 rgb2 |
	rgb1 := string getInteger32: 1.
	rgb2 := string getInteger32: 5.
	a := string getInteger32: 9.
	rgb := rgb2 << 16 + rgb1.

	a < 255
		ifTrue: [ ^TranslucentColor basicNew setRgb: rgb  alpha: a/255.0 ]
		ifFalse: [ ^Color basicNew setRGB: rgb ]! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'RAA 7/28/2000 08:33'!
decodeFillStyle: string

	^DataStream unStream: string! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'yo 12/17/2005 22:42'!
decodeFontSet: fontString

	^ StrikeFontSet decodedFromRemoteCanvas: fontString
! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'yo 12/17/2005 22:44'!
decodeFont: fontString

	^StrikeFont decodedFromRemoteCanvas: fontString.
! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'nice 7/22/2014 22:45'!
decodeImage: string
	| bitsStart depth width height bits rs numColors colorArray |

	bitsStart := string indexOf: $|.
	bitsStart = 0 ifTrue: [^nil].
	rs := ReadStream on: string.
	rs peek == $C ifTrue: [
		rs next.
		numColors := Integer readFromString: (rs upTo: $,).
		colorArray := Array new: numColors.
		1 to: numColors do: [ :i |
			colorArray at: i put: (self decodeColor: (rs next: 12))
		].
	].
	depth := Integer readFromString: (rs upTo: $,).
	width :=  Integer readFromString: (rs upTo: $,).
	height :=  Integer readFromString: (rs upTo: $|).

	bits := Bitmap newFromStream: rs upToEnd asByteArray readStream.

	colorArray ifNil: [
		^Form extent: width@height depth: depth bits: bits
	].
	^(ColorForm extent: width@height depth: depth bits: bits)
		colors: colorArray
! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'ls 9/24/1999 20:10'!
decodeInteger: string
	^Integer readFromString: string! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'ls 3/25/2000 23:02'!
decodePoints: aString
	^(aString findTokens: '|') asArray collect: [ :encPoint | self decodePoint: encPoint ]! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'ls 3/27/2000 00:36'!
decodePoint: string
	| x y |
	x := string getInteger32: 1.
	y := string getInteger32: 5.

	^x@y! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'ls 3/27/2000 22:24'!
decodeRectangle: string
	| x y cornerX cornerY |
	x := string getInteger32: 1.
	y := string getInteger32: 5.
	cornerX := string getInteger32: 9.
	cornerY := string getInteger32: 13.

	^x@y corner: cornerX@cornerY! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'ls 10/9/1999 20:28'!
decodeTransform: transformEnc
	"decode an encoded transform"
	^DisplayTransform fromRemoteCanvasEncoding: transformEnc! !

!CanvasDecoder class methodsFor: 'decoding' stamp: 'yo 12/7/2007 15:38'!
decodeTTCFont: fontString

	"Decode a string that consists of <familyName> <pointSize> <emphasis> (e.g. 'ComicSansMS 12 0') into a proper instance."

	| first second |
	first _ fontString indexOf: $  startingAt: 1.
	second _ fontString indexOf: $  startingAt: first + 1.

	(first ~= 0 and: [second ~= 0]) ifTrue: [
		^ TTCFont familyName: (fontString copyFrom: 1 to: (first - 1))
			size: (fontString copyFrom: first + 1 to: second - 1) asNumber
				emphasized: (fontString copyFrom: second + 1 to: fontString size) asNumber.
	].

	^ TextStyle defaultFont.
! !

!CanvasDecoder class methodsFor: 'decode table modification' stamp: 'nk 6/25/2003 12:49'!
decodeVerb: verb toSelector: selector
	"verb is a single character which will be ferformed by my instances using selector"
	DecodeTable at: verb asciiValue + 1 put: selector.	! !

!CanvasDecoder class methodsFor: 'class initialization' stamp: 'yo 12/15/2005 16:07'!
initialize
	"CanvasDecoder initialize"
	"Set up my cache and decode table if necessary."
	CachedForms ifNil: [CachedForms := Array new: 100].
	DecodeTable ifNotNil: [ ^self ].

	DecodeTable := Array new: 128.
	#((codeClip setClip:)
	(codeTransform setTransform:)
	(codeText drawText:)
	(codeLine drawLine:)
	(codeRect drawRect:)
	(codeBalloonRect drawBalloonRect:)
	(codeBalloonOval drawBalloonOval:)
	(codeInfiniteFill drawInfiniteFill:)
	(codeOval drawOval:)
	(codeImage drawImage:)
	(codeReleaseCache releaseImage:)
	(codePoly drawPoly:)
	(codeStencil drawStencil:)
	(codeForce forceToScreen:)
	(codeFont addFontToCache:)
	(codeFontSet addFontSetToCache:)
	(codeMultiText drawMultiText:) 
	(codeTTCFont addTTCFontToCache:)
	(codeExtentDepth extentDepth:)
	(codeShadowColor shadowColor:))
		do: [ :arr |
			(DecodeTable at: ((CanvasEncoder perform: arr first) asciiValue + 1)) ifNotNil: [self error: 'duplicated code'].
			DecodeTable
				at: ((CanvasEncoder perform: arr first) asciiValue + 1)
				put: arr second
		].
! !

!CanvasDecoder class methodsFor: 'class initialization' stamp: 'nk 6/25/2003 12:46'!
reinitialize
	"CanvasDecoder reinitialize"
	"Set up my cache and decode table, removing old contents."
	CachedForms := nil.
	DecodeTable := nil.
	self initialize.
! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'yo 10/23/2002 23:36'!
addFontSetToCache: command

	| index font |
	index := self class decodeInteger: command second.
	font := self class decodeFontSet: command third.

	index > fonts size ifTrue: [
		| newFonts |
		newFonts  := Array new: index.
		newFonts replaceFrom: 1 to: fonts size with: fonts.
		fonts := newFonts ].

	fonts at: index put: font
! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'ls 3/27/2000 17:57'!
addFontToCache: command
	| index font |
	index := self class decodeInteger: command second.
	font := self class decodeFont: command third.

	index > fonts size ifTrue: [
		| newFonts |
		newFonts  := Array new: index.
		newFonts replaceFrom: 1 to: fonts size with: fonts.
		fonts := newFonts ].

	fonts at: index put: font! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'yo 3/21/2003 23:02'!
addTTCFontToCache: command
	| index font |
	index := self class decodeInteger: command second.
	font := self class decodeTTCFont: command third.

	index > fonts size ifTrue: [
		| newFonts |
		newFonts  := Array new: index.
		newFonts replaceFrom: 1 to: fonts size with: fonts.
		fonts := newFonts ].

	fonts at: index put: font.
! !

!CanvasDecoder methodsFor: 'network' stamp: 'ls 9/26/1999 14:59'!
connection: aStringSocket
	"set this terminal to talk over the given socket"
	connection := aStringSocket! !

!CanvasDecoder methodsFor: 'shutting down' stamp: 'ls 4/9/2000 14:33'!
delete
	connection ifNotNil: [ connection destroy ].! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:42'!
drawBalloonOval: command 
	| aRectangle aFillStyle borderWidth borderColor |
	aRectangle := self class decodeRectangle: command second.
	aFillStyle := self class decodeFillStyle: command third.
	borderWidth := self class decodeInteger: command fourth.
	borderColor := self class decodeColor: (command fifth).
	self drawCommand: 
			[:c | 
			c asBalloonCanvas 
				fillOval: aRectangle
				fillStyle: aFillStyle
				borderWidth: borderWidth
				borderColor: borderColor]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:42'!
drawBalloonRect: command 
	| aRectangle aFillStyle |
	aRectangle := self class decodeRectangle: (command second).
	aFillStyle := self class decodeFillStyle: command third.
	self drawCommand: 
			[:c | 
			c asBalloonCanvas fillRectangle: aRectangle fillStyle: aFillStyle]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'ls 4/9/2000 14:26'!
drawCommand: aBlock
	"call aBlock with the canvas it should actually draw on so that the clipping rectangle and transform are set correctly"
	drawingCanvas transformBy: transform clippingTo: clipRect during: aBlock! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'nk 6/25/2003 12:24'!
drawImage: command 
	| image point sourceRect rule cacheID cacheNew previousImage |
	image := self class decodeImage: command second.
	point := self class decodePoint: command third.
	sourceRect := self class decodeRectangle: command fourth.
	rule := self class decodeInteger: command fifth.
	command size >= 7 
		ifTrue: 
			[false ifTrue: [self showSpaceUsed].	"debugging"
			cacheID := self class decodeInteger: (command sixth).
			cacheNew := (self class decodeInteger: command seventh) = 1.
			cacheID > 0 
				ifTrue: 
					[
					cacheNew 
						ifTrue: [CachedForms at: cacheID put: image]
						ifFalse: 
							[previousImage := CachedForms at: cacheID.
							image ifNil: [image := previousImage]
								ifNotNil: 
									[(previousImage notNil and: [image depth > 8]) 
										ifTrue: [image := previousImage addDeltasFrom: image].
									CachedForms at: cacheID put: image]]]].
	self drawCommand: 
			[:c | 
			c 
				image: image
				at: point
				sourceRect: sourceRect
				rule: rule]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:43'!
drawInfiniteFill: command 
	| aRectangle aFillStyle |
	aRectangle := self class decodeRectangle: (command second).
	aFillStyle := InfiniteForm with: (self class decodeImage: command third).
	self drawCommand: 
			[:c | 
			c asBalloonCanvas fillRectangle: aRectangle fillStyle: aFillStyle]! !

!CanvasDecoder methodsFor: 'attributes' stamp: 'ls 4/9/2000 14:29'!
drawingForm
	"return the form that we are drawing on behind thescenes"
	^drawingCanvas form! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 13:18'!
drawLine: command 
	| verb pt1Enc pt2Enc widthEnc colorEnc pt1 pt2 width color |
	verb := command first.
	pt1Enc := command second.
	pt2Enc := command third.
	widthEnc := command fourth.
	colorEnc := command fifth.
""
	pt1 := self class decodePoint: pt1Enc.
	pt2 := self class decodePoint: pt2Enc.
	width := self class decodeInteger: widthEnc.
	color := self class decodeColor: colorEnc.
""
	self
		drawCommand: [:c | c
				line: pt1
				to: pt2
				width: width
				color: color]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'sd 11/20/2005 21:25'!
drawMultiText: command

	| boundsEnc colorEnc  text bounds color fontIndexEnc fontIndex |

	text := WideString fromByteArray: (command at: 2) asByteArray.
	"text asByteArray printString displayAt: 800@0."
	"self halt."
	boundsEnc := command at: 3.
	fontIndexEnc := command at: 4.
	colorEnc := command at: 5.


	bounds := self class decodeRectangle: boundsEnc.
	fontIndex := self class decodeInteger: fontIndexEnc.
	color := self class decodeColor: colorEnc.

	self drawCommand: [ :c |
		c drawString: text in: bounds font: (fonts at: fontIndex) color: color ]
! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 13:19'!
drawOval: command 
	| verb rectEnc colorEnc borderWidthEnc borderColorEnc rect color borderWidth borderColor |
	verb := command first.
	rectEnc := command second.
	colorEnc := command third.
	borderWidthEnc := command fourth.
	borderColorEnc := command fifth.
	""
	rect := self class decodeRectangle: rectEnc.
	color := self class decodeColor: colorEnc.
	borderWidth := self class decodeInteger: borderWidthEnc.
	borderColor := self class decodeColor: borderColorEnc.
	""
	self
		drawCommand: [:c | c
				fillOval: rect
				color: color
				borderWidth: borderWidth
				borderColor: borderColor]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:43'!
drawPoly: command 
	| verticesEnc fillColorEnc borderWidthEnc borderColorEnc vertices fillColor borderWidth borderColor |
	fillColorEnc := command second.
	borderWidthEnc := command third.
	borderColorEnc := command fourth.
	verticesEnc := command copyFrom: 5 to: command size.
	fillColor := self class decodeColor: fillColorEnc.
	borderWidth := self class decodeInteger: borderWidthEnc.
	borderColor := self class decodeColor: borderColorEnc.
	vertices := verticesEnc collect: [:enc | self class decodePoint: enc].
	self drawCommand: 
			[:c | 
			c 
				drawPolygon: vertices
				color: fillColor
				borderWidth: borderWidth
				borderColor: borderColor]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 13:19'!
drawRect: command 
	| verb rectEnc fillColorEnc borderWidthEnc borderColorEnc rect fillColor borderWidth borderColor |
	verb := command first.
	rectEnc := command second.
	fillColorEnc := command third.
	borderWidthEnc := command fourth.
	borderColorEnc := command fifth.
	""
	rect := self class decodeRectangle: rectEnc.
	fillColor := self class decodeColor: fillColorEnc.
	borderWidth := self class decodeInteger: borderWidthEnc.
	borderColor := self class decodeColor: borderColorEnc.
	""
	self
		drawCommand: [:c | c
				frameAndFillRectangle: rect
				fillColor: fillColor
				borderWidth: borderWidth
				borderColor: borderColor]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:44'!
drawStencil: command 
	| stencilFormEnc locationEnc sourceRectEnc colorEnc stencilForm location sourceRect color |
	stencilFormEnc := command second.
	locationEnc := command third.
	sourceRectEnc := command fourth.
	colorEnc := command fifth.
	stencilForm := self class decodeImage: stencilFormEnc.
	location := self class decodePoint: locationEnc.
	sourceRect := self class decodeRectangle: sourceRectEnc.
	color := self class decodeColor: colorEnc.
	self drawCommand: 
			[:executor | 
			executor 
				stencil: stencilForm
				at: location
				sourceRect: sourceRect
				color: color]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:44'!
drawText: command 
	| boundsEnc colorEnc text bounds color fontIndexEnc fontIndex |
	text := command second.
	boundsEnc := command third.
	fontIndexEnc := command fourth.
	colorEnc := command fifth.
	bounds := self class decodeRectangle: boundsEnc.
	fontIndex := self class decodeInteger: fontIndexEnc.
	color := self class decodeColor: colorEnc.
	self drawCommand: 
			[:c | 
			c 
				drawString: text
				in: bounds
				font: (fonts at: fontIndex)
				color: color]! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:44'!
extentDepth: command 
	| depth extent |
	extent := self class decodePoint: (command second).
	depth := self class decodeInteger: (command third).
	drawingCanvas := FormCanvas extent: extent depth: depth! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'ls 3/26/2000 22:04'!
forceToScreen: aCommand  withBlock: forceBlock
	| region |
	region := self class decodeRectangle: aCommand second.
	forceBlock value: region.! !

!CanvasDecoder methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:25'!
initialize
	"set the canvas to draw on"
	drawingCanvas := FormCanvas extent: 100@100 depth: 16.
	clipRect := drawingCanvas extent.
	transform := MorphicTransform identity.

	fonts := Array new: 2.! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'sd 11/20/2005 21:25'!
processCommand: command  onForceDo: forceBlock
	"Decode the given string command and perform the required action.
	If the command is a forceToScreen command, also pass the forceBlock.
	The previous chained equality tests and conditionals have been replaced by a lookup table in my class variable DecodeTable, which is set in the class-side initialize method."
	| verb verbCode selector |
	command isEmpty ifTrue: [ ^self ].

	verb := command first.
	verbCode := verb first.

	selector := DecodeTable
		at: (verbCode asciiValue + 1)
		ifAbsent: [ self error: 'unknown command: ', verb ].

	"note: codeForce is the only odd one"
	^(selector == #forceToScreen:)
		ifTrue: [ self forceToScreen: command withBlock: forceBlock ]
		ifFalse: [ self perform: selector withArguments: { command } ]
! !

!CanvasDecoder methodsFor: 'network' stamp: 'sd 11/20/2005 21:25'!
processIO
	| command didSomething |
	connection ifNil: [ ^self ].
	connection processIO.
	didSomething := false.
	[ command := connection nextOrNil.  command notNil ] whileTrue: [
		didSomething := true.
		self processCommand: command ].

	^didSomething! !

!CanvasDecoder methodsFor: 'network' stamp: 'sd 11/20/2005 21:25'!
processIOOnForce: forceBlock
	| command didSomething |
	connection ifNil: [ ^self ].
	connection processIO.
	didSomething := false.
	[ command := connection nextOrNil.  command notNil ] whileTrue: [
		didSomething := true.
		self processCommand: command onForceDo: forceBlock].

	^didSomething! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:44'!
releaseImage: command 
	| cacheID |
	CachedForms ifNil: [^self].
	cacheID := self class decodeInteger: (command second).
	CachedForms at: cacheID put: nil! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:44'!
setClip: command 
	| clipRectEnc |
	clipRectEnc := command second.
	clipRect := self class decodeRectangle: clipRectEnc! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'dgd 2/22/2003 18:45'!
setTransform: command 
	| transformEnc |
	transformEnc := command second.
	transform := self class decodeTransform: transformEnc! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'RAA 3/3/2001 18:29'!
shadowColor: command

	drawingCanvas shadowColor: (
		command second = '0' ifTrue: [nil] ifFalse: [self class decodeColor: command second]
	)
! !

!CanvasDecoder methodsFor: 'decoding' stamp: 'RAA 8/28/2000 11:46'!
showSpaceUsed

	| total |
	CachedForms ifNil: [^self].
	total := 0.
	CachedForms do: [ :each |
		each ifNotNil: [
			total := total + (each depth * each width * each height // 8).
		].
	].
	(total // 1024) printString,'     ',
	(Smalltalk garbageCollectMost // 1024) printString,'     ' displayAt: 0@0! !

Object subclass: #CanvasEncoder
	instanceVariableNames: 'connection lastClipRect lastTransform fontCache cachedObjects cachingEnabled'
	classVariableNames: 'SentTypesAndSizes SimpleCounters'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!CanvasEncoder commentStamp: 'nice 3/24/2010 07:40' prior: 0!
Encodes canvas commands into string-arrays format.

---possible further compression for forms ---
600 * 359 * 4    861600

self encodeForRemoteCanvas size 76063
Time millisecondsToRun: [self encodeForRemoteCanvas]

| raw data |
data := self encodeForRemoteCanvas.
raw := RWBinaryOrTextStream on: (String new: 1000).
Time millisecondsToRun: [(GZipWriteStream on: raw) nextPutAll: data; close].
raw contents size
(GZipReadStream on: (ReadStream on: raw contents)) upToEnd size

| raw |
raw := RWBinaryOrTextStream on: (String new: bits size).
raw nextPutAll: bits

Time millisecondsToRun: [bits compressGZip]   50

bits compressGZip size 861620!

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:29'!
aaaReadme
	"these codes are used instead of strings, because String>>= was taking around 20% of the decoder's time"
	! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 17:27'!
at: anIndex count: anInteger

	SimpleCounters ifNil: [(SimpleCounters := Array new: 10) atAllPut: 0].
	SimpleCounters at: anIndex put: (SimpleCounters at: anIndex) + anInteger.! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'RAA 7/28/2000 09:01'!
beginStats

	SentTypesAndSizes := Dictionary new.! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 17:30'!
clearTestVars
"
CanvasEncoder clearTestVars
"
	SimpleCounters := nil

! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'RAA 11/6/2000 15:28'!
codeBalloonOval

	^$O! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'RAA 7/28/2000 07:43'!
codeBalloonRect
	^$R! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeClip
	^$A! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 4/9/2000 14:39'!
codeExtentDepth
	^$M! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeFont
	^$L! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'yo 10/23/2002 23:41'!
codeFontSet

	^ $S
! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeForce
	^$J! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeImage
	^$G! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'RAA 8/25/2000 13:31'!
codeInfiniteFill

	^$i! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeLine
	^$D! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'yo 10/23/2002 23:42'!
codeMultiText

	^ $c
! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeOval
	^$F! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codePoly
	^$H! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeRect
	^$E! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'RAA 7/28/2000 16:50'!
codeReleaseCache
	^$z! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'RAA 3/3/2001 18:24'!
codeShadowColor

	^$s! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeStencil
	^$I! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:34'!
codeText
	^$C! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'ls 3/27/2000 22:35'!
codeTransform
	^$B! !

!CanvasEncoder class methodsFor: 'codes' stamp: 'yo 3/21/2003 23:00'!
codeTTCFont

	^ $T.
! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'RAA 7/24/2000 13:24'!
encodeColor: color
	
	^color encodeForRemoteCanvas! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'RAA 7/28/2000 07:53'!
encodeFillStyle: aFillStyle
	
	^aFillStyle encodeForRemoteCanvas! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'ls 3/27/2000 17:57'!
encodeFont: aFont
	^aFont encodedForRemoteCanvas! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'RAA 12/14/2000 11:30'!
encodeImage: form
	
	| t answer |

	form ifNil: [^''].
	t := Time millisecondsToRun: [answer := form encodeForRemoteCanvas].
	form boundingBox area > 5000 ifTrue: [
		NebraskaDebug at: #FormEncodeTimes add: {t. form extent. answer size}
	].
	^answer

	"HandMorph>>restoreSavedPatchOn: is one culprit here"

! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'ls 3/26/2000 23:12'!
encodeInteger: integer
	^integer asInteger storeString! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'RAA 7/28/2000 08:20'!
encodePoint: point
	
	^point encodeForRemoteCanvas! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'sd 11/20/2005 21:26'!
encodeRectangle: rectangle
	| x y encoded cornerX cornerY |

	x := rectangle origin x asInteger.
	y := rectangle origin y asInteger.
	cornerX := rectangle corner x asInteger.
	cornerY := rectangle corner y asInteger.

	CanvasEncoder at: 2 count:  1.
	encoded := String new: 16.
	encoded putInteger32: x at: 1.
	encoded putInteger32: y at: 5.
	encoded putInteger32: cornerX at: 9.
	encoded putInteger32: cornerY at: 13.

	^encoded! !

!CanvasEncoder class methodsFor: 'encoding' stamp: 'ls 10/9/1999 18:54'!
encodeTransform: transform
	^transform encodeForRemoteCanvas! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'nice 12/25/2013 22:10'!
explainTestVars
"
CanvasEncoder explainTestVars
"
	| answer oneBillion total |

	SimpleCounters ifNil: [^ Beeper beep].
	total := 0.
	oneBillion := 1000 * 1000 * 1000.
	answer := String streamContents: [ :strm | | data putter |
		data := SimpleCounters copy.
		putter := [ :msg :index :nSec | | nReps |
			nReps := data at: index.
			total := total + (nSec * nReps).
			strm nextPutAll: nReps asStringWithCommas,' * ',nSec printString,' ',
					(nSec * nReps / oneBillion printShowingMaxDecimalPlaces: 2),' secs for ',msg; cr
		].
		putter value: 'string socket' value: 1 value: 8000.
		putter value: 'rectangles' value: 2 value: 40000.
		putter value: 'points' value: 3 value: 18000.
		putter value: 'colors' value: 4 value: 8000.
	].
	StringHolder new
		contents: answer;
		openLabel: 'put integer times'.

! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 17:26'!
inspectTestVars
"
CanvasEncoder inspectTestVars
"
	^SimpleCounters

! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'RAA 7/28/2000 09:01'!
killStats

	SentTypesAndSizes := nil! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'nk 8/30/2004 07:47'!
nameForCode: aStringOrChar

	| ch |
	ch := (aStringOrChar isString) ifTrue: [aStringOrChar first] ifFalse: [aStringOrChar].
	ch == self codeBalloonOval ifTrue: [^'balloon oval'].
	ch == self codeBalloonRect ifTrue: [^'balloon rectangle'].
	ch == self codeClip ifTrue: [^'clip'].
	ch == self codeExtentDepth ifTrue: [^'codeExtentDepth'].
	ch == self codeFont ifTrue: [^'codeFont'].
	ch == self codeTTCFont ifTrue: [^'codeTTCFont'].
	ch == self codeForce ifTrue: [^'codeForce'].
	ch == self codeImage ifTrue: [^'codeImage'].
	ch == self codeLine ifTrue: [^'codeLine'].
	ch == self codeOval ifTrue: [^'codeOval'].
	ch == self codePoly ifTrue: [^'codePoly'].
	ch == self codeRect ifTrue: [^'codeRect'].
	ch == self codeReleaseCache ifTrue: [^'codeReleaseCache'].
	ch == self codeStencil ifTrue: [^'codeStencil'].
	ch == self codeText ifTrue: [^'codeText'].
	ch == self codeTransform ifTrue: [^'codeTransform'].
	ch == self codeInfiniteFill ifTrue: [^'codeInfiniteFill'].
	ch == self codeShadowColor ifTrue: [^'shadowColor'].
	^'????'
! !

!CanvasEncoder class methodsFor: 'instance creation' stamp: 'ls 10/20/1999 21:17'!
on: connection
	^self new connection: connection! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'nice 12/27/2009 03:11'!
showStats
"
CanvasEncoder showStats
"
	| answer |

	SentTypesAndSizes ifNil: [^Beeper beep].
	answer := WriteStream on: String new.
	SentTypesAndSizes keys asArray sort do: [ :each | | bucket |
		bucket := SentTypesAndSizes at: each.
		answer nextPutAll: each printString,' ',
				bucket first printString,'  ',
				bucket second asStringWithCommas,' ',
				(self nameForCode: each); cr.
	].
	StringHolder new contents: answer contents; openLabel: 'send/receive stats'.
! !

!CanvasEncoder class methodsFor: 'as yet unclassified' stamp: 'nice 12/25/2013 22:10'!
timeSomeThings
"
CanvasEncoder timeSomeThings
"
	| answer array color iter |

	iter := 1000000.
	array := Array new: 4.
	color := Color red.
	answer := String streamContents: [ :strm | | bm rect writer pt s |
		writer := [ :msg :doer | | ms |
			ms := [iter timesRepeat: doer] timeToRun.
			strm nextPutAll: msg,((ms * 1000 / iter) printShowingMaxDecimalPlaces: 2),' usec'; cr.
		].
		s := String new: 4.
		bm := Bitmap new: 20.
		pt := 100@300.
		rect := pt extent: pt.
	iter := 1000000.
		writer value: 'empty loop ' value: [self].
		writer value: 'modulo ' value: [12345678 \\ 256].
		writer value: 'bitAnd: ' value: [12345678 bitAnd: 255].
		strm cr.
	iter := 100000.
		writer value: 'putInteger ' value: [s putInteger32: 12345678 at: 1].
		writer value: 'bitmap put ' value: [bm at: 1 put: 12345678].
		writer value: 'encodeBytesOf: (big) ' value: [bm encodeInt: 12345678 in: bm at: 1].
		writer value: 'encodeBytesOf: (small) ' value: [bm encodeInt: 5000 in: bm at: 1].
		writer value: 'array at: (in) ' value: [array at: 1].
		writer value: 'array at: (out) ' value: [array at: 6 ifAbsent: []].
		strm cr.
	iter := 10000.
		writer value: 'color encode ' value: [color encodeForRemoteCanvas].
		writer value: 'pt encode ' value: [pt encodeForRemoteCanvas].
		writer value: 'rect encode ' value: [self encodeRectangle: rect].
		writer value: 'rect encode2 ' value: [rect encodeForRemoteCanvas].
		writer value: 'rect encodeb ' value: [rect encodeForRemoteCanvasB].
	].

	StringHolder new contents: answer; openLabel: 'send/receive stats'.
! !

!CanvasEncoder methodsFor: 'connection' stamp: 'RAA 8/1/2000 00:17'!
backlog

	^connection backlog! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 11/6/2000 15:38'!
balloonFillOval: aRectangle fillStyle: aFillStyle borderWidth: bw borderColor: bc

	self sendCommand: {
		String with: CanvasEncoder codeBalloonOval.
		self class encodeRectangle: aRectangle.
		aFillStyle encodeForRemoteCanvas.
		self class encodeInteger: bw.
		self class encodeColor: bc.
	}! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 8/25/2000 13:30'!
balloonFillRectangle: aRectangle fillStyle: aFillStyle

	self sendCommand: {
		String with: CanvasEncoder codeBalloonRect.
		self class encodeRectangle: aRectangle.
		aFillStyle encodeForRemoteCanvas
	}! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 11/7/2000 17:56'!
cachingEnabled: aBoolean

	(cachingEnabled := aBoolean) ifFalse: [
		cachedObjects := nil.
	].
! !

!CanvasEncoder methodsFor: 'connection' stamp: 'RAA 11/7/2000 17:54'!
connection: aStringSocket
	"set this connection to talk over the given socket"

	cachingEnabled := true.
	connection := aStringSocket! !

!CanvasEncoder methodsFor: 'objects from disk' stamp: 'RAA 12/20/2000 17:44'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	cachingEnabled ifNil: [cachingEnabled := true].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.
! !

!CanvasEncoder methodsFor: 'network' stamp: 'ls 9/24/1999 19:52'!
destroy
	self disconnect.! !

!CanvasEncoder methodsFor: 'connection' stamp: 'ls 9/26/1999 15:47'!
disconnect
	connection ifNotNil: [
		connection destroy.
		connection := nil.
	].! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'ls 4/11/2000 18:59'!
drawPolygon: vertices color: aColor borderWidth: bw borderColor: bc
	| encodedVertices |
	encodedVertices := vertices collect: [ :vertex | self class encodePoint: vertex ].

	self sendCommand: {
		String with: CanvasEncoder codePoly.
		self class encodeColor: aColor.
		self class encodeInteger: bw.
		self class encodeColor: bc},  encodedVertices .! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'yo 12/17/2005 22:44'!
drawString: s from: firstIndex to: lastIndex in: boundsRect font: fontOrNil color: c
	| fontIndex str |
	fontIndex := self establishFont: (fontOrNil ifNil: [ TextStyle defaultFont ]).
	str := s asString copyFrom: firstIndex to: lastIndex.
	str isWideString ifTrue: [
		self sendCommand: {
			String with: CanvasEncoder codeMultiText.
			str asByteArray asString.
			self class encodeRectangle: boundsRect.
			self class encodeInteger: fontIndex.
			self class encodeColor: c
		}
	] ifFalse: [
		self sendCommand: {
			String with: CanvasEncoder codeText.
			str.
			self class encodeRectangle: boundsRect.
			self class encodeInteger: fontIndex.
			self class encodeColor: c
		}
	].
! !

!CanvasEncoder methodsFor: 'fonts' stamp: 'ls 3/27/2000 18:06'!
establishFont: aFont
	"make sure that the given font is in the fonts cache.  If it is not there already, then transmit it.  Either way, after this returns, the font is in the cache at the index specified by the return value"
	| index |
	(fontCache includesFont: aFont) ifTrue: [ ^fontCache indexOf: aFont ].
	index := fontCache indexForNewFont: aFont.
	self sendFont: aFont atIndex: index.
	^index! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'ls 4/9/2000 14:39'!
extent: newExtent  depth: newDepth
	self sendCommand: {
		self class codeExtentDepth asString.
		self class encodePoint: newExtent. 
		self class encodeInteger: newDepth.
	}! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'ls 4/11/2000 18:59'!
fillOval: r color: c borderWidth: borderWidth borderColor: borderColor
	self sendCommand: {
		String with: CanvasEncoder codeOval.
		self class encodeRectangle: r.
		self class encodeColor: c.
		self class encodeInteger: borderWidth.
		self class encodeColor: borderColor
	}! !

!CanvasEncoder methodsFor: 'network' stamp: 'ls 3/21/2000 23:22'!
flush
	connection ifNotNil: [
		connection flush ]! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'ls 4/11/2000 18:59'!
forceToScreen: aRectangle
	self sendCommand: {
		String with: CanvasEncoder codeForce.
		self class encodeRectangle: aRectangle }! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 8/25/2000 13:12'!
frameAndFillRectangle: r fillColor: fillColor borderWidth: borderWidth borderColor: borderColor

	self sendCommand: {
		String with: CanvasEncoder codeRect.
		self class encodeRectangle: r.
		fillColor encodeForRemoteCanvas.
		self class encodeInteger: borderWidth.
		self class encodeColor: borderColor
	}! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'yo 9/11/2007 18:39'!
image: aForm at: aPoint sourceRect: sourceRect rule: argRule

	| cacheID cacheNew cacheReply formToSend cacheEntry destRect visRect aFormArea d2 rule |

	rule _ argRule.

	"first if we are only going to be able to draw a small part of the form,
	it may be faster just to send the part of the form that will actually show up"

	destRect _ aPoint extent: sourceRect extent.
	d2 _ (lastTransform invertBoundsRect: destRect) expandBy: 1.
	(d2 intersects: lastClipRect) ifFalse: [
		^NebraskaDebug at: #bigImageSkipped add: {lastClipRect. d2}.
	].
	aFormArea _ aForm boundingBox area.
	(aFormArea > 20000 and: [aForm isStatic not and: [lastTransform isPureTranslation]]) ifTrue: [
		visRect _ destRect intersect: lastClipRect.
		visRect area < (aFormArea // 20) ifTrue: [
			"NebraskaDebug 
				at: #bigImageReduced 
				add: {lastClipRect. aPoint. sourceRect extent. lastTransform}."
			formToSend _ aForm copy: (visRect translateBy: sourceRect origin - aPoint).
			formToSend depth = 32 ifTrue: [
				formToSend _ formToSend asFormOfDepth: 16.
				(rule = 24 or: [rule = 34]) ifTrue: [rule _ 25]].
			^self 
				image: formToSend 
				at: visRect origin 
				sourceRect: formToSend boundingBox
				rule: rule
				cacheID: 0 		"no point in trying to cache this - it's a one-timer"
				newToCache: false.
		].
	].

	cacheID _ 0.
	cacheNew _ false.
	formToSend _ aForm.
	(aFormArea > 1000 and: [(cacheReply _ self testCache: aForm) notNil]) ifTrue: [
		cacheID _ cacheReply first.
		cacheEntry _ cacheReply third.
		(cacheNew _ cacheReply second) ifFalse: [
			formToSend _ aForm isStatic 
				ifTrue: [nil] 
				ifFalse: [aForm depth <= 8 ifTrue: [aForm] ifFalse: [aForm deltaFrom: cacheEntry fourth]].
		].
		cacheEntry at: 4 put: (aForm isStatic ifTrue: [aForm] ifFalse: [aForm deepCopy]).
	].
	(formToSend notNil and: [
		formToSend depth = 32 and: [
			rule ~= 24 and: [
				rule ~= 34]]]) ifTrue: [
		formToSend _ formToSend asFormOfDepth: 16.
	].
	self
		image: formToSend 
		at: aPoint 
		sourceRect: sourceRect 
		rule: rule 
		cacheID: cacheID 
		newToCache: cacheNew.

! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 12/14/2000 11:30'!
image: aFormOrNil at: aPoint sourceRect: sourceRect rule: rule cacheID: cacheID newToCache: newToCache

	| t destRect d2 |

	destRect := aPoint extent: sourceRect extent.
	d2 := (lastTransform invertBoundsRect: destRect) expandBy: 1.
	(d2 intersects: lastClipRect) ifFalse: [
		^NebraskaDebug at: #bigImageSkipped add: {lastClipRect. d2}.
	].
	t := Time millisecondsToRun: [
		self sendCommand: {
			String with: CanvasEncoder codeImage.
			self class encodeImage: aFormOrNil.
			self class encodePoint: aPoint.
			self class encodeRectangle: sourceRect.
			self class encodeInteger: rule.
			self class encodeInteger: cacheID.
			self class encodeInteger: (newToCache ifTrue: [1] ifFalse: [0]).
		}.
	].
	(aFormOrNil notNil and: [aFormOrNil boundingBox area > 10000]) ifTrue: [
		NebraskaDebug 
			at: #bigImage 
			add: {lastClipRect. aPoint. sourceRect extent. t. cacheID. newToCache}.
	].

! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 8/25/2000 13:32'!
infiniteFillRectangle: aRectangle fillStyle: aFillStyle

	self sendCommand: {
		String with: CanvasEncoder codeInfiniteFill.
		self class encodeRectangle: aRectangle.
		aFillStyle encodeForRemoteCanvas
	}! !

!CanvasEncoder methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:25'!
initialize

	cachingEnabled := true.
	fontCache := FontCache new: 5.! !

!CanvasEncoder methodsFor: 'connection' stamp: 'ls 9/26/1999 15:45'!
isConnected
	^connection notNil and: [ connection isConnected ]! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 8/14/2000 14:27'!
line: pt1  to: pt2  width: w  color: c

"Smalltalk at: #Q3 put: thisContext longStack."
	self sendCommand: {
		String with: CanvasEncoder codeLine.
		self class encodePoint: pt1.
		self class encodePoint: pt2.
		self class encodeInteger: w.
		self class encodeColor: c
	}! !

!CanvasEncoder methodsFor: 'network' stamp: 'ls 9/24/1999 19:52'!
processIO
	connection ifNil: [ ^self ].
	connection isConnected ifFalse: [ ^self ].
	connection processIO.! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'nice 1/11/2010 20:53'!
purgeCache

	| spaceUsed spaceBefore s | 
	spaceBefore := spaceUsed := self purgeCacheInner.
	spaceBefore > 8000000 ifTrue: [
		Smalltalk garbageCollect.
		spaceUsed := self purgeCacheInner.
	].
	false ifTrue: [
		s := (spaceBefore // 1024) printString,'  ',(spaceUsed // 1024) printString,'  ',
			Time now printString,'     '.
		WorldState addDeferredUIMessage: [s displayAt: 0@0.].
	].
	^spaceUsed
! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'nice 12/27/2009 03:11'!
purgeCacheInner

	| totalSize |

	cachedObjects ifNil: [^0].
	totalSize := 0.
	cachedObjects withIndexDo: [ :each :index | | cachedObject thisSize |
		cachedObject := each first first.
		cachedObject ifNil: [
			each second ifNotNil: [
				2 to: each size do: [ :j | each at: j put: nil].
				self sendCommand: {
					String with: CanvasEncoder codeReleaseCache.
					self class encodeInteger: index.
				}.
			].
		] ifNotNil: [
			thisSize := cachedObject depth * cachedObject width * cachedObject height // 8.
			totalSize := totalSize + thisSize.
		].
	].
	^totalSize
	"---
	newEntry := {
		WeakArray with: anObject.
		1.
		Time millisecondClockValue.
		nil.
	}.
	---"
! !

!CanvasEncoder methodsFor: 'connection' stamp: 'RAA 11/8/2000 15:06'!
purgeOutputQueue

	connection purgeOutputQueue.! !

!CanvasEncoder methodsFor: 'private' stamp: 'dgd 2/22/2003 14:41'!
sendCommand: stringArray 
	| bucket |
	connection ifNil: [^self].
	connection isConnected ifFalse: [^self].
	connection nextPut: stringArray.
	SentTypesAndSizes ifNil: [^self].
	bucket := SentTypesAndSizes at: stringArray first
				ifAbsentPut: 
					[{ 
						0.
						0.
						0}].
	bucket at: 1 put: bucket first + 1.
	bucket at: 2
		put: (bucket second) 
				+ (stringArray inject: 4 into: [:sum :array | sum + (array size + 4)])! !

!CanvasEncoder methodsFor: 'fonts' stamp: 'yo 9/28/2005 12:06'!
sendFont: aFont atIndex: index
	"Transmits the given fint to the other side"

	| code |
	code _ CanvasEncoder codeFont.
	(aFont isMemberOf: StrikeFontSet) ifTrue: [code _ CanvasEncoder codeFontSet].
	aFont isTTCFont ifTrue: [code _ CanvasEncoder codeTTCFont].
	self sendCommand: {
		String with: code.
		self class encodeInteger: index.
		self class encodeFont: aFont }.
! !

!CanvasEncoder methodsFor: 'clipping and transforming' stamp: 'ls 4/11/2000 18:59'!
setClipRect: newClipRect
	self sendCommand: {
		String with: CanvasEncoder codeClip.
		self class encodeRectangle: newClipRect }! !

!CanvasEncoder methodsFor: 'clipping and transforming' stamp: 'ls 4/11/2000 18:59'!
setTransform: newTransform
	self sendCommand: {
		String with: CanvasEncoder codeTransform.
		self class encodeTransform: newTransform }! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 3/3/2001 18:26'!
shadowColor: aFillStyle

	self sendCommand: {
		String with: CanvasEncoder codeShadowColor.
		aFillStyle ifNil: ['0'] ifNotNil: [aFillStyle encodeForRemoteCanvas].
	}! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'ls 4/11/2000 18:59'!
stencil: stencilForm at: aPoint sourceRect: sourceRect color: aColor
	self sendCommand: {
		String with: CanvasEncoder codeStencil.
		self class encodeImage: stencilForm.
		self class encodePoint: aPoint.
		self class encodeRectangle: sourceRect.
		self class encodeColor: aColor }! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'nice 12/27/2009 03:11'!
testCache: anObject 
	| newEntry firstFree |
	cachingEnabled 
		ifFalse: 
			[cachedObjects := nil.
			^nil].
	cachedObjects ifNil: 
			[cachedObjects := (1 to: 100) collect: 
							[:x | 
							{ 
								WeakArray new: 1.
								nil.
								nil.
								nil}]].
	self purgeCache.
	firstFree := nil.
	cachedObjects withIndexDo: 
			[:each :index | | cachedObject | 
			cachedObject := each first first.
			firstFree ifNil: [cachedObject ifNil: [firstFree := index]].
			cachedObject == anObject 
				ifTrue: 
					[each at: 2 put: (each second) + 1.
					^{ 
						index.
						false.
						each}]].
	firstFree ifNil: [^nil].
	newEntry := { 
				WeakArray with: anObject.
				1.
				Time millisecondClockValue.
				nil}.
	cachedObjects at: firstFree put: newEntry.
	^{ 
		firstFree.
		true.
		newEntry}! !

!CanvasEncoder methodsFor: 'drawing' stamp: 'RAA 7/22/2000 08:02'!
testRectangleFillTiming
| r fillColor borderWidth borderColor t |
"
CanvasEncoder new testRectangleFillTiming
"
	r := 100@100 extent: 300@300.
	fillColor := Color blue.
	borderWidth := 1.
	borderColor := Color red.
	t := Time millisecondsToRun: [
		1000 timesRepeat: [
		{
		String with: CanvasEncoder codeRect.
		self class encodeRectangle: r.
		self class encodeColor: fillColor.
		self class encodeInteger: borderWidth.
		self class encodeColor: borderColor }
		].
	].
	t inspect.! !

!CanvasEncoder methodsFor: 'clipping and transforming' stamp: 'ls 10/9/1999 18:19'!
updateTransform: aTransform andClipRect: aClipRect
	"sets the given transform and clip rectangle, if they aren't already the ones being used"
	aTransform = lastTransform ifFalse: [
		self setTransform: aTransform.
		lastTransform := aTransform ].

	aClipRect = lastClipRect ifFalse: [
		self setClipRect: aClipRect.
		lastClipRect := aClipRect. ].! !

Object subclass: #EToyIncomingMessage
	instanceVariableNames: ''
	classVariableNames: 'MessageHandlers MessageTypes'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Experimental'!

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'yo 9/24/2008 00:44'!
allTypes

	^MessageTypes ifNil: [
		MessageTypes _ {
			self typeKeyboardChat.
			self typeMorph.
			self typeFridge.
			self typeStatusRequest.
			self typeStatusReply.
			self typeSeeDesktop.
			self typeMultiChat.
		}
	]
! !

!EToyIncomingMessage class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 13:21'!
forType: aMessageType send: aSymbol to: anObject

	self messageHandlers at: aMessageType put: {aSymbol. anObject}! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'RAA 8/4/2000 13:32'!
handleNewChatFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	^ EToyChatMorph 
		chatFrom: ipAddressString 
		name: senderName 
		text: (self newObjectFromStream: dataStream).
	! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'nice 1/11/2010 20:54'!
handleNewFridgeMorphFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	| newObject |

	newObject := self newObjectFromStream: dataStream.
	newObject
		setProperty: #fridgeSender toValue: senderName;
		setProperty: #fridgeIPAddress toValue: ipAddressString;
		setProperty: #fridgeDate toValue: Time dateAndTimeNow.
	WorldState addDeferredUIMessage: [EToyFridgeMorph newItem: newObject].
	! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'yo 5/8/2007 15:11'!
handleNewMorphFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	| newObject thumbForm targetWorld |

	newObject _ self newObjectFromStream: dataStream.
	EToyCommunicatorMorph playArrivalSound.
	targetWorld _ self currentWorld.
	(EToyMorphsWelcomeMorph morphsWelcomeInWorld: targetWorld) ifTrue: [
		newObject position: (
			newObject 
				valueOfProperty: #positionInOriginatingWorld 
				ifAbsent: [(targetWorld randomBoundsFor: newObject) topLeft]
		).
		WorldState addDeferredUIMessage: [
			newObject openInWorld: targetWorld.
		] fixTemps.
		^self
	].
	thumbForm _ newObject imageForm scaledToSize: 50@50.
	SugarListenerMorph addToGlobalIncomingQueue: {
		thumbForm. newObject. senderName. ipAddressString
	}.
	WorldState addDeferredUIMessage: [
		SugarListenerMorph ensureListenerInCurrentWorld
	] fixTemps.
! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'RAA 8/17/2000 09:22'!
handleNewMultiChatFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	^ EToyMultiChatMorph 
		chatFrom: ipAddressString 
		name: senderName 
		text: (self newObjectFromStream: dataStream).
	! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'KR 3/25/2008 18:00'!
handleNewSeeDesktopFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	"more later"

	^ EToyChatMorph 
		chatFrom: ipAddressString 
		name: senderName 
		text: ipAddressString,' would like to see your desktop' translated.
	! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'RAA 8/4/2000 13:34'!
handleNewStatusReplyFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	(EToyGateKeeperMorph entryForIPAddress: ipAddressString) statusReplyReceived: (
		self newObjectFromStream: dataStream
	)
! !

!EToyIncomingMessage class methodsFor: 'handlers' stamp: 'KR 3/25/2008 18:01'!
handleNewStatusRequestFrom: dataStream sentBy: senderName ipAddress: ipAddressString

	"more later"

	^ EToyChatMorph 
		chatFrom: ipAddressString 
		name: senderName 
		text: ipAddressString,' would like to know if you are available' translated.
	! !

!EToyIncomingMessage class methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 07:52'!
initializeMessageHandlers

	self
		forType: self typeMorph 
		send: #handleNewMorphFrom:sentBy:ipAddress: 
		to: self;

		forType: self typeFridge 
		send: #handleNewFridgeMorphFrom:sentBy:ipAddress: 
		to: self;

		forType: self typeKeyboardChat 
		send: #handleNewChatFrom:sentBy:ipAddress: 
		to: self;

		forType: self typeMultiChat 
		send: #handleNewMultiChatFrom:sentBy:ipAddress: 
		to: self;

		forType: self typeStatusRequest 
		send: #handleNewStatusRequestFrom:sentBy:ipAddress: 
		to: self;

		forType: self typeStatusReply 
		send: #handleNewStatusReplyFrom:sentBy:ipAddress: 
		to: self;

		forType: self typeSeeDesktop 
		send: #handleNewSeeDesktopFrom:sentBy:ipAddress: 
		to: self.


! !

!EToyIncomingMessage class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 13:35'!
messageHandlers

	^MessageHandlers ifNil: [MessageHandlers := Dictionary new].! !

!EToyIncomingMessage class methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 13:36'!
newObjectFromStream: dataStream

	| newObject |

	[newObject := SmartRefStream objectFromStreamedRepresentation: dataStream upToEnd.]
		on: ProgressInitiationException
		do: [ :ex | 
			ex sendNotificationsTo: [ :min :max :curr |
				"self flashIndicator: #working."
			].
		].
	"self resetIndicator: #working."
	^newObject
! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 11/13/2000 14:39'!
registerType: aMessageType

	MessageTypes := self allTypes copyWith: aMessageType! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 13:20'!
typeAudioChat

	^'audiochat'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/5/2000 19:21'!
typeAudioChatContinuous

	^'audiochat2'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 11:49'!
typeFridge

	^'fridge'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 11:46'!
typeKeyboardChat

	^'chat'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 11:59'!
typeMorph

	^'morph'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/17/2000 07:41'!
typeMultiChat

	^'multichat'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 11:56'!
typeSeeDesktop

	^'seedesktop'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 11:53'!
typeStatusReply

	^'statusreply'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 8/4/2000 11:51'!
typeStatusRequest

	^'statusrequest'! !

!EToyIncomingMessage class methodsFor: 'message types' stamp: 'RAA 11/13/2000 14:39'!
unregisterType: aMessageType

	MessageTypes := self allTypes copyWithout: aMessageType! !

!EToyIncomingMessage methodsFor: 'as yet unclassified' stamp: 'RAA 8/17/2000 09:22'!
incomingMessgage: dataStream fromIPAddress: ipAddress

	| nullChar messageType senderName  selectorAndReceiver |

	nullChar := 0 asCharacter.
	messageType := dataStream upTo: nullChar.
	senderName := dataStream upTo: nullChar.
	(EToyGateKeeperMorph acceptRequest: messageType from: senderName at: ipAddress) ifFalse: [
		^self
	].
	selectorAndReceiver := self class messageHandlers at: messageType ifAbsent: [^self].
	^selectorAndReceiver second 
		perform: selectorAndReceiver first 
		withArguments: {dataStream. senderName. ipAddress}

! !

Object subclass: #EToyPeerToPeer
	instanceVariableNames: 'socket communicatorMorph process ipAddress connectionQueue dataQueue remoteSocketAddress leftOverData'
	classVariableNames: 'DEBUG PREVTICK'
	poolDictionaries: ''
	category: 'Nebraska-Network-EToy Communications'!

!EToyPeerToPeer class methodsFor: 'as yet unclassified' stamp: 'RAA 7/9/2000 06:21'!
eToyCommunicationsPort

	^34151		"picked at random"! !

!EToyPeerToPeer class methodsFor: 'as yet unclassified' stamp: 'mir 10/10/2000 12:51'!
transmitStreamedObject: outData as: objectCategory to: anIPAddress for: aCommunicator

	| null |
	null := String with: 0 asCharacter.
	self new 
		sendSomeData: {
			objectCategory,null. 
			Preferences defaultAuthorName,null.
			outData
		}
		to: anIPAddress
		for: aCommunicator

! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'bf 7/1/2008 19:51'!
awaitDataFor: aCommunicatorMorph

	Socket initializeNetwork.
	connectionQueue _ ConnectionQueue 
		portNumber: self class eToyCommunicationsPorts 
		queueLength: 6.
	communicatorMorph _ aCommunicatorMorph.
	process _ [self doAwaitData] newProcess.
	process priority: Processor highIOPriority.
	process resume.
! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'nice 4/11/2011 16:45'!
doAwaitData

	[
		socket := connectionQueue getConnectionOrNilLenient.
		socket ifNil: [
			(Delay forMilliseconds: 50) wait
		] ifNotNil: [
			self class new receiveDataOn: socket for: communicatorMorph
		]
	] repeat
! !

!EToyPeerToPeer methodsFor: 'sending' stamp: 'bf 7/1/2008 19:55'!
doConnectForSend

	| addr port |

	addr := NetNameResolver addressForName: (ipAddress copyUpTo: $:).
	addr ifNil: [
		communicatorMorph commResult: {#message -> ('could not find ',ipAddress)}.
		^false].

	port := (ipAddress copyAfter: $:) asInteger.
	port ifNil: [port := self class eToyCommunicationsPorts first].

	socket connectNonBlockingTo: addr port: port.
	[socket waitForConnectionFor: 15]
		on: ConnectionTimedOut
		do: [:ex |
			communicatorMorph commResult: {#message -> ('no connection to ',ipAddress,' (',
				ipAddress,')')}.
			^false].
	^true

! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'nice 1/18/2010 18:50'!
doReceiveData

	| answer |

	answer := [self doReceiveOneMessage] 
		on: Error
		do: [ :ex | 
			communicatorMorph commResult: {#message -> (ex description,' ',socket printString)}.
			^false
		].
	communicatorMorph commResult: {
		#message -> 'OK'. 
		#data -> answer .
		#ipAddress -> remoteSocketAddress.
	}.
	^answer size > 0

! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'bf 7/3/2008 19:24'!
doReceiveOneMessage

	| awaitingLength i length answer header |

	awaitingLength _ true.
	answer _ WriteStream on: String new.
	[awaitingLength] whileTrue: [
		leftOverData _ leftOverData , socket receiveData.
		(i _ leftOverData indexOf: $ ) > 0 ifTrue: [
			awaitingLength _ false.
			header _ leftOverData first: i - 1.
			length _ header asNumber.
			self parseOptionalHeader: header.
			answer nextPutAll: (leftOverData allButFirst: i).
		].
	].
	leftOverData _ ''.
	[answer size < length] whileTrue: [
		answer nextPutAll: socket receiveData.
		communicatorMorph commResult: {#commFlash -> true}.
	].
	answer _ answer contents.
	answer size > length ifTrue: [
		leftOverData _ answer allButFirst: length.
		answer _ answer first: length
	].
	^answer

! !

!EToyPeerToPeer methodsFor: 'sending' stamp: 'bf 7/3/2008 19:36'!
doSendData

	| totalLength myData allTheData |

	myData _ dataQueue next ifNil: [socket sendData: '0 '. ^false].
	totalLength _ (myData collect: [ :x | x size]) sum.
	socket sendData: totalLength printString, self makeOptionalHeader, ' '.
	allTheData _ WriteStream on: (String new: totalLength).
	myData do: [ :chunk | allTheData nextPutAll: chunk asString].
	NebraskaDebug at: #peerBytesSent add: {totalLength}.
	self sendDataCautiously: allTheData contents.
	^true

! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'mir 6/17/2007 20:05'!
receiveDataOn: aSocket for: aCommunicatorMorph

	socket _ aSocket.
	remoteSocketAddress _ socket remoteSocketAddress hostNumber.
	communicatorMorph _ aCommunicatorMorph.
	process _ [
		leftOverData _ ''.
		[self doReceiveData] whileTrue.
		socket closeAndDestroy.
	] newProcess.
	process priority: Processor highIOPriority.
	process resume.
! !

!EToyPeerToPeer methodsFor: 'sending' stamp: 'RAA 8/9/2000 16:30'!
sendDataCautiously: aStringOrByteArray
	"Send all of the data in the given array, even if it requires multiple calls to send it all. Return the number of bytes sent. Try not to send too much at once since this seemed to cause problems talking to a port on the same machine"

	| bytesSent bytesToSend count |

	bytesToSend := aStringOrByteArray size.
	bytesSent := 0.
	[bytesSent < bytesToSend] whileTrue: [
		count := socket 
			sendSomeData: aStringOrByteArray 
			startIndex: bytesSent + 1  
			count: (bytesToSend - bytesSent min: 4000).
		bytesSent := bytesSent + count.
		communicatorMorph commResult: {#commFlash -> true}.
		(Delay forMilliseconds: 10) wait.
	].
	^ bytesSent
! !

!EToyPeerToPeer methodsFor: 'sending' stamp: 'RAA 8/6/2000 09:34'!
sendSomeData: arrayOfByteObjects to: anIPAddress for: aCommunicatorMorph

	dataQueue := self 
		sendSomeData: arrayOfByteObjects 
		to: anIPAddress 
		for: aCommunicatorMorph 
		multiple: false.
	dataQueue nextPut: nil.		"only this message to send"
! !

!EToyPeerToPeer methodsFor: 'sending' stamp: 'RAA 8/6/2000 09:34'!
sendSomeData: arrayOfByteObjects to: anIPAddress for: aCommunicatorMorph multiple: aBoolean

	Socket initializeNetwork.
	socket := Socket newTCP.
	dataQueue := SharedQueue new.
	dataQueue nextPut: arrayOfByteObjects.
	communicatorMorph := aCommunicatorMorph.
	ipAddress := anIPAddress.
	process := [
		self doConnectForSend ifTrue: [
			[self doSendData] whileTrue.
			communicatorMorph commResult: {#message -> 'OK'}.
			socket closeAndDestroy.
		].
	] newProcess.
	process priority: Processor highIOPriority.
	process resume.
	^dataQueue
! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'RAA 7/9/2000 08:22'!
stopListening

	process ifNotNil: [process terminate. process := nil].
	connectionQueue ifNotNil: [connectionQueue destroy. connectionQueue := nil].

! !

Object subclass: #FontCache
	instanceVariableNames: 'fonts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!FontCache commentStamp: '<historical>' prior: 0!
Used by MREncoder and MRDecoder.  It associates an integer index with a number of fonts.  Fonts can be searched by index, and the index can be found for a font that isn't present.  If a font is added to the cache, sometimes the cache will discard another font to make room.!

!FontCache class methodsFor: 'instance creation' stamp: 'ls 3/27/2000 17:33'!
new: size
	^super new initialize: size! !

!FontCache methodsFor: 'lookups' stamp: 'ls 3/27/2000 17:28'!
fontAt: index
	"return the font associated with the given index"
	^fonts at: index! !

!FontCache methodsFor: 'lookups' stamp: 'ls 3/27/2000 17:25'!
includesFont: aFont
	"decide whether the given font is included in the collection"
	^fonts identityIncludes: aFont	! !

!FontCache methodsFor: 'lookups' stamp: 'ls 3/27/2000 17:28'!
indexForNewFont: aFont
	"add aFont to the cache.  Return its index.  The receiver will sometimes choose an index that is already used; that means that aFont is replacing the other font"
	| index |
	index := fonts size atRandom.      "random is simpler to manage than anything else"
	fonts at: index put: aFont.
	^index! !

!FontCache methodsFor: 'lookups' stamp: 'ls 3/27/2000 17:25'!
indexOf: aFont
	"return the index for a given font"
	^fonts identityIndexOf: aFont! !

!FontCache methodsFor: 'initialization' stamp: 'ls 3/27/2000 17:23'!
initialize: cacheSize
	fonts := Array new: cacheSize.! !

Object subclass: #LoopbackStringSocket
	instanceVariableNames: 'associate inArrays outArrays'
	classVariableNames: 'WRITESTRINGSIZES'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!LoopbackStringSocket commentStamp: '<historical>' prior: 0!
a string socket which is connected to another string sockt on the local computer.  Used mostly for testing.!

!LoopbackStringSocket class methodsFor: 'as yet unclassified' stamp: 'RAA 7/22/2000 08:14'!
clearStats

	WRITESTRINGSIZES := nil! !

!LoopbackStringSocket class methodsFor: 'instance creation' stamp: 'ls 3/18/2000 13:24'!
newPair
	"create a connected pair of sockets"
	| a b |
	a := self new.
	b := self new.
	a associate: b.
	b associate: a.
	^{a. b}! !

!LoopbackStringSocket class methodsFor: 'as yet unclassified' stamp: 'RAA 7/22/2000 08:15'!
stats

	^WRITESTRINGSIZES! !

!LoopbackStringSocket methodsFor: 'private' stamp: 'RAA 7/22/2000 07:47'!
arraysFromAssociate: arrays
	"new string-arrays have arrived from our associate"

	inArrays ifNil: [^self].
	inArrays addAll: arrays.! !

!LoopbackStringSocket methodsFor: 'initialization' stamp: 'ls 3/18/2000 13:28'!
associate: aLoopbackStringSocket
	associate := aLoopbackStringSocket.
	inArrays := OrderedCollection new.
	outArrays := OrderedCollection new.! !

!LoopbackStringSocket methodsFor: 'as yet unclassified' stamp: 'RAA 7/22/2000 07:44'!
destroy

	associate := inArrays := outArrays := nil.! !

!LoopbackStringSocket methodsFor: 'I/O' stamp: 'ls 3/21/2000 23:40'!
flush! !

!LoopbackStringSocket methodsFor: 'I/O' stamp: 'ls 3/18/2000 13:27'!
isConnected
	^true! !

!LoopbackStringSocket methodsFor: 'I/O' stamp: 'RAA 7/22/2000 07:47'!
nextOrNil

	inArrays ifNil: [^nil].

	inArrays isEmpty 
		ifTrue: [ ^nil ]
		ifFalse: [
			^inArrays removeFirst. ]	! !

!LoopbackStringSocket methodsFor: 'I/O' stamp: 'nice 3/19/2010 19:05'!
nextPut: aStringArray

	inArrays ifNil: [^aStringArray].

	^outArrays add: aStringArray.
	"WRITESTRINGSIZES ifNil: [WRITESTRINGSIZES := Bag new].
	aStringArray do: [ :each | WRITESTRINGSIZES add: each size]."! !

!LoopbackStringSocket methodsFor: 'I/O' stamp: 'RAA 7/22/2000 07:48'!
processIO

	inArrays ifNil: [^self].

	associate arraysFromAssociate: outArrays.
	outArrays := OrderedCollection new.! !

Object subclass: #MorphicEventDecoder
	instanceVariableNames: 'connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!MorphicEventDecoder commentStamp: '<historical>' prior: 0!
decode messages sent via a MorphicEventEncoder.!

!MorphicEventDecoder class methodsFor: 'instance creation' stamp: 'ls 3/24/2000 21:43'!
on: aStringArray
	^self basicNew connection: aStringArray! !

!MorphicEventDecoder methodsFor: 'handling messages' stamp: 'ar 10/26/2000 01:55'!
applyEventMessage: aStringArray to: aHand
	| event |
	event := MorphicEvent fromStringArray: (aStringArray copyFrom: 2 to: aStringArray size).
	event ifNotNil:[aHand queueEvent: event].! !

!MorphicEventDecoder methodsFor: 'handling messages' stamp: 'ls 3/25/2000 16:56'!
applyMessagesTo: aHand
	| msg |
	"apply all queued events to the given hand"
	"currently, there is no way to extract the rawmessages.  This is simply because I didn't feel like implementing individual classes for each message -lex"
	[ msg := connection nextOrNil.  msg notNil ] whileTrue: [
		self apply: msg to: aHand ].
! !

!MorphicEventDecoder methodsFor: 'handling messages' stamp: 'ls 4/11/2000 19:00'!
applyViewExtentMessage: aStringArray to: aHand
	| newViewExtent |
	newViewExtent := CanvasDecoder decodePoint: aStringArray second.

	aHand setViewExtent: newViewExtent! !

!MorphicEventDecoder methodsFor: 'handling messages' stamp: 'RAA 11/8/2000 15:15'!
apply: aStringArray to: aHand
	"decode aStringArray, and apply the encoded command to aHand"

	aStringArray first = 'event' ifTrue: [
		^self applyEventMessage: aStringArray to: aHand
	].
	aStringArray first = 'viewExtent' ifTrue: [
		^self applyViewExtentMessage: aStringArray to: aHand
	].
	aStringArray first = 'beginBuffering' ifTrue: [
		^aHand convertRemoteClientToBuffered
	].

	^self error: 'unknown message type: ', aStringArray first! !

!MorphicEventDecoder methodsFor: 'initialization' stamp: 'ls 3/24/2000 21:42'!
connection: aConnection
	connection := aConnection! !

!MorphicEventDecoder methodsFor: 'handling messages' stamp: 'ls 3/24/2000 22:54'!
processIO
	connection processIO! !

Object subclass: #MorphicEventEncoder
	instanceVariableNames: 'connection lastEventSent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!MorphicEventEncoder commentStamp: '<historical>' prior: 0!
A filter which translates MorphEvent's into StringArray's.!

!MorphicEventEncoder class methodsFor: 'instance creation' stamp: 'ls 3/24/2000 21:43'!
on: aStringArray
	^self basicNew connection: aStringArray! !

!MorphicEventEncoder methodsFor: 'initialization' stamp: 'ls 3/24/2000 21:38'!
connection: aConnection
	connection := aConnection! !

!MorphicEventEncoder methodsFor: 'network I/O' stamp: 'ls 3/24/2000 21:42'!
flush
	connection flush! !

!MorphicEventEncoder methodsFor: 'network I/O' stamp: 'ls 3/24/2000 21:38'!
processIO
	connection processIO! !

!MorphicEventEncoder methodsFor: 'network I/O' stamp: 'RAA 12/13/2000 08:19'!
requestBufferedConnection
	"request the opposite side to send complete screen updates rather than discrete drawing commands"
	
	connection nextPut: { 'beginBuffering' }
! !

!MorphicEventEncoder methodsFor: 'network I/O' stamp: 'ls 3/26/2000 01:08'!
sendEvent: anEvent
	(anEvent isMouseMove and: [ anEvent = lastEventSent ]) ifTrue: [
		"save on network traffic--don't send duplicate mouse moves"
		^self ].
	lastEventSent := anEvent.
	connection nextPut: #('event'), anEvent encodedAsStringArray! !

!MorphicEventEncoder methodsFor: 'network I/O' stamp: 'ls 4/11/2000 18:59'!
sendViewExtent: newExtent
	"inform the opposite side that our view extent has changed"
	
	connection nextPut: { 'viewExtent'. CanvasEncoder encodePoint: newExtent }
! !

Object subclass: #NebraskaClient
	instanceVariableNames: 'connection encoder hand canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NebraskaClient commentStamp: '<historical>' prior: 0!
A client that has connected to a Nebraska server, seen from the server's point of view.!

!NebraskaClient class methodsFor: 'instance creation' stamp: 'ls 3/25/2000 22:28'!
onConnection: aStringSocket
	^self new initialize: aStringSocket! !

!NebraskaClient methodsFor: 'as yet unclassified' stamp: 'RAA 7/31/2000 23:59'!
backlog

	^connection backlog! !

!NebraskaClient methodsFor: 'attributes' stamp: 'ls 3/25/2000 22:27'!
canvas
	"return the hand this canvas that should be drawn on for this client"
	^canvas! !

!NebraskaClient methodsFor: 'initialization' stamp: 'RAA 11/8/2000 15:05'!
convertToBuffered

	canvas purgeOutputQueue.
	canvas := canvas asBufferedCanvas.! !

!NebraskaClient methodsFor: 'as yet unclassified' stamp: 'mir 6/17/2007 20:06'!
currentStatusString

	(connection isNil or: [connection isConnected not]) ifTrue: [^'nada'].
	^connection remoteSocketAddress hostNumber,
		' - ',
		(self backlog // 1024) printString,'k'! !

!NebraskaClient methodsFor: 'initialization' stamp: 'ar 10/26/2000 14:28'!
destroy
	hand ifNotNil:[hand world ifNotNil:[hand world removeHand: hand]].
	connection ifNotNil:[connection destroy].
	encoder := canvas := hand := connection := nil.! !

!NebraskaClient methodsFor: 'network' stamp: 'ls 4/9/2000 14:43'!
extent: newExtent  depth: newDepth
	encoder extent: newExtent  depth: newDepth! !

!NebraskaClient methodsFor: 'attributes' stamp: 'ls 3/25/2000 22:27'!
hand
	"return the hand this client is controlling"
	^hand! !

!NebraskaClient methodsFor: 'initialization' stamp: 'yo 9/11/2007 17:51'!
initialize: aConnection

	| remoteAddress userPicture |

	connection := aConnection.
	hand := RemoteControlledHandMorph on: (MorphicEventDecoder on: aConnection).
	hand nebraskaClient: self.
	remoteAddress _ connection remoteSocketAddress.
	userPicture _ EToySenderMorph pictureForIPAddress: remoteAddress.
	hand
		userInitials: ((EToySenderMorph nameForIPAddress: remoteAddress) ifNil: ['???'])
		andPicture: (userPicture ifNotNil: [userPicture scaledToSize: 16@20]).
	encoder := CanvasEncoder on: aConnection.
	canvas := RemoteCanvas
		connection: encoder
		clipRect: NebraskaServer extremelyBigRectangle
		transform: MorphicTransform identity! !

!NebraskaClient methodsFor: 'network' stamp: 'ls 3/25/2000 22:25'!
isConnected
	^connection isConnected! !

!NebraskaClient methodsFor: 'network' stamp: 'ls 3/25/2000 22:25'!
processIO
	connection processIO.! !

Object subclass: #NebraskaDebug
	instanceVariableNames: ''
	classVariableNames: 'DEBUG Details'
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NebraskaDebug commentStamp: '<historical>' prior: 0!
BufferedCanvas enabled: false.
BufferedCanvas enabled: true.

NebraskaDebug beginStats
NebraskaDebug showStats
NebraskaDebug stopAndShowAll
NebraskaDebug killStats
StringSocket showRatesSeen
StringSocket clearRatesSeen
NebraskaDebug showAndClearStats: #allStats
NebraskaDebug showAndClearStats: #queuedbufferSizes


CanvasEncoder beginStats
CanvasEncoder showStats
CanvasEncoder killStats
NebraskaDebug showStats: #peerBytesSent
NebraskaDebug showStats: #soundReductionTime
NebraskaDebug showStats: #FormEncodeTimes
NebraskaDebug showStats: #SendReceiveStats
NebraskaDebug showStats: #sendDeltas
NebraskaDebug showStats: #bigImage
NebraskaDebug showStats: #sketch
NebraskaDebug showStats: #addToOutBuf:
----
buffered off, painting 125kb/s, dragging 400kb/s
buffered on, painting 100kb/s, dragging 170kb/s!

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'RAA 8/14/2000 13:41'!
at: queueName add: anArray

	| now |

	DEBUG ifNil: [
		queueName == #sketchZZZ ifFalse: [^self].
		"Details := OrderedCollection new."
		self beginStats.
	].
	(Details notNil and: [Details size < 20]) ifTrue: [
		Details add: thisContext longStack
	].
	now := Time millisecondClockValue.
	DEBUG add: {now},anArray,{queueName}.
! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'RAA 8/14/2000 10:33'!
beginStats

	DEBUG := OrderedCollection new! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'RAA 8/13/2000 19:22'!
killStats

	DEBUG := nil.
! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'nb 6/17/2003 12:25'!
showAndClearStats: queueName

	DEBUG ifNil: [^Beeper beep].
	self 
		showStats: queueName 
		from: DEBUG.
	DEBUG := nil.! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'nb 6/17/2003 12:25'!
showStats

	DEBUG ifNil: [^Beeper beep].
	DEBUG explore.! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'nb 6/17/2003 12:25'!
showStats: queueName

	DEBUG ifNil: [^Beeper beep].
	self 
		showStats: queueName 
		from: DEBUG.
! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'nice 12/27/2009 03:11'!
showStats: queueName from: aCollection

	| answer prevTime |

	prevTime := nil.
	answer := String streamContents: [ :s | 
		s nextPutAll: (aCollection last first - aCollection first first) asStringWithCommas,' ms';cr;cr.
		aCollection withIndexDo: [ :each :index | | xx currTime | 
			(queueName == #allStats or: [queueName == each last]) ifTrue: [
				currTime := each first.
				xx := currTime printString.
				prevTime ifNil: [prevTime := currTime].
				s nextPutAll: index printString,'.  ',
					(xx allButLast: 3),'.',(xx last: 3),' ',(currTime - prevTime) printString,' '.
				s nextPutAll: each allButFirst printString; cr.
				prevTime := currTime.
			].
		]
	].
	StringHolder new 
		contents: answer;
		openLabel: queueName! !

!NebraskaDebug class methodsFor: 'as yet unclassified' stamp: 'nb 6/17/2003 12:25'!
stopAndShowAll

	| prev |

self halt.	"not updated to new format"

	prev := DEBUG.
	DEBUG := nil.
	prev ifNil: [^Beeper beep].
	prev keysAndValuesDo: [ :k :v |
		self showStats: k from: v
	].! !

Stream subclass: #ObjectSocket
	instanceVariableNames: 'socket outBuf outBufIndex outBufSize inBuf inBufIndex inBufLastIndex outObjects inObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Network-ObjectSocket'!

!ObjectSocket commentStamp: 'ls 2/10/2005 21:27' prior: 0!
This is a socket which exchanges medium-level packets instead of low-level bytes.  This class is abstract; see the subclasses for particular kinds of medium-level packets which can be used.!

ObjectSocket subclass: #ArbitraryObjectSocket
	instanceVariableNames: 'encodingOfLastEncodedObject lastEncodedObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Network-ObjectSocket'!

!ArbitraryObjectSocket commentStamp: '<historical>' prior: 0!
A network connection that passes objects instead of bytes.  The objects are encoded with SmartRefStreams.

!

!ArbitraryObjectSocket methodsFor: 'private' stamp: 'ls 4/25/2000 19:18'!
encodeObject: object  into: buffer  startingAt: startIndex
	"encode the given object into the given buffer"
	| encoded |
	encoded := self smartRefStreamEncode: object.
	buffer putInteger32: encoded size at: startIndex.
	buffer replaceFrom: startIndex+4 to: startIndex+4+(encoded size)-1 with: encoded.
! !

!ArbitraryObjectSocket methodsFor: 'private' stamp: 'ls 2/10/2005 20:40'!
inBufSize

	inBuf ifNil: [^0].
	^inBufLastIndex - inBufIndex + 1! !

!ArbitraryObjectSocket methodsFor: 'private' stamp: 'ls 4/25/2000 19:19'!
nextObjectLength
	"read the next object length from inBuf.  Returns nil if less than 4 bytes are available in inBuf"
	self inBufSize < 4 ifTrue: [ ^nil ].

	^inBuf getInteger32: inBufIndex! !

!ArbitraryObjectSocket methodsFor: 'private' stamp: 'sd 11/20/2005 21:25'!
processInput
	"recieve some data"
	| inObjectData |

	[ socket dataAvailable ] whileTrue: [
		"read as much data as possible"
		self addToInBuf: socket receiveAvailableData.


		"decode as many objects as possible"
		[self nextObjectLength ~~ nil and: [ self nextObjectLength <= (self inBufSize + 4) ]] whileTrue: [
			"a new object has arrived"
			inObjectData := inBuf copyFrom: (inBufIndex + 4) to: (inBufIndex + 3 + self 	nextObjectLength).
			inBufIndex := inBufIndex + 4 + self nextObjectLength.
	
			inObjects addLast: (RWBinaryOrTextStream with: inObjectData) reset fileInObjectAndCode ].

		self shrinkInBuf. ].! !

!ArbitraryObjectSocket methodsFor: 'private' stamp: 'ls 4/25/2000 19:33'!
smartRefStreamEncode: anObject
	| encodingStream |
	"encode an object using SmartRefStream"

	anObject == lastEncodedObject ifTrue: [
		^encodingOfLastEncodedObject ].


	encodingStream := RWBinaryOrTextStream on: ''.
	encodingStream reset.
	(SmartRefStream on: encodingStream) nextPut: anObject.
	
	lastEncodedObject := anObject.
	encodingOfLastEncodedObject := encodingStream contents.

	^encodingOfLastEncodedObject! !

!ArbitraryObjectSocket methodsFor: 'private' stamp: 'ls 4/25/2000 19:36'!
spaceToEncode: anObject
	"return the number of characters needed to encode the given object"
	^ 4 + (self smartRefStreamEncode: anObject) size! !

!ObjectSocket class methodsFor: 'as yet unclassified' stamp: 'RAA 7/20/2000 15:47'!
on: aSocket

	^self basicNew initialize: aSocket! !

!ObjectSocket methodsFor: 'encoding/decoding' stamp: 'sd 11/20/2005 21:26'!
addToInBuf: aString

	| newAlloc |
	newAlloc := aString size * 2 max: 8000.
	inBuf ifNil: [
		inBuf := String new: newAlloc.
		inBufIndex := 1.
		inBufLastIndex := 0.
	].
	aString size > (inBuf size - inBufLastIndex) ifTrue: [
		inBuf := inBuf , (String new: newAlloc)
	].
	inBuf 
		replaceFrom: inBufLastIndex + 1 
		to: inBufLastIndex + aString size
		with: aString 
		startingAt: 1.
	inBufLastIndex := inBufLastIndex + aString size.
! !

!ObjectSocket methodsFor: 'as yet unclassified' stamp: 'RAA 8/2/2000 12:01'!
destroy
	socket destroy.
	socket := nil.! !

!ObjectSocket methodsFor: 'private-initialization' stamp: 'sd 11/20/2005 21:26'!
initialize: aSocket
	socket := aSocket.
	inBuf := String new: 1000.
	inBufIndex := 1.
	inBufLastIndex := 0.

	outBuf := nil.

	inObjects := OrderedCollection new.
	outObjects := OrderedCollection new.
! !

!ObjectSocket methodsFor: 'as yet unclassified' stamp: 'RAA 7/20/2000 15:10'!
isConnected

	^socket notNil and: [socket isConnected]! !

!ObjectSocket methodsFor: 'stream protocol' stamp: 'ls 4/25/2000 18:48'!
next
	^inObjects removeFirst	! !

!ObjectSocket methodsFor: 'stream protocol' stamp: 'ls 4/25/2000 18:48'!
nextOrNil
	inObjects isEmpty
		ifTrue: [ ^nil ]
		ifFalse: [ ^inObjects removeFirst ]! !

!ObjectSocket methodsFor: 'stream protocol' stamp: 'nice 3/19/2010 19:06'!
nextPut: anObject
	^outObjects addLast: anObject! !

!ObjectSocket methodsFor: 'as yet unclassified' stamp: 'ls 1/8/1999 16:14'!
processIO
	"do some as much network IO as possible"

	self processOutput.
	self processInput.! !

!ObjectSocket methodsFor: 'encoding/decoding' stamp: 'sd 11/20/2005 21:26'!
processOutput
	"loop sending data as long as there is data to send, and the socket is ready to receive more data"
	[ socket sendDone and: [ outBuf notNil or: [ outObjects isEmpty not ] ] ] whileTrue: [
		| amountSent |

		outBuf isNil ifTrue: [
			| nextSize |
			"no data in the current buffer; make a new buffer and encode some more"
			outBuf := String new: ((self spaceToEncode: outObjects first) max: 8000).
			outBufIndex := 1.
			outBufSize := 0.

			[	outObjects isEmpty not and: [
					nextSize := self spaceToEncode: outObjects first.
					nextSize <= (outBuf size - outBufSize + 1) ]
			] whileTrue: [
				self encodeObject: outObjects first into: outBuf startingAt: outBufSize+1.
				outBufSize := outBufSize + nextSize.
				outObjects removeFirst ] ].

		"at this point, the buffer definitely has data in it to send.  Send some"
		amountSent := socket sendSomeData: outBuf startIndex: outBufIndex count: (outBufSize - outBufIndex + 1).
		outBufIndex := outBufIndex + amountSent.
		outBufIndex > outBufSize ifTrue: [ outBuf := nil ]  ].

! !

!ObjectSocket methodsFor: 'as yet unclassified' stamp: 'RAA 8/4/2000 15:38'!
remoteAddress

	self isConnected ifFalse: [^nil].
	^socket remoteAddress! !

!ObjectSocket methodsFor: 'encoding/decoding' stamp: 'sd 11/20/2005 21:26'!
shrinkInBuf

	inBuf ifNil: [^self].
	inBufLastIndex < inBufIndex ifTrue: [
		inBufLastIndex := 0.
		inBufIndex := 1.
		inBuf size > 20000 ifTrue: [inBuf := nil].	"if really big, kill it"
		^self
	].
	inBuf := inBuf copyFrom: inBufIndex to: inBufLastIndex.
	inBufLastIndex := inBuf size.
	inBufIndex := 1.

! !

ObjectSocket subclass: #StringSocket
	instanceVariableNames: 'numStringsInNextArray stringsForNextArray nextStringSize files startTime stringCounter socketWriterProcess outputQueue bytesInOutputQueue extraUnsentBytes transmissionError readBuffer'
	classVariableNames: 'MaxRatesSeen RecentSendHistory RunningSendCount'
	poolDictionaries: ''
	category: 'Nebraska-Network-ObjectSocket'!

!StringSocket commentStamp: 'ls 8/4/2004 15:15' prior: 0!
This is a socket which sends arrays of strings back and forth.  This is less convenient than ObjectSockets, but it is more secure and it makes it easier to reason about updates to the protocol.

An array of strings is represented on the network as:

	4-bytes		number of strings in the array
	4-byte		number of bytes in the first string
	n1-bytes		characters in the first string
	4-bytes		number of bytes in the second string
	n2-bytes	characters in the second string
	...

!

!StringSocket class methodsFor: 'as yet unclassified' stamp: 'RAA 8/14/2000 14:02'!
clearRatesSeen
"
StringSocket clearRatesSeen
"
	MaxRatesSeen := nil ! !

!StringSocket class methodsFor: 'as yet unclassified' stamp: 'RAA 7/21/2000 09:37'!
compareFiles
"
StringSocket compareFiles
"
	| data1 data2 |

	data1 := (FileStream fileNamed: 'Macintosh HD:bob:nebraska test:58984048.1')
			contentsOfEntireFile.
	data2 := (FileStream fileNamed: 'BobsG3:squeak:dsqueak:DSqueak2.7 folder:58795431.3')
			contentsOfEntireFile.
	1 to: (data1 size min: data2 size) do: [ :i |
		(data1 at: i) = (data2 at: i) ifFalse: [self halt].
	].
! !

!StringSocket class methodsFor: 'as yet unclassified' stamp: 'nice 10/21/2009 00:14'!
showRatesSeen
"
StringSocket showRatesSeen
"
	| answer |

	MaxRatesSeen ifNil: [^Beeper beep].
	answer := WriteStream on: String new.
	MaxRatesSeen keys asArray sort do: [ :key |
		answer nextPutAll: key printString,'  ',((MaxRatesSeen at: key) // 10000) printString; cr
	].
	StringHolder new contents: answer contents; openLabel: 'send rates at 10 second intervals'.! !

!StringSocket methodsFor: 'private-IO' stamp: 'yo 10/10/2005 18:49'!
addToInBuf: aString

	| newAlloc |
	newAlloc := aString size * 2 max: 80000.
	inBuf ifNil: [
		inBuf := String new: newAlloc.
		inBufIndex := 1.
		inBufLastIndex := 0.
	].
	aString size > (inBuf size - inBufLastIndex) ifTrue: [
		inBuf := inBuf , (String new: newAlloc)
	].
	inBuf 
		replaceFrom: inBufLastIndex + 1 
		to: inBufLastIndex + aString size
		with: aString 
		startingAt: 1.
	inBufLastIndex := inBufLastIndex + aString size.
! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 7/31/2000 17:24'!
addToOutBuf: arrayToWrite

	| size newAlloc |
	size := self spaceToEncode: arrayToWrite.
	newAlloc := size * 2 max: 8000.	"gives us room to grow"
	outBuf ifNil: [
		outBuf := String new: newAlloc.
		outBufIndex := 1.
	].
	outBuf size - outBufIndex + 1 < size ifTrue: [
		outBuf := outBuf , (String new: newAlloc).
	].
	CanvasEncoder at: 1 count: arrayToWrite size + 1.
	outBuf putInteger32: arrayToWrite size at: outBufIndex.
	outBufIndex := outBufIndex + 4.
	arrayToWrite do: [ :each |
		outBuf putInteger32: each size at: outBufIndex.
		outBufIndex := outBufIndex + 4.
		outBuf 
			replaceFrom: outBufIndex 
			to: outBufIndex + each size - 1 
			with: each 
			startingAt: 1.
		outBufIndex := outBufIndex + each size.
	].
	^size! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 12/13/2000 08:29'!
backlog

	^bytesInOutputQueue + extraUnsentBytes! !

!StringSocket methodsFor: 'as yet unclassified' stamp: 'RAA 12/12/2000 19:42'!
destroy

	socketWriterProcess ifNotNil: [socketWriterProcess terminate. socketWriterProcess := nil].
	outputQueue := nil.
	bytesInOutputQueue := 0.
	socket ifNotNil: [socket destroy. socket := nil.].
! !

!StringSocket methodsFor: 'private-IO' stamp: 'sd 11/20/2005 21:26'!
gotSomething

	numStringsInNextArray ifNil: [^self tryForNumStringsInNextArray ].
	numStringsInNextArray = 0 ifTrue: [
		inObjects add: #().
		numStringsInNextArray := nil.
		^true ].
	nextStringSize ifNil: [^ self tryForNextStringSize ].
	^self tryForString
! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 7/20/2000 15:55'!
inBufNext: anInteger
	
	| answer |
	answer := inBuf copyFrom: inBufIndex to: inBufIndex + anInteger - 1.
	inBufIndex := inBufIndex + anInteger.
	^answer! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 7/20/2000 15:55'!
inBufSize

	inBuf ifNil: [^0].
	^inBufLastIndex - inBufIndex + 1! !

!StringSocket methodsFor: 'as yet unclassified' stamp: 'RAA 12/14/2000 09:46'!
initialize: aSocket

	transmissionError := false.
	super initialize: aSocket.
	outputQueue := SharedQueue new.
	extraUnsentBytes := bytesInOutputQueue := 0.
	socketWriterProcess := [
		[self transmitQueueNext] whileTrue.
		socketWriterProcess := nil.
		outputQueue := nil.
		bytesInOutputQueue := 0.
	] forkAt: Processor lowIOPriority.! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 12/14/2000 09:54'!
isConnected

	^super isConnected and: [socketWriterProcess notNil]! !

!StringSocket methodsFor: 'private-IO' stamp: 'md 2/24/2006 19:51'!
nextPut: anObject

	socketWriterProcess ifNil: [^self].
	outObjects addLast: anObject.
	"return the argument - added by kwl"
	^ anObject! !

!StringSocket methodsFor: 'private-IO' stamp: 'yo 10/10/2005 18:47'!
processInput
	| totalReceived chunkOfData |
	"do as much input as possible"

	self flag: #XXX.  "should have resource limits here--no more than X objects and Y bytes"

	chunkOfData := socket receiveAvailableDataIntoBuffer: self readBuffer.
	self addToInBuf: chunkOfData.
	totalReceived := chunkOfData size.

	totalReceived > 0 ifTrue: [
		NebraskaDebug at: #SendReceiveStats add: {'GET'. totalReceived}.
	].

	[ self gotSomething ] whileTrue: [].		"decode as many string arrays as possible"

	self shrinkInBuf.! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 12/14/2000 09:46'!
processIO
	"do some as much network IO as possible"

	socketWriterProcess ifNil: [^self].
	self processOutput.
	self processInput.! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 8/14/2000 14:00'!
processOutput

	| arrayToWrite size bytesSent timeStartSending t itemsSent now timeSlot bucketAgeInMS bytesThisSlot |

	outBufIndex := 1.
	itemsSent := bytesSent := 0.
	timeStartSending := Time millisecondClockValue.
	[outObjects isEmpty not and: [self isConnected]] whileTrue: [
		arrayToWrite := outObjects removeFirst.
		size := self addToOutBuf: arrayToWrite.
		bytesSent := bytesSent + size.
		itemsSent := itemsSent + 1.
		outBufIndex > 10000 ifTrue: [self queueOutBufContents].
	].
	outBufIndex > 1 ifTrue: [self queueOutBufContents].
	bytesSent > 0 ifTrue: [
		MaxRatesSeen ifNil: [MaxRatesSeen := Dictionary new].
		now := Time millisecondClockValue.
		t := now - timeStartSending.
		timeSlot := now // 10000.	"ten second buckets"
		bucketAgeInMS := now \\ 10.
		bytesThisSlot := (MaxRatesSeen at: timeSlot ifAbsent: [0]) + bytesSent.
		MaxRatesSeen 
			at: timeSlot 
			put: bytesThisSlot.
		NebraskaDebug 
			at: #SendReceiveStats 
			add: {'put'. bytesSent. t. itemsSent. bytesThisSlot // (bucketAgeInMS max: 100)}.
	].
! !

!StringSocket methodsFor: 'private-IO' stamp: 'nice 3/31/2011 01:18'!
purgeOutputQueue

	bytesInOutputQueue := 0.
	[outputQueue nextOrNil == nil] whileFalse.! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 12/12/2000 19:42'!
queueOutBufContents

	bytesInOutputQueue := bytesInOutputQueue + outBufIndex - 1.
	outputQueue nextPut: {outBuf. outBufIndex - 1}.
	NebraskaDebug at: #queuedbufferSizes add: {outBufIndex - 1}.
	outBufIndex := 1.
	outBuf := String new: 11000.
	
! !

!StringSocket methodsFor: 'as yet unclassified' stamp: 'yo 10/10/2005 18:47'!
readBuffer

	^ readBuffer ifNil: [readBuffer := String new: 20000].
! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 12/13/2000 08:29'!
sendDataCautiously: aStringOrByteArray bytesToSend: bytesToSend
	"Send all of the data in the given array, even if it requires multiple calls to send it all. Return the number of bytes sent. Try not to send too much at once since this seemed to cause problems talking to a port on the same machine"

	| bytesSent count |

	bytesSent := 0.
	[bytesSent < bytesToSend] whileTrue: [
		extraUnsentBytes := bytesToSend - bytesSent.
		count := socket 
			sendSomeData: aStringOrByteArray 
			startIndex: bytesSent + 1  
			count: (bytesToSend - bytesSent min: 6000).
		bytesSent := bytesSent + count.
		(Delay forMilliseconds: 1) wait.
	].
	extraUnsentBytes := 0.
	^ bytesSent
! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 7/22/2000 09:06'!
shrinkInBuf

	inBuf ifNil: [^self].
	inBufLastIndex < inBufIndex ifTrue: [
		inBufLastIndex := 0.
		inBufIndex := 1.
		inBuf size > 20000 ifTrue: [inBuf := nil].	"if really big, kill it"
		^self
	].
	inBuf := inBuf copyFrom: inBufIndex to: inBufLastIndex.
	inBufLastIndex := inBuf size.
	inBufIndex := 1.

! !

!StringSocket methodsFor: 'private-IO' stamp: 'ls 4/25/2000 18:36'!
spaceToEncode: anArray
	"return the number of characters needed to encode the given string array"
	^anArray inject: 4 into: [ :sum :array |
		sum + (array size + 4) ].! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 12/14/2000 09:42'!
transmitQueueNext

	| bufTuple |

	bufTuple := outputQueue next.
	bytesInOutputQueue := bytesInOutputQueue - bufTuple second max: 0.
	[
		self 
			sendDataCautiously: bufTuple first 
			bytesToSend: bufTuple second.
	]
		on: Error
		do: [ :ex |
			transmissionError := true.
		].
	^transmissionError not

! !

!StringSocket methodsFor: 'private-IO' stamp: 'sd 11/20/2005 21:26'!
tryForNextStringSize
	"grab the size of the next string, if it's available"

	self inBufSize >= 4 ifFalse: [^false].

	nextStringSize := inBuf getInteger32: inBufIndex.
	"nextStringSize > 100000 ifTrue: [self barf]."
	inBufIndex := inBufIndex + 4.
	
	nextStringSize < 0 ifTrue: [
		socket disconnect.
		^false ].
	
	^true
! !

!StringSocket methodsFor: 'private-IO' stamp: 'sd 11/20/2005 21:26'!
tryForNumStringsInNextArray
	"input numStringsInNextARray, if 4 bytes are available"

	self inBufSize >= 4 ifFalse: [^false].

	numStringsInNextArray := inBuf getInteger32: inBufIndex.
	"(numStringsInNextArray > 100 or: [numStringsInNextArray < 1]) ifTrue: [self barf]."
	inBufIndex := inBufIndex + 4.

	numStringsInNextArray < 0 ifTrue: [
		socket disconnect.
		^false ].
	
	stringsForNextArray := Array new: numStringsInNextArray.
	stringCounter := 0.
	nextStringSize := nil. 
	^true! !

!StringSocket methodsFor: 'private-IO' stamp: 'RAA 7/22/2000 09:04'!
tryForString
	"try to grab an actual string"

	self inBufSize >= nextStringSize ifFalse: [^false].

	stringsForNextArray 
		at: (stringCounter := stringCounter + 1)
		put: (self inBufNext: nextStringSize) asString.

	stringCounter = numStringsInNextArray ifTrue: [	"we have finished another array!!"
		inObjects addLast: stringsForNextArray.
		stringCounter := stringsForNextArray := numStringsInNextArray := nextStringSize := nil.
	] ifFalse: [	"still need more strings for this array"
		nextStringSize := nil.
	].

	^true
! !

!StrikeFont class methodsFor: '*nebraska-instance creation' stamp: 'yo 12/17/2005 22:41'!
decodedFromRemoteCanvas: aString

	| array style base |
	array := aString findTokens: #($ ).
	style := TextStyle named: (array at: 1) asSymbol.
	style ifNil: [^ TextStyle defaultFont].
	(style fontArray first name = style fontArray first name withoutTrailingDigits) ifTrue: [
			^ self familyName: (array at: 1) size: (array at: 3) asNumber emphasized: (array at: 4) asNumber].
	base := style fontArray detect: [:f | (array at: 2) beginsWith: f name].
	^ base emphasized: (array at: 4) asNumber.

	"^ self familyName: (array at: 1) size: (array at: 2) asNumber emphasized: (array at: 3) asNumber."
! !

!StrikeFont methodsFor: '*nebraska-file in/out' stamp: 'ul 5/16/2011 01:11'!
encodedForRemoteCanvas

	^String streamContents: [ :stream |
		stream
			nextPutAll: self familyName;
			space;
			nextPutAll: self name;
			space;
			print: self height;
			space;
			print: self emphasis ]! !

!StrikeFontSet methodsFor: '*nebraska-file in/out' stamp: 'ul 5/16/2011 01:11'!
encodedForRemoteCanvas

	^String streamContents: [ :stream |
		stream
			nextPutAll: self familyName;
			space;
			print: self pointSize;
			space;
			print: self emphasis ]! !

!Form methodsFor: '*nebraska-encoding' stamp: 'RAA 7/29/2000 09:01'!
addDeltasFrom: previousForm

	(BitBlt 
		destForm: self 
		sourceForm: previousForm 
		fillColor: nil 
		combinationRule: Form reverse
		destOrigin: 0@0
		sourceOrigin: 0@0
		extent: self extent 
		clipRect: self boundingBox) copyBits.
	^self! !

!Form methodsFor: '*nebraska-encoding' stamp: 'RAA 7/29/2000 09:01'!
deltaFrom: previousForm

	| newForm |
	newForm := previousForm deepCopy.
	(BitBlt 
		destForm: newForm 
		sourceForm: self 
		fillColor: nil 
		combinationRule: Form reverse 
		destOrigin: 0@0
		sourceOrigin: 0@0
		extent: self extent 
		clipRect: self boundingBox) copyBits.
	^newForm! !

!Form methodsFor: '*nebraska-encoding' stamp: 'RAA 8/1/2000 06:15'!
deltaFrom: smallerForm at: offsetInMe

	| newForm |
	newForm := smallerForm deepCopy.
	(BitBlt 
		destForm: newForm 
		sourceForm: self 
		fillColor: nil 
		combinationRule: Form reverse 
		destOrigin: 0@0
		sourceOrigin: offsetInMe
		extent: smallerForm extent 
		clipRect: newForm boundingBox) copyBits.
	^newForm! !

!Form methodsFor: '*nebraska-encoding' stamp: 'RAA 8/13/2000 15:32'!
encodeForRemoteCanvas
	| header binaryForm |
	"encode into a bitstream for use with RemoteCanvas.  The format does not require invoking the Compiler"
	header := String streamContents: [ :str |
	str "nextPutAll: 'F|';"
		nextPutAll: self depth printString;
		nextPut: $,;
		nextPutAll: self width printString;
		nextPut: $,;
		nextPutAll: self height printString;
		nextPut: $|. ].

	binaryForm := ByteArray streamContents: [ :str |
		self unhibernate.
		bits writeOn: str. ].

	^header, binaryForm asString
! !

!DisplayTransform class methodsFor: '*nebraska-instance creation' stamp: 'ls 3/19/2000 16:44'!
fromRemoteCanvasEncoding: encoded
	| type |
	"decode a transform from the given encoded string"
	type := (ReadStream on: encoded) upTo: $,.
	type = 'Morphic' ifTrue: [
		^MorphicTransform fromRemoteCanvasEncoding: encoded ].
	type = 'Matrix' ifTrue: [
		^MatrixTransform2x3 fromRemoteCanvasEncoding: encoded ].
	type = 'Composite' ifTrue: [
		^CompositeTransform fromRemoteCanvasEncoding: encoded ].
	^self error: 'invalid transform encoding'! !

!DisplayTransform methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ls 10/9/1999 18:56'!
encodeForRemoteCanvas
	"encode this transform into a string for use by a RemoteCanvas"
	^self subclassResponsibility! !

!MorphicTransform class methodsFor: '*nebraska-instance creation' stamp: 'sd 11/20/2005 21:25'!
fromRemoteCanvasEncoding: encoded
	"DisplayTransform fromRemoteCanvasEncoding:  'Morphic,-88,-128,1.345165663873898,0.1352584843149221'"
	| type offsetXEnc offsetYEnc scaleEnc angleEnc offsetX offsetY angle scale rs |

	"separate the numbers"
	rs := ReadStream on: encoded.
	type := rs upTo: $,.
	offsetXEnc := rs upTo: $,.
	offsetYEnc := rs upTo: $,.
	scaleEnc := rs upTo: $,.
	angleEnc := rs upToEnd.

	"decode the numbers"
	offsetX := Integer readFromString: offsetXEnc.
	offsetY := Integer readFromString: offsetYEnc.

	scale := Number readFromString: scaleEnc.
	angle := Number readFromString: angleEnc.

	"create an instance"
	^self offset: offsetX@offsetY angle: angle scale: scale! !

!MorphicTransform methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'yo 9/13/2007 11:21'!
encodeForRemoteCanvas
	"encode this transform into a string for use by a RemoteCanvas"
	^String streamContents: [ :str |
		str nextPutAll: 'Morphic,';
			print: offset x truncated;
			nextPut: $,;
			print: offset y truncated;
			nextPut: $,;
			print: scale asFloat;
			nextPut: $,;
			print: angle asFloat
	]! !

HandMorph subclass: #RemoteControlledHandMorph
	instanceVariableNames: 'eventDecoder viewExtent nebraskaClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!RemoteControlledHandMorph commentStamp: '<historical>' prior: 0!
Used as part of the Nebraska system.  It is controlled by commands sent through a socket.  The encoding is interpreted via a MorphicEventDecoder.!

!RemoteControlledHandMorph class methodsFor: 'instance creation' stamp: 'nk 7/30/2004 18:10'!
on: aDecoder 
	^self new  decoder: aDecoder! !

!RemoteControlledHandMorph methodsFor: 'events' stamp: 'RAA 11/8/2000 15:15'!
convertRemoteClientToBuffered

	self world convertRemoteClientToBuffered: nebraskaClient! !

!RemoteControlledHandMorph methodsFor: 'initialization' stamp: 'ls 3/24/2000 21:53'!
decoder: aDecoder
	eventDecoder := aDecoder! !

!RemoteControlledHandMorph methodsFor: 'initialization' stamp: 'ls 3/25/2000 16:44'!
initialize
	super initialize.
	viewExtent := 100@100.! !

!RemoteControlledHandMorph methodsFor: 'initialization' stamp: 'RAA 11/8/2000 15:13'!
nebraskaClient: aNebraskaClient

	nebraskaClient := aNebraskaClient! !

!RemoteControlledHandMorph methodsFor: 'drawing' stamp: 'RAA 12/12/2000 14:45'!
needsToBeDrawn

	^true! !

!RemoteControlledHandMorph methodsFor: 'event handling' stamp: 'ls 3/25/2000 16:51'!
processEvents
	| |
	eventDecoder processIO.
	eventDecoder applyMessagesTo: self.
! !

!RemoteControlledHandMorph methodsFor: 'events' stamp: 'ar 10/25/2000 23:29'!
queueEvent: anEvent
	"add an event to be handled"

	anEvent setHand: self.
	self handleEvent: anEvent resetHandlerFields.! !

!RemoteControlledHandMorph methodsFor: 'events' stamp: 'ls 3/25/2000 16:44'!
setViewExtent: newExtent
	"set the extent of this hand's view of the world"
	viewExtent := newExtent! !

!RemoteControlledHandMorph methodsFor: 'geometry' stamp: 'ls 3/25/2000 16:44'!
worldBounds
	^0@0 extent: viewExtent! !

!CompositeTransform class methodsFor: '*nebraska-instance creation' stamp: 'ls 3/19/2000 16:49'!
fromRemoteCanvasEncoding: encoding
	| firstStart firstEnd firstEncoding firstTransform secondStart secondEnd secondEncoding secondTransform |
	"format: Composite,(enc1)(enc2)"

	"decode the first encoding"
	firstStart := encoding indexOf: $(.
	firstStart = 0 ifTrue: [ self error: 'invalid encoding' ].
	firstEnd := encoding findCloseParenthesisFor: firstStart.
	firstEncoding := encoding copyFrom: firstStart+1 to: firstEnd-1.
	firstTransform := DisplayTransform fromRemoteCanvasEncoding: firstEncoding.

	"decode the second encoding"
	secondStart := firstEnd + 1.
	(encoding at: secondStart) = $( ifFalse: [ ^self error: 'invalid encoding' ].
	secondEnd := encoding findCloseParenthesisFor: secondStart.
	secondEncoding := encoding copyFrom: secondStart+1 to: secondEnd-1.
	secondTransform := DisplayTransform fromRemoteCanvasEncoding: secondEncoding.
	

	"put it together"
	^self globalTransform: firstTransform localTransform: secondTransform! !

!CompositeTransform methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ls 3/19/2000 16:28'!
encodeForRemoteCanvas
	^String streamContents: [ :str |
		str
			nextPutAll: 'Composite,';
			nextPutAll: '(';
			nextPutAll: globalTransform encodeForRemoteCanvas;
			nextPutAll: ')(';
			nextPutAll: localTransform encodeForRemoteCanvas;
			nextPutAll: ')' ]! !

!GradientFillStyle methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 7/28/2000 08:37'!
encodeForRemoteCanvas

	^(DataStream streamedRepresentationOf: self) asString
! !

!Point methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 7/31/2000 17:27'!
encodeForRemoteCanvas

	| encoded |

	CanvasEncoder at: 3 count:  1.
	encoded := String new: 8.
	encoded putInteger32: x asInteger at: 1.
	encoded putInteger32: y asInteger at: 5.
	^encoded! !

EllipseMorph subclass: #AlertMorph
	instanceVariableNames: 'onColor offColor myObjSock socketOwner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Audio Chat'!

!AlertMorph methodsFor: 'visual properties' stamp: 'TBP 3/5/2000 13:47'!
canHaveFillStyles
	^false! !

!AlertMorph methodsFor: 'accessing' stamp: 'TBP 3/5/2000 13:47'!
color: aColor

	super color: aColor.
	onColor := aColor.! !

!AlertMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:11'!
defaultBorderWidth
"answer the default border width for the receiver"
	^ 2! !

!AlertMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:11'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color red! !

!AlertMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:11'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	self extent: 25 @ 25.
! !

!AlertMorph methodsFor: 'accessing' stamp: 'mir 8/31/2004 15:47'!
onColor
	^onColor ifNil: [onColor := Color green]! !

!AlertMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/2/2000 10:39'!
socketOwner: aChatGUI

	socketOwner := aChatGUI.! !

!AlertMorph methodsFor: 'stepping and presenter' stamp: 'mir 8/31/2004 15:47'!
step

	super step.
	offColor ifNil: [offColor := self onColor mixed: 0.5 with: Color black].
	socketOwner objectsInQueue = 0 ifTrue: [
		color = offColor ifFalse: [super color: offColor].
	] ifFalse: [
		super color: (color = onColor ifTrue: [offColor] ifFalse: [onColor]).
	].
! !

!AlertMorph methodsFor: 'testing' stamp: 'TBP 3/5/2000 13:47'!
stepTime
	"Answer the desired time between steps in milliseconds."

	^ 500! !

!Rectangle methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 7/31/2000 17:25'!
encodeForRemoteCanvas

	| encoded |

	CanvasEncoder at: 2 count:  1.
	encoded := String new: 16.
	encoded putInteger32: origin x asInteger at: 1.
	encoded putInteger32: origin y asInteger at: 5.
	encoded putInteger32: corner x asInteger at: 9.
	encoded putInteger32: corner y asInteger at: 13.

	^encoded! !

!Rectangle methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 7/31/2000 16:39'!
encodeForRemoteCanvasB

	| encoded |

	encoded := Bitmap new: 4.
	encoded at: 1 put: origin x asInteger.
	encoded at: 2 put: origin y asInteger.
	encoded at: 3 put: corner x asInteger.
	encoded at: 4 put: corner y asInteger.

	^encoded! !

!KeyboardEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'dgd 2/22/2003 18:53'!
decodeFromStringArray: array 
	"decode the receiver from an array of strings"

	type := array first asSymbol.
	position := CanvasDecoder decodePoint: (array second).
	buttons := CanvasDecoder decodeInteger: (array third).
	keyValue := CanvasDecoder decodeInteger: array fourth! !

!KeyboardEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ar 10/9/2000 00:14'!
encodedAsStringArray
	"encode the receiver into an array of strings, such that it can be retrieved via the fromStringArray: class method"
	^{
		type.
		CanvasEncoder encodePoint: position.
		CanvasEncoder encodeInteger: buttons.
		CanvasEncoder encodeInteger: keyValue asInteger
	}! !

!ColorForm methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 7/24/2000 13:32'!
encodeForRemoteCanvas

	"encode into a bitstream for use with RemoteCanvas."

	| colorsToSend |

	colorsToSend := self colors.
	^String streamContents: [ :str |
		str
			nextPut: $C;		"indicates color form"
			nextPutAll: colorsToSend size printString;
			nextPut: $,.
		colorsToSend do: [ :each |
			str nextPutAll: each encodeForRemoteCanvas
		].
		str nextPutAll: super encodeForRemoteCanvas
	].

! !

StringHolder subclass: #ChatNotes
	instanceVariableNames: 'name notesIndex names notes recorder player sound isPlaying isRecording isSaving nameTextMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Audio Chat'!

!ChatNotes class methodsFor: 'instance creation' stamp: 'RAA 8/2/2000 01:06'!
openAsMorph

	^self new openAsMorph! !

!ChatNotes methodsFor: 'file i/o' stamp: 'mir 11/27/2001 12:04'!
audioDirectory

	(FileDirectory default directoryExists: 'audio')
		ifFalse: [FileDirectory default createDirectory: 'audio'].
	^FileDirectory default directoryNamed: 'audio'! !

!ChatNotes methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 1 g: 0.7 b: 0.8)! !

!ChatNotes methodsFor: 'file i/o' stamp: 'RAA 8/1/2000 18:08'!
deleteSelection
	"Delete the selection in the list"
	| dir |

	notesIndex <= 0 ifTrue: [^self].
	dir := self audioDirectory.
	dir deleteFileNamed: ((notes at: notesIndex), 'name') ifAbsent: [].
	dir deleteFileNamed: ((notes at: notesIndex), 'aiff') ifAbsent: [].
	names removeAt: notesIndex.
	notes removeAt: notesIndex.
	self notesListIndex: 0.
	self changed: #notesList.
	self changed: #name.! !

!ChatNotes methodsFor: 'file i/o' stamp: 'RAA 8/1/2000 18:09'!
getNextName
	"Return the next name available.
	All names are of the form '#.name' and '#.aiff'."
	| dir num |

	dir := self audioDirectory.
	num := 1.
	[dir fileExists: (num asString, '.name')] whileTrue: [num := num + 1].
	^(num asString, '.')! !

!ChatNotes methodsFor: 'morphic' stamp: 'TBP 2/23/2000 21:07'!
initialExtent
	"Nice and small--that was the idea.
	It shouldn't take up much screen real estate."
	^200@100! !

!ChatNotes methodsFor: 'initialization' stamp: 'RAA 8/1/2000 18:05'!
initialize

	self loadNotes.
	notesIndex := 0.
	recorder := ChatRecorder new.
	recorder initialize.! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:03'!
isPlaying

	^isPlaying ifNil: [isPlaying := false]! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:04'!
isPlaying: aBoolean

	isPlaying = aBoolean ifTrue: [^self].
	isPlaying := aBoolean.
	self changed: #isPlaying	! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:04'!
isRecording

	^isRecording ifNil: [isRecording := false]! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:05'!
isRecording: aBoolean
	
	isRecording = aBoolean ifTrue: [^self].
	isRecording := aBoolean.
	self changed: #isRecording	! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:05'!
isSaving

	^isSaving ifNil: [isSaving := false]! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:05'!
isSaving: aBoolean

	isSaving = aBoolean ifTrue: [^self].
	isSaving := aBoolean.
	self changed: #isSaving! !

!ChatNotes methodsFor: 'testing' stamp: 'RAA 8/1/2000 18:05'!
isStopped

	^false! !

!ChatNotes methodsFor: 'initialization' stamp: 'mir 11/27/2001 12:01'!
loadNotes
	"Load notes from the files"
	| dir |

	names := OrderedCollection new.
	notes := OrderedCollection new.
	(FileDirectory default directoryExists: 'audio')
		ifFalse: [^self].
	dir := self audioDirectory.
	dir fileNames do: [:fname |
		(fname endsWith: '.name') ifTrue: [
			names add: ((dir fileNamed: fname) contentsOfEntireFile).
			notes add: (fname copyFrom: 1 to: (fname size - 4))]].! !

!ChatNotes methodsFor: 'accessing' stamp: 'RAA 8/1/2000 17:59'!
name

	^name ifNil: [name := '']! !

!ChatNotes methodsFor: 'accessing' stamp: 'TBP 2/23/2000 21:07'!
name: aString
	name := aString.
	self changed: #name.! !

!ChatNotes methodsFor: 'accessing' stamp: 'RAA 8/1/2000 18:01'!
notesList
	
	self flag: #why.
	^names copy asArray! !

!ChatNotes methodsFor: 'accessing' stamp: 'RAA 8/1/2000 18:02'!
notesListIndex

	^notesIndex ifNil: [notesIndex := 0]! !

!ChatNotes methodsFor: 'accessing' stamp: 'RAA 8/1/2000 18:02'!
notesListIndex: index
	
	notesIndex := index = notesIndex ifTrue: [0] ifFalse: [index].
	self name: (self notesList at: notesIndex ifAbsent: ['']).
	self changed: #notesListIndex.! !

!ChatNotes methodsFor: 'morphic' stamp: 'TBP 2/23/2000 21:07'!
notesMenu: aMenu
	"Simple menu to delete notes"
	^(notesIndex = 0)
		ifTrue: [aMenu labels: 'update notes' lines: #() selections: #(updateNotes)]
		ifFalse: [aMenu labels: ('delete', String cr, 'update notes') lines: #() selections: #(deleteSelection updateNotes)]! !

!ChatNotes methodsFor: 'initialization' stamp: 'RAA 8/2/2000 01:15'!
openAsMorph
	| window aColor recordButton stopButton playButton saveButton |

	window := (SystemWindow labelled: 'Audio Notes') model: self.

	window addMorph: (
		(PluggableListMorph 
			on: self 
			list: #notesList 
			selected: #notesListIndex 
			changeSelected: #notesListIndex: 
			menu: #notesMenu:
		) autoDeselect: false) frame: (0@0 corner: 0.5@1.0).

	nameTextMorph := PluggableTextMorph on: self text: #name accept: nil.
	nameTextMorph askBeforeDiscardingEdits: false.
	window addMorph: nameTextMorph frame: (0.5@0 corner: 1.0@0.4).

	aColor := Color colorFrom: self defaultBackgroundColor.

	(recordButton := PluggableButtonMorph on: self getState: #isRecording action: #record)
		label: 'record';
		askBeforeChanging: true;
		color: aColor;
		onColor: aColor darker offColor: aColor.
	window addMorph: recordButton frame: (0.5@0.4 corner: 0.75@0.7).

	(stopButton := PluggableButtonMorph on: self getState: #isStopped action: #stop)
		label: 'stop';
		askBeforeChanging: true;
		color: aColor;
		onColor: aColor darker offColor: aColor.
	window addMorph: stopButton frame: (0.75@0.4 corner: 1.0@0.7).

	(playButton := PluggableButtonMorph on: self getState: #isPlaying action: #play)
		label: 'play';
		askBeforeChanging: true;
		color: aColor;
		onColor: aColor darker offColor: aColor.
	window addMorph: playButton frame: (0.5@0.7 corner: 0.75@1.0).

	(saveButton := PluggableButtonMorph on: self getState: #isSaving action: #save)
		label: 'save';
		askBeforeChanging: true;
		color: aColor;
		onColor: aColor darker offColor: aColor.
	window addMorph: saveButton frame: (0.75@0.7 corner: 1.0@1.0).

	window openInWorld.! !

!ChatNotes methodsFor: 'file i/o' stamp: 'RAA 8/1/2000 19:05'!
play
	| separator |
	self isPlaying: true.
	notesIndex = 0 ifTrue: [
		recorder pause.
		recorder playback.
		self isPlaying: false.
		^self
	].
	separator := FileDirectory pathNameDelimiter asString.
	sound := (AIFFFileReader new readFromFile: (
		FileDirectory default pathName, 
		separator, 'audio', separator, (notes at: notesIndex), 'aiff')) sound.
	[
		sound playAndWaitUntilDone.
		self isPlaying: false
	] fork! !

!ChatNotes methodsFor: 'button commands' stamp: 'RAA 8/1/2000 19:05'!
record

	self isRecording: true.
	notesIndex = 0 ifFalse: [self notesListIndex: 0].
	sound := nil.
	recorder clearRecordedSound.
	recorder resumeRecording.! !

!ChatNotes methodsFor: 'accessing' stamp: 'TBP 2/23/2000 21:07'!
recorder
	^recorder! !

!ChatNotes methodsFor: 'button commands' stamp: 'RAA 8/1/2000 18:03'!
save

	self isSaving: true.
	notesIndex = 0
		ifTrue: [self saveSound]
		ifFalse: [self saveName].
	self isSaving: false.! !

!ChatNotes methodsFor: 'file i/o' stamp: 'RAA 8/2/2000 01:09'!
saveName
	"Save the name to the '.name' file."
	| dir file |

	self name: self textMorphString.
	dir := self audioDirectory.
	file := (notes at: notesIndex), 'name'.
	(dir fileExists: file) ifTrue: [dir deleteFileNamed: file].
	file := dir newFileNamed: file.
	file nextPutAll: name.
	file close.
	names at: notesIndex put: name.
	self changed: #notesList.! !

!ChatNotes methodsFor: 'file i/o' stamp: 'RAA 8/2/2000 01:09'!
saveSound
	"Move the sound from the recorder to the files."
	| fname file |
	
	recorder recordedSound ifNil: [^self].
	self isSaving: true.
	fname := self getNextName.
	"Create .name file"
	file := self audioDirectory newFileNamed: (fname, 'name').
	file nextPutAll: self textMorphString.
	file close.
	"Create .aiff file"
	file := (self audioDirectory newFileNamed: (fname, 'aiff')) binary.
	self storeAIFFOnFile: file.
	file close.
	"Add to names and notes"
	names add: self textMorphString.
	notes add: fname.
	self changed: #notesList.
	self notesListIndex: (notes size).
	"Clear Recorder"
	recorder := SoundRecorder new.
	"Stop Button"
	self isSaving: false! !

!ChatNotes methodsFor: 'button commands' stamp: 'TBP 2/23/2000 21:07'!
stop
	recorder pause.
	self isRecording: false! !

!ChatNotes methodsFor: 'file i/o' stamp: 'nice 12/26/2009 09:35'!
storeAIFFOnFile: file
	"In a better design, this would be handled by SequentialSound,
	but I figure you will need a new primitive anyway, so it can
	be implemented at that time."
	| sampleCount |
	sampleCount := recorder recordedSound sounds inject: 0 into: [ :sum :rsound |
		sum + rsound samples monoSampleCount
	].
	file nextPutAll: 'FORM' asByteArray.
	file nextInt32Put: (2 * sampleCount) + 46.
	file nextPutAll: 'AIFF' asByteArray.
	file nextPutAll: 'COMM' asByteArray.
	file nextInt32Put: 18.
	file nextNumber: 2 put: 1. "channels"
	file nextInt32Put: sampleCount.
	file nextNumber: 2 put: 16. "bits/sample"
	(AbstractSound new) storeExtendedFloat: (recorder samplingRate) on: file.
	file nextPutAll: 'SSND' asByteArray.
	file nextInt32Put: (2 * sampleCount) + 8.
	file nextInt32Put: 0.
	file nextInt32Put: 0.
	(recorder recordedSound sounds) do: [:rsound |
		1 to: (rsound samples monoSampleCount) do: [:i |
			| s |
			s := rsound samples at: i.
			file nextPut: ((s bitShift: -8) bitAnd: 16rFF).
			file nextPut: (s bitAnd: 16rFF)]].! !

!ChatNotes methodsFor: 'morphic' stamp: 'RAA 8/2/2000 01:11'!
textMorphString

	^nameTextMorph text string! !

!ChatNotes methodsFor: 'file i/o' stamp: 'TBP 2/23/2000 21:07'!
updateNotes
	"Probably not necessary unless several audio notes are
	open at the same time"

	"Clear Notes"
	self loadNotes.
	self changed: #notesList.
	self notesListIndex: 0.
	self name: ''.! !

SoundRecorder subclass: #ChatRecorder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Audio Chat'!

!ChatRecorder methodsFor: 'as yet unclassified' stamp: 'RAA 8/11/2000 09:47'!
initialize
	
	"setting a higher desired recording rate seemed to fix certain powerbook problems.
	I'm still trying to understand it all, but there it is for now"

	super initialize.
	samplingRate := 44100.

! !

!ChatRecorder methodsFor: 'as yet unclassified' stamp: 'RAA 8/13/2000 11:34'!
pause
	"Go into pause mode. The record level continues to be updated, but no sound is recorded."

	paused := true.
	((currentBuffer ~~ nil) and: [nextIndex > 1])
		ifTrue: [self emitPartialBuffer.
				self allocateBuffer].

	soundPlaying ifNotNil: [
		soundPlaying pause.
		soundPlaying := nil].

	self stopRecording.

	"Preferences canRecordWhilePlaying ifFalse: [self stopRecording]."
! !

!ChatRecorder methodsFor: 'as yet unclassified' stamp: 'RAA 8/13/2000 11:34'!
playback
	"Playback the sound that has been recorded."

	self pause.
	soundPlaying := self recordedSound ifNil: [^self].
	soundPlaying play.
! !

!ChatRecorder methodsFor: 'accessing' stamp: 'TBP 2/23/2000 20:54'!
recordedSound: aSound

	self clearRecordedSound.
	recordedSound := aSound.! !

!ChatRecorder methodsFor: 'as yet unclassified' stamp: 'RAA 8/13/2000 11:38'!
resumeRecording
	"Continue recording from the point at which it was last paused."

	self startRecording.
	paused := false.
! !

AlignmentMorphBob1 subclass: #NebraskaServerMorph
	instanceVariableNames: 'server slowCounter previousBacklog lastFullUpdateTime currentStatusString fullDisplay previousClients currentBacklogString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NebraskaServerMorph commentStamp: '<historical>' prior: 0!
A cheezy morph that simply steps a Nebraska server instance over and over.!

!NebraskaServerMorph class methodsFor: 'as yet unclassified' stamp: 'RAA 8/15/2000 14:49'!
killOldServers

	NebraskaServerMorph allInstances do: [ :each |
		each delete.
	].
	NebraskaServer allInstances do: [ :each |
		each stopListening.
		DependentsFields removeKey: each ifAbsent: [].
	].
! !

!NebraskaServerMorph class methodsFor: 'as yet unclassified' stamp: 'yo 12/17/2005 23:06'!
serveWorld

	^ self serveWorld: ActiveWorld.
! !

!NebraskaServerMorph class methodsFor: 'as yet unclassified' stamp: 'yo 12/17/2005 23:07'!
serveWorldButton

	| button |
	button := ScriptableButton new.
	button target: NebraskaServerMorph.
	button actionSelector: #serveWorld.
	button arguments: #().
	button label: 'Share'.
	button color: Color yellow.
	^ button.
! !

!NebraskaServerMorph class methodsFor: 'as yet unclassified' stamp: 'bf 7/4/2008 18:16'!
serveWorld: aWorld
	"Check to make sure things won't crash. See Mantis #0000519"
	^aWorld isSafeToServe ifTrue:[
		self serveWorld: aWorld onPort: NebraskaServer defaultPorts]
	! !

!NebraskaServerMorph class methodsFor: 'as yet unclassified' stamp: 'bf 7/4/2008 18:14'!
serveWorld: aWorld onPort: aPortNumber

	| server |
	server := NebraskaServer serveWorld: aWorld onPort: aPortNumber.
	(self new) openInWorld: aWorld.
	^server
	"server acceptNullConnection"		"server acceptPhonyConnection."
! !

!NebraskaServerMorph class methodsFor: 'as yet unclassified' stamp: 'tak 7/25/2007 00:09'!
supplementaryPartsDescriptions
	^ {DescriptionForPartsBin
		formalName: 'NebraskaServer' translatedNoop
		categoryList: #()
		documentation: 'A button to start the Nebraska desktop sharing server' translatedNoop
		globalReceiverSymbol: #NebraskaServerMorph
		nativitySelector: #serveWorldButton
	}! !

!NebraskaServerMorph methodsFor: 'accessing' stamp: 'RAA 5/31/2001 15:03'!
currentBacklogString

	^currentBacklogString! !

!NebraskaServerMorph methodsFor: 'accessing' stamp: 'RAA 11/8/2000 16:07'!
currentStatusString

	^currentStatusString! !

!NebraskaServerMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:29'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color white! !

!NebraskaServerMorph methodsFor: 'submorphs-add/remove' stamp: 'KR 3/25/2008 17:24'!
delete
	self server ifNotNil:[
		(self confirm:'Shutdown the server?' translated) 
			ifTrue:[self world remoteServer: nil]].
	super delete.! !

!NebraskaServerMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 21:29'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	fullDisplay := false.
	
	lastFullUpdateTime := 0.
	self listDirection: #topToBottom;
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap! !

!NebraskaServerMorph methodsFor: 'initialization' stamp: 'kfr 10/21/2011 00:04'!
rebuild

	| myServer toggle closeBox font |

	font _ StrikeFont familyName: #Palatino size: 14.
	self removeAllMorphs.
	self setColorsAndBorder.
	self updateCurrentStatusString.
	toggle _ SimpleHierarchicalListMorph new perform: (
		fullDisplay ifTrue: [#expandedForm] ifFalse: [#notExpandedForm]
	).
	closeBox _ SimpleButtonMorph new borderWidth: 0;
			label: 'X' font: Preferences standardEToysButtonFont; color: Color transparent;
			actionSelector: #delete; target: self; extent: 14@14;
			setBalloonText: 'End Nebraska session' translated.

	self addARow: {
		self inAColumn: {closeBox}.
		self inAColumn: {
			UpdatingStringMorph new
				useStringFormat;
				target:  self;
				font: font;
				getSelector: #currentStatusString;
				contents: self currentStatusString;
				stepTime: 2000;
				lock.
		}.
		self inAColumn: {
			toggle asMorph
				on: #mouseUp send: #toggleFull to: self;
				setBalloonText: 'Show more or less of Nebraska Status' translated
		}.
	}.
	myServer _ self server.
	(myServer isNil or: [fullDisplay not]) ifTrue: [
		^World startSteppingSubmorphsOf: self
	].
	"--- the expanded display ---"
	self addARow: {
		self inAColumn: {
			UpdatingStringMorph new
				useStringFormat;
				target:  self;
				font: font;
				getSelector: #currentBacklogString;
				contents: self currentBacklogString;
				stepTime: 2000;
				lock.
		}.
	}.

	self addARow: {
		self inAColumn: {
			(StringMorph contents: '--clients--' translated) lock; font: font.
		}.
	}.

	myServer clients do: [ :each |
		self addARow: {
			UpdatingStringMorph new
				useStringFormat;
				target: each;
				font: font;
				getSelector: #currentStatusString;
				contents: each currentStatusString;
				stepTime: 2000;
				lock.
		}
	].
	World startSteppingSubmorphsOf: self.! !

!NebraskaServerMorph methodsFor: 'accessing' stamp: 'ar 10/26/2000 14:05'!
server
	^self world remoteServer! !

!NebraskaServerMorph methodsFor: 'initialization' stamp: 'kfr 4/19/2015 12:18'!
setColorsAndBorder
	| worldColor c |
	c := ((Preferences menuColorFromWorld and: [Display depth > 4]) 
				and: [(worldColor := self currentWorld color) isColor]) 
					ifTrue: 
						[worldColor luminance > 0.7 
							ifTrue: [worldColor mixed: 0.8 with: Color black]
							ifFalse: [worldColor mixed: 0.4 with: Color white]]
					ifFalse: [MenuMorph menuColor]. 
	self color: c.
	self borderColor: #raised.
	self borderWidth: MenuMorph menuBorderWidth.
	self useRoundedCorners! !

!NebraskaServerMorph methodsFor: 'stepping and presenter' stamp: 'RAA 12/11/2000 12:19'!
step

	| now |

	self server ifNil: [ ^self ].
	self server step.
	now := Time millisecondClockValue.
	(now - lastFullUpdateTime) abs > 5000 ifTrue: [
		lastFullUpdateTime := now.
		(previousBacklog = self server backlog and: [self server clients = previousClients]) ifFalse: [
			previousClients := self server clients copy.
			self rebuild
		]
	].
! !

!NebraskaServerMorph methodsFor: 'testing' stamp: 'RAA 11/8/2000 15:57'!
stepTime

	^10! !

!NebraskaServerMorph methodsFor: 'initialization' stamp: 'RAA 11/8/2000 16:13'!
toggleFull

	fullDisplay := fullDisplay not.
	self rebuild.
! !

!NebraskaServerMorph methodsFor: 'drawing' stamp: 'KR 3/25/2008 17:28'!
updateCurrentStatusString

	self server ifNil:[
		currentStatusString _ '<Nebraska not active>' translated.
		currentBacklogString _ ''.
	] ifNotNil:[
		currentStatusString _ 
			' Nebraska: {1} clients' translated format: {self server numClients printString}.
		currentBacklogString _ 'backlog: ' translated,
				((previousBacklog _ self server backlog) // 1024) printString,'k'
	].
! !

!NebraskaServerMorph methodsFor: 'updating' stamp: 'RAA 11/8/2000 16:03'!
update: aSymbol

	self rebuild.! !

!MouseButtonEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'dgd 2/22/2003 19:00'!
decodeFromStringArray: array 
	"decode the receiver from an array of strings"

	type := array first asSymbol.
	position := CanvasDecoder decodePoint: (array second).
	buttons := CanvasDecoder decodeInteger: (array third).
	whichButton := CanvasDecoder decodeInteger: (array fourth)! !

!MouseButtonEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ar 10/25/2000 23:24'!
encodedAsStringArray
	"encode the receiver into an array of strings, such that it can be retrieved via the fromStringArray: class method"
	^{
		type.
		CanvasEncoder encodePoint: position.
		CanvasEncoder encodeInteger: buttons.
		CanvasEncoder encodeInteger: whichButton.
	}! !

!Color methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 7/31/2000 17:25'!
encodeForRemoteCanvas

	| encoded |

	CanvasEncoder at: 4 count:  1.
	(encoded := String new: 12)
		putInteger32: (rgb bitAnd: 16rFFFF) at: 1;
		putInteger32: (rgb >> 16) at: 5;
		putInteger32: self privateAlpha at: 9.
	^encoded! !

!UserInputEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ar 10/25/2000 23:23'!
encodedAsStringArray
	"encode the receiver into an array of strings, such that it can be retrieved via the fromStringArray: class method"
	^{
		type.
		CanvasEncoder encodePoint: position.
		CanvasEncoder encodeInteger: buttons.
	}! !

!MorphicEvent class methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ar 10/25/2000 23:32'!
fromStringArray: array
	"decode an event that was encoded with encodedAsStringArray"
	| type |
	type := (array at: 1).
	(type = 'mouseMove')
		ifTrue:[^MouseMoveEvent new decodeFromStringArray: array].
	(type = 'mouseDown' or:[type = 'mouseUp']) 
		ifTrue:[^MouseButtonEvent new decodeFromStringArray: array].
	(type = 'keystroke' or:[type = 'keyDown' or:[type = 'keyUp']]) 
		ifTrue:[^KeyboardEvent new decodeFromStringArray: array].
	^nil! !

PluggableCanvas subclass: #BufferedCanvas
	instanceVariableNames: 'remote previousVersion lastTick dirtyRect mirrorOfScreen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!BufferedCanvas methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 15:04'!
asBufferedCanvas

	^self! !

!BufferedCanvas methodsFor: 'as yet unclassified' stamp: 'RAA 11/7/2000 15:03'!
checkIfTimeToDisplay

	remote backlog > 0 ifTrue: [^self].	"why bother if network full?"
	dirtyRect ifNil: [^self].
	self sendDeltas.
	lastTick := Time millisecondClockValue.

! !

!BufferedCanvas methodsFor: 'drawing-support' stamp: 'RAA 7/31/2000 20:44'!
clipBy: aRectangle during: aBlock
! !

!BufferedCanvas methodsFor: 'accessing' stamp: 'RAA 11/7/2000 13:04'!
clipRect
	
	^0@0 extent: 99999@99999
! !

!BufferedCanvas methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 14:35'!
connection: connection clipRect: newClipRect transform: transform remoteCanvas: remoteCanvas

	remote := remoteCanvas.
	lastTick := 0.
! !

!BufferedCanvas methodsFor: 'Nebraska/embeddedWorlds' stamp: 'RAA 11/7/2000 13:54'!
displayIsFullyUpdated

	self checkIfTimeToDisplay! !

!BufferedCanvas methodsFor: 'drawing-general' stamp: 'RAA 7/31/2000 20:32'!
drawMorph: x
! !

!BufferedCanvas methodsFor: 'accessing' stamp: 'RAA 7/31/2000 22:36'!
extent

	^Display extent! !

!BufferedCanvas methodsFor: 'other' stamp: 'RAA 11/7/2000 15:00'!
forceToScreen: rect

	mirrorOfScreen ifNil: [
		mirrorOfScreen := (previousVersion ifNil: [Display]) deepCopy.
	].
	mirrorOfScreen copy: rect from: rect origin in: Display rule: Form over.
	dirtyRect := dirtyRect ifNil: [rect] ifNotNil: [dirtyRect merge: rect].
! !

!BufferedCanvas methodsFor: 'accessing' stamp: 'RAA 7/31/2000 22:36'!
origin

	^0@0! !

!BufferedCanvas methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 15:06'!
purgeOutputQueue! !

!BufferedCanvas methodsFor: 'as yet unclassified' stamp: 'RAA 11/7/2000 18:08'!
sendDeltas
"
NebraskaDebug showStats: #sendDeltas
"
	| t deltas dirtyFraction |

	previousVersion ifNil: [
		previousVersion := Display deepCopy.
		remote 
			image: previousVersion 
			at: 0@0 
			sourceRect: previousVersion boundingBox 
			rule: Form paint.
		^remote forceToScreen: previousVersion boundingBox.
	].
	dirtyRect ifNil: [^self].
	t := Time millisecondClockValue.
	dirtyFraction := dirtyRect area / previousVersion boundingBox area roundTo: 0.0001.

	deltas := mirrorOfScreen deltaFrom: (previousVersion copy: dirtyRect) at: dirtyRect origin.
	previousVersion := mirrorOfScreen.
	mirrorOfScreen := nil.

	remote 
		image: deltas at: dirtyRect origin sourceRect: deltas boundingBox rule: Form reverse;
		forceToScreen: dirtyRect.

	t := Time millisecondClockValue - t.
	NebraskaDebug at: #sendDeltas add: {t. dirtyFraction. deltas boundingBox}.
	dirtyRect := nil.
! !

!MouseMoveEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'dgd 2/22/2003 19:01'!
decodeFromStringArray: array 
	"decode the receiver from an array of strings"

	type := array first asSymbol.
	position := CanvasDecoder decodePoint: (array second).
	buttons := CanvasDecoder decodeInteger: (array third).
	startPoint := CanvasDecoder decodePoint: (array fourth)! !

!MouseMoveEvent methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ar 10/25/2000 23:25'!
encodedAsStringArray
	"encode the receiver into an array of strings, such that it can be retrieved via the fromStringArray: class method"
	^{
		type.
		CanvasEncoder encodePoint: position.
		CanvasEncoder encodeInteger: buttons.
		CanvasEncoder encodePoint: startPoint.
	}! !

ProjectNavigationMorph subclass: #NebraskaNavigationMorph
	instanceVariableNames: 'nebraskaBorder nebraskaTerminal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 12/13/2000 08:49'!
addButtons

	self addARow: {
		self inAColumn: {self buttonScale}.
		self inAColumn: {self buttonQuit}.
		self inAColumn: {self buttonBuffered}.
	}.
! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 12/13/2000 08:26'!
bufferNebraska

	nebraskaTerminal requestBufferedConnection
! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'KR 11/14/2007 13:29'!
buttonBuffered

	^self makeButton: 'B' 
			balloonText: 'Request buffered Nebraska session' translated 
			for: #bufferNebraska
! !

!NebraskaNavigationMorph methodsFor: 'the buttons' stamp: 'KR 11/14/2007 13:30'!
buttonQuit

	^self makeButton: 'Quit' translated 
			balloonText: 'Quit this Nebraska session' translated 
			for: #quitNebraska
! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'KR 11/14/2007 13:28'!
buttonScale

	^self makeButton: '1x1' 
			balloonText: 'Switch between 1x1 and scaled view' translated 
			for: #toggleFullView
! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 11:47'!
currentNavigatorVersion

	^1		"not particularly relevant here"! !

!NebraskaNavigationMorph methodsFor: 'initialization' stamp: 'dgd 2/16/2003 14:11'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color yellow ! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'yo 11/4/2002 21:06'!
fontForButtons

	^ TextStyle defaultFont.
	"^Preferences standardButtonFont"! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 11:35'!
nebraskaBorder: aNebraskaBorder

	nebraskaBorder := aNebraskaBorder! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 12/13/2000 08:25'!
nebraskaTerminal: aNebraskaTerminal

	nebraskaTerminal := aNebraskaTerminal! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 11:44'!
positionVertically

	| w |
	w := self world ifNil: [^self].
	self top < w top ifTrue: [self top: w top].
	self bottom > w bottom ifTrue: [self bottom: w bottom].! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 11:47'!
quitNebraska

	nebraskaBorder ifNotNil: [nebraskaBorder delete].
	self delete.! !

!NebraskaNavigationMorph methodsFor: 'stepping and presenter' stamp: 'RAA 11/8/2000 11:37'!
step

	super step.
	(nebraskaBorder isNil or: [nebraskaBorder world isNil]) ifTrue: [self delete].! !

!NebraskaNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 11/8/2000 11:48'!
toggleFullView

	nebraskaBorder ifNotNil: [nebraskaBorder toggleFullView]! !

!NebraskaNavigationMorph methodsFor: 'dropping/grabbing' stamp: 'RAA 11/8/2000 11:42'!
wantsToBeDroppedInto: aMorph

	"avoid difficulties in placement"
	^(aMorph isKindOf: NetworkTerminalMorph) not! !

!ProjectNavigationMorph methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 9/19/2000 13:43'!
buttonShare

	^self makeButton: 'Share' 
		balloonText: 'Share this project so that others can explore it with you.' 
		for: #shareThisWorld
! !

!ProjectNavigationMorph methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'RAA 8/4/2000 15:24'!
shareThisWorld

	NebraskaServerMorph serveWorld: self world! !

Morph subclass: #NetworkTerminalMorph
	instanceVariableNames: 'connection decoder eventEncoder backgroundForm enteringHand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NetworkTerminalMorph commentStamp: '<historical>' prior: 0!
A morph used to communicate with a remote image.  It sends all mouse/keyboard events to the remote side, and it displays canvas commands that are sent back.!

!NetworkTerminalMorph class methodsFor: 'instance creation' stamp: 'bf 7/1/2008 19:42'!
connectTo: hostAndPort
	| host port |
	host := hostAndPort copyUpTo: $:.
	port := (hostAndPort copyAfter: $:) asInteger.
	port ifNil: [port := NebraskaServer defaultPorts first].
	^self connectTo: host port:port
! !

!NetworkTerminalMorph class methodsFor: 'instance creation' stamp: 'RAA 8/4/2000 15:12'!
connectTo: serverHost port: serverPort

	| stringSock |

	stringSock := self socketConnectedTo: serverHost port: serverPort.
	^self new connection: stringSock
! !

!NetworkTerminalMorph class methodsFor: 'instance creation' stamp: 'RAA 7/24/2000 12:08'!
openAndConnectTo: serverHost

	^self openAndConnectTo: serverHost port: NebraskaServer defaultPort

! !

!NetworkTerminalMorph class methodsFor: 'instance creation' stamp: 'RAA 8/15/2000 10:09'!
openAndConnectTo: serverHost port: serverPort

	| stringSock me |

	stringSock := self socketConnectedTo: serverHost port: serverPort.
	me := self new connection: stringSock.
	^me openInStyle: #naked
! !

!NetworkTerminalMorph class methodsFor: 'instance creation' stamp: 'KR 3/25/2008 17:31'!
socketConnectedTo: serverHost  port: serverPort

	| sock |

	Socket initializeNetwork.
	sock _ Socket new.
	[sock connectTo: (NetNameResolver addressForName: serverHost) port: serverPort]
		on: ConnectionTimedOut
		do: [:ex | self error: 'could not connect to server' translated ].
	^StringSocket on: sock

! !

!NetworkTerminalMorph methodsFor: 'layout' stamp: 'mir 6/17/2007 20:07'!
acceptDroppingMorph: morphToDrop event: evt

	| myCopy outData null |

	(morphToDrop isKindOf: NewHandleMorph) ifTrue: [			"don't send these"
		^morphToDrop rejectDropMorphEvent: evt.
	].
	self eToyRejectDropMorph: morphToDrop event: evt.		"we don't really want it"

	"7 mar 2001 - remove #veryDeepCopy"
	myCopy _ morphToDrop.	"gradient fills require doing this second"
	myCopy setProperty: #positionInOriginatingWorld toValue: morphToDrop position.

	outData _ myCopy eToyStreamedRepresentationNotifying: nil.
	null _ String with: 0 asCharacter.
	EToyPeerToPeer new 
		sendSomeData: {
			EToyIncomingMessage typeMorph,null. 
			Preferences defaultAuthorName,null.
			outData
		}
		to: connection remoteSocketAddress hostNumber
		for: self.
! !

!NetworkTerminalMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/29/2000 12:13'!
addScalingMenuItems: menu hand: aHandMorph

	"for comaptibility when in scaled frame"! !

!NetworkTerminalMorph methodsFor: 'drawing' stamp: 'RAA 7/25/2000 13:00'!
areasRemainingToFill: aRectangle
	"I assume that we are opaque"

	^ aRectangle areasOutside: self bounds! !

!NetworkTerminalMorph methodsFor: 'event handling' stamp: 'RAA 11/5/2000 13:23'!
commResult: anArrayOfAssociations

	"ignore for now"! !

!NetworkTerminalMorph methodsFor: 'initialization' stamp: 'RAA 8/15/2000 12:06'!
connection: aConnection

	connection := aConnection.
	decoder := CanvasDecoder connection: aConnection.
	eventEncoder := MorphicEventEncoder on: aConnection.! !

!NetworkTerminalMorph methodsFor: 'shutting down' stamp: 'ls 4/11/2000 18:36'!
disconnect
	connection ifNotNil: [ connection destroy ].
	eventEncoder := connection := decoder := nil.! !

!NetworkTerminalMorph methodsFor: 'drawing' stamp: 'RAA 7/24/2000 11:43'!
drawOn: aCanvas

	backgroundForm ifNotNil: [
		aCanvas clipBy: bounds during: [ :c |
			c drawImage: backgroundForm at: bounds topLeft
		].
	].
! !

!NetworkTerminalMorph methodsFor: 'geometry' stamp: 'RAA 7/24/2000 11:35'!
extent: newExtent

	super extent: newExtent.
	eventEncoder sendViewExtent: self extent! !

!NetworkTerminalMorph methodsFor: 'drawing' stamp: 'RAA 8/28/2000 12:06'!
forceToFront: aRegion
	| highQuality |
	"force the given region from the drawing form onto the background form"

	highQuality := false.		"highQuality is slower"

	self updateBackgroundForm.
	backgroundForm
		copy: aRegion
		from: aRegion topLeft
		in: decoder drawingForm
		rule: Form over.
	self invalidRect: (
		highQuality ifTrue: [
			bounds
		] ifFalse: [
			(aRegion expandBy: 4) translateBy: bounds topLeft	"try to remove gribblys"
		]
	)
! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:02'!
handleKeyDown: anEvent
	anEvent wasHandled ifTrue:[^self].
	(self handlesKeyboard: anEvent) ifFalse:[^self].
	anEvent wasHandled: true.
	self sendEventAsIs: anEvent.! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:11'!
handleKeystroke: anEvent
	anEvent wasHandled ifTrue:[^self].
	anEvent wasHandled: true.
	self sendEventAsIs: anEvent.! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:02'!
handleKeyUp: anEvent
	anEvent wasHandled ifTrue:[^self].
	(self handlesKeyboard: anEvent) ifFalse:[^self].
	anEvent wasHandled: true.
	self sendEventAsIs: anEvent.! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:03'!
handleMouseDown: anEvent
	anEvent wasHandled ifTrue:[^self].
	anEvent hand removePendingBalloonFor: self.
	anEvent hand removePendingHaloFor: self.
	anEvent wasHandled: true.
	anEvent hand newMouseFocus: self event: anEvent.
	anEvent hand removeHaloFromClick: anEvent on: self.
	self sendEventAsIs: anEvent.! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:04'!
handleMouseMove: anEvent
	anEvent wasHandled ifTrue:[^self]. "not interested"
	(anEvent hand hasSubmorphs) ifTrue:[^self].
	(anEvent anyButtonPressed and:[anEvent hand mouseFocus ~~ self]) ifTrue:[^self].
	anEvent wasHandled: true.
	self sendEventAsIs: anEvent.! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:03'!
handleMouseUp: anEvent
	anEvent wasHandled ifTrue:[^self]. "not interested"
	anEvent hand mouseFocus == self ifFalse:[^self]. "Not interested in other parties"
	anEvent hand releaseMouseFocus: self.
	anEvent wasHandled: true.
	self sendEventAsIs: anEvent.! !

!NetworkTerminalMorph methodsFor: 'events-processing' stamp: 'ar 10/24/2000 18:06'!
handlerForMouseDown: anEvent
	^self! !

!NetworkTerminalMorph methodsFor: 'event handling' stamp: 'ar 10/26/2000 15:25'!
handlesMouseOver: evt
	^true! !

!NetworkTerminalMorph methodsFor: 'initialization' stamp: 'ar 11/2/2000 21:18'!
initialize
	super initialize.
	backgroundForm := (
		(StringMorph contents: '......' font: (TextStyle default fontOfSize: 24))
			color: Color white
	) imageForm.
	bounds := backgroundForm boundingBox.
! !

!NetworkTerminalMorph methodsFor: 'event handling' stamp: 'ar 10/26/2000 15:24'!
mouseEnter: evt
	evt hand newKeyboardFocus: self.
	evt hand needsToBeDrawn ifTrue:[Cursor blank show].! !

!NetworkTerminalMorph methodsFor: 'event handling' stamp: 'ar 10/26/2000 15:25'!
mouseLeave: evt

	evt hand needsToBeDrawn ifTrue:[Cursor normal show].! !

!NetworkTerminalMorph methodsFor: 'initialization' stamp: 'RAA 8/15/2000 10:45'!
openInStyle: aSymbol

	aSymbol == #naked ifTrue: [
		self openInWorld.
	].
	aSymbol == #scaled ifTrue: [
		self openScaled.
	].
	aSymbol == #bordered ifTrue: [
		AlignmentMorph newColumn
			hResizing: 	#shrinkWrap;
			vResizing: 	#shrinkWrap;
			borderWidth: 8;
			borderColor: Color blue;
			addMorph: self;
			openInWorld.
	].

	[
		[self world isNil] whileFalse: [(Delay forSeconds: 2) wait].
		self disconnect.
	] fork.

! !

!NetworkTerminalMorph methodsFor: 'initialization' stamp: 'RAA 12/13/2000 08:25'!
openScaled

	| window tm |
	window := NetworkTerminalBorderMorph new
		minWidth: 100;
		minHeight: 100;
		borderWidth: 8;
		borderColor: Color orange;
		bounds: (0@0 extent: Display extent * 3 // 4).
	tm := BOBTransformationMorph new.
	tm useRegularWarpBlt: true.		"try to reduce memory used"
	window addMorphBack: tm.
	tm addMorph: self.
	window openInWorld.
	NebraskaNavigationMorph new 
		nebraskaBorder: window;
		nebraskaTerminal: self;
		openInWorld.! !

!NetworkTerminalMorph methodsFor: 'shutting down' stamp: 'RAA 12/13/2000 08:21'!
requestBufferedConnection

	eventEncoder ifNotNil: [eventEncoder requestBufferedConnection].
! !

!NetworkTerminalMorph methodsFor: 'event handling' stamp: 'RAA 8/15/2000 11:33'!
sendEventAsIs: evt

	eventEncoder ifNil: [ ^self ].
	eventEncoder sendEvent: evt.! !

!NetworkTerminalMorph methodsFor: 'event handling' stamp: 'RAA 8/15/2000 11:34'!
sendEvent: evt

	self sendEventAsIs: (evt translatedBy: bounds topLeft negated).! !

!NetworkTerminalMorph methodsFor: 'stepping and presenter' stamp: 'ar 10/25/2000 23:19'!
step

	decoder ifNil: [ ^self ].
	decoder processIOOnForce: [ :rectangle | self forceToFront: rectangle ].! !

!NetworkTerminalMorph methodsFor: 'testing' stamp: 'ls 3/25/2000 16:58'!
stepTime
	^10! !

!NetworkTerminalMorph methodsFor: 'drawing' stamp: 'RAA 8/28/2000 11:28'!
updateBackgroundForm
	"make sure that our background form matches what the server has most recently requested"

	| drawingForm |

	drawingForm := decoder drawingForm.
	(drawingForm extent = backgroundForm extent and: [
		drawingForm depth = backgroundForm depth ]) ifTrue: [
			"they match just fine"
			^self ].

	backgroundForm := drawingForm deepCopy.		"need copy to capture the moment"
	self extent: backgroundForm extent.! !

!NetworkTerminalMorph methodsFor: 'dropping/grabbing' stamp: 'RAA 11/5/2000 13:26'!
wantsDroppedMorph: aMorph event: evt

	^true.! !

NetworkTerminalMorph subclass: #NullTerminalMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!NullTerminalMorph methodsFor: 'drawing' stamp: 'RAA 7/22/2000 07:20'!
drawOn: aCanvas

	aCanvas fillRectangle: self bounds fillStyle: Color orange.
	aCanvas frameRectangle: self bounds color: Color black! !

!NullTerminalMorph methodsFor: 'geometry' stamp: 'RAA 7/22/2000 07:21'!
extent: newExtent

	| aPoint |
	aPoint := 50@50.
	bounds extent = aPoint ifFalse: [
		self changed.
		bounds := bounds topLeft extent: aPoint.
		self layoutChanged.
		self changed
	].
	eventEncoder sendViewExtent: newExtent! !

!NullTerminalMorph methodsFor: 'drawing' stamp: 'RAA 7/22/2000 07:22'!
forceToFront: aRegion
	"force the given region from the drawing form onto the background form"
	self updateBackgroundForm.

! !

!MatrixTransform2x3 class methodsFor: '*nebraska-instance creation' stamp: 'ls 10/9/1999 19:51'!
fromRemoteCanvasEncoding: encoded
	"DisplayTransform fromRemoteCanvasEncoding:  'Matrix,1065353216,0,1137541120,0,1065353216,1131610112,'"
	| nums transform encodedNums |
	"split the numbers up"
	encodedNums := encoded findTokens: ','.

	"remove the initial 'Matrix' specification"
	encodedNums := encodedNums asOrderedCollection.
	encodedNums removeFirst.

	"parse the numbers"
	nums := encodedNums collect: [ :enum |
		Integer readFromString: enum ].

	"create an instance"
	transform := self new.

	"plug in the numbers"
	nums doWithIndex: [ :num :i |
		transform basicAt: i put: num ].

	^transform! !

!MatrixTransform2x3 methodsFor: '*nebraska-*nebraska-Morphic-Remote' stamp: 'ls 10/9/1999 19:03'!
encodeForRemoteCanvas
	"encode this transform into a string for use by a RemoteCanvas"
	^String streamContents: [ :str |
		str nextPutAll: 'Matrix,'.
		1 to: 6 do: [ :i |
			str print: (self basicAt: i).
			str nextPut: $, ].
	]! !

Canvas subclass: #RemoteCanvas
	instanceVariableNames: 'innerClipRect outerClipRect transform connection shadowColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nebraska-Morphic-Remote'!

!RemoteCanvas commentStamp: '<historical>' prior: 0!
A canvas which draws to a terminal across the network.  Note that multiple RemoteCanvas's might exist for a single MREncoder, each having different transformations and clipping rectangles.!

!RemoteCanvas class methodsFor: 'instance creation' stamp: 'ls 10/20/1999 21:14'!
connection: connection  clipRect: clipRect  transform: transform
	^self new connection: connection clipRect: clipRect transform: transform! !

!RemoteCanvas methodsFor: 'as yet unclassified' stamp: 'RAA 8/1/2000 08:17'!
apply: ignored

	"added for the convenience of BufferedCanvas"! !

!RemoteCanvas methodsFor: 'initialization' stamp: 'RAA 11/8/2000 15:00'!
asBufferedCanvas

	| bufferedCanvas |

	bufferedCanvas := BufferedCanvas new.
	connection cachingEnabled: false.
	bufferedCanvas
		connection: connection
		clipRect: NebraskaServer extremelyBigRectangle
		transform: MorphicTransform identity
		remoteCanvas: self.
	^bufferedCanvas! !

!RemoteCanvas methodsFor: 'nil' stamp: 'RAA 8/1/2000 00:13'!
backlog

	^connection backlog! !

!RemoteCanvas methodsFor: 'drawing' stamp: 'RAA 11/6/2000 15:26'!
balloonFillOval: aRectangle fillStyle: aFillStyle borderWidth: bw borderColor: bc

	self drawCommand: [ :executor |
		executor balloonFillOval: aRectangle fillStyle: aFillStyle borderWidth: bw borderColor: bc
	].! !

!RemoteCanvas methodsFor: 'drawing' stamp: 'RAA 7/28/2000 07:41'!
balloonFillRectangle: aRectangle fillStyle: aFillStyle

	self drawCommand: [ :executor |
		executor balloonFillRectangle: aRectangle fillStyle: aFillStyle
	].! !

!RemoteCanvas methodsFor: 'drawing-support' stamp: 'RAA 3/3/2001 19:05'!
clipBy: aRectangle during: aBlock
	| newCanvas newR |
	"Set a clipping rectangle active only during the execution of aBlock."

	newR := transform localBoundsToGlobal: aRectangle.

	newCanvas := RemoteCanvas 
		connection: connection 
		clipRect: (outerClipRect intersect: newR) 
		transform: transform.
	newCanvas privateShadowColor: shadowColor.
	aBlock value: newCanvas.
	connection shadowColor: shadowColor.! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'ls 3/19/2000 15:56'!
clipRect
	^innerClipRect! !

!RemoteCanvas methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
connection: connection0 clipRect: clipRect0 transform: transform0
	connection := connection0.
	outerClipRect := clipRect0.
	transform := transform0.


	innerClipRect := transform globalBoundsToLocal: outerClipRect. ! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'RAA 11/6/2000 14:17'!
contentsOfArea: aRectangle into: aForm
	"this should never be called; normally, RemoteCanvas's are used in conjunction with a CachingCanvas"

	self flag: #roundedRudeness.	

	"aForm fillWhite.
	^aForm"

	^Display getCanvas contentsOfArea: aRectangle into: aForm! !

!RemoteCanvas methodsFor: 'private' stamp: 'ls 3/27/2000 18:50'!
drawCommand: aBlock
	"set up the connection for a drawing command, and then execute aBlock with the connection as an argument"
	connection updateTransform: transform andClipRect: outerClipRect.
	aBlock value: connection! !

!RemoteCanvas methodsFor: 'drawing-polygons' stamp: 'ls 3/25/2000 22:44'!
drawPolygon: vertices color: aColor borderWidth: bw borderColor: bc
	self drawCommand: [ :c |
		c drawPolygon: vertices color: aColor borderWidth: bw borderColor: bc ]! !

!RemoteCanvas methodsFor: 'drawing-text' stamp: 'yo 10/6/2005 15:57'!
drawString: s from: firstIndex to: lastIndex in: boundsRect font: fontOrNil color: c
	"Draw the given string in the given font and color clipped to the given rectangle. If the font is nil, the default font is used."
	"(innerClipRect intersects: (transform transformBoundsRect: boundsRect)) ifFalse: [ ^self ]."
		"clip rectangles seem to be all screwed up...."
	s isAllSeparators ifTrue: [ ^self ].   "is this correct??  it sure does speed things up!!"
	self drawCommand: [ :executor |
		executor drawString: s from: firstIndex to: lastIndex in: boundsRect font: fontOrNil color: (self isShadowDrawing ifTrue: [self shadowColor] ifFalse: [c])]! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'ls 4/8/2000 22:33'!
extent
	self flag: #hack.
	^1500@1500! !

!RemoteCanvas methodsFor: 'drawing-ovals' stamp: 'ls 10/9/1999 18:36'!
fillOval: bounds color: color borderWidth: borderWidth borderColor: borderColor
	"Fill the given oval."
	self drawCommand: [ :executor |
		executor fillOval: bounds color: color borderWidth: borderWidth borderColor: borderColor
	].! !

!RemoteCanvas methodsFor: 'drawing-ovals' stamp: 'RAA 11/6/2000 15:25'!
fillOval: aRectangle fillStyle: aFillStyle borderWidth: bw borderColor: bc
	"Fill the given oval."
	self shadowColor ifNotNil: [
		^self fillOval: aRectangle color: aFillStyle asColor borderWidth: bw borderColor: bc
	].
	(aFillStyle isBitmapFill and:[aFillStyle isKindOf: InfiniteForm]) ifTrue:[
		self flag: #fixThis.
		^self fillOval: aRectangle color: aFillStyle borderWidth: bw borderColor: bc
	].
	(aFillStyle isSolidFill) ifTrue:[
		^self fillOval: aRectangle color: aFillStyle asColor borderWidth: bw borderColor: bc
	].
	"Use a BalloonCanvas instead"
	self balloonFillOval: aRectangle fillStyle: aFillStyle borderWidth: bw borderColor: bc! !

!RemoteCanvas methodsFor: 'drawing-rectangles' stamp: 'yo 10/6/2005 15:53'!
fillRectangle: aRectangle fillStyle: aFillStyle
	"Fill the given rectangle."
	| pattern |
	(self isShadowDrawing not and: [self shadowColor notNil]) ifTrue:
		[^self fillRectangle: aRectangle color: self shadowColor].

	(aFillStyle isKindOf: InfiniteForm) ifTrue: [
		^self infiniteFillRectangle: aRectangle fillStyle: aFillStyle
	].

	(aFillStyle isSolidFill) 
		ifTrue:[^self fillRectangle: aRectangle color: aFillStyle asColor].
	"We have a very special case for filling with infinite forms"
	(aFillStyle isBitmapFill and:[aFillStyle origin = (0@0)]) ifTrue:[
		pattern := aFillStyle form.
		(aFillStyle direction = (pattern width @ 0) 
			and:[aFillStyle normal = (0@pattern height)]) ifTrue:[
				"Can use an InfiniteForm"
				^self fillRectangle: aRectangle color: (InfiniteForm with: pattern)].
	].
	"Use a BalloonCanvas instead"
	self balloonFillRectangle: aRectangle fillStyle: aFillStyle.! !

!RemoteCanvas methodsFor: 'initialization' stamp: 'ls 3/21/2000 23:22'!
flush
	connection ifNotNil: [ connection flush ]! !

!RemoteCanvas methodsFor: 'other' stamp: 'RAA 7/20/2000 17:47'!
forceToScreen: rect

		self drawCommand: [ :exec |
			exec forceToScreen: rect ]! !

!RemoteCanvas methodsFor: 'drawing-rectangles' stamp: 'RAA 8/25/2000 13:13'!
frameAndFillRectangle: bounds fillColor: fillColor borderWidth: borderWidth borderColor: borderColor
	"Draw the rectangle using the given attributes"

	self drawCommand: [ :executor |
		executor 
			frameAndFillRectangle: bounds 
			fillColor: fillColor 
			borderWidth: borderWidth 
			borderColor: borderColor
	].! !

!RemoteCanvas methodsFor: 'private' stamp: 'ls 10/9/1999 18:38'!
image: aForm at: aPoint sourceRect: sourceRect rule: rule
	"Draw the given form."
	self drawCommand: [ :executor |
		executor image: aForm at: aPoint sourceRect: sourceRect rule: rule
	].! !

!RemoteCanvas methodsFor: 'drawing' stamp: 'RAA 8/25/2000 14:49'!
infiniteFillRectangle: aRectangle fillStyle: aFillStyle

	self drawCommand: [ :c |
		c infiniteFillRectangle: aRectangle fillStyle: aFillStyle
	]! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'yo 10/6/2005 14:44'!
isShadowDrawing
	^ self shadowColor notNil! !

!RemoteCanvas methodsFor: 'drawing' stamp: 'ls 10/9/1999 18:25'!
line: point1 to: point2 width: width color: color
	"Draw a line using the given width and color"
	self drawCommand: [ :executor |
		executor line: point1 to: point2 width: width color: color ]! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'ls 4/8/2000 22:33'!
origin
	^0@0! !

!RemoteCanvas methodsFor: 'drawing' stamp: 'yo 6/23/2003 18:09'!
paragraph: paragraph bounds: bounds color: c

	| scanner |
	scanner := CanvasCharacterScanner new.
	scanner
		 canvas: self;
		text: paragraph text textStyle: paragraph textStyle;
		textColor: c; defaultTextColor: c.

	paragraph displayOn: self using: scanner at: bounds topLeft.
! !

!RemoteCanvas methodsFor: 'drawing-support' stamp: 'RAA 3/3/2001 18:43'!
privateShadowColor: x

	shadowColor := x.
! !

!RemoteCanvas methodsFor: 'misc' stamp: 'ls 2/28/2000 00:16'!
processIO
	connection processIO! !

!RemoteCanvas methodsFor: 'initialization' stamp: 'RAA 11/8/2000 15:06'!
purgeOutputQueue

	connection purgeOutputQueue.! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'RAA 3/4/2001 08:15'!
shadowColor

	^shadowColor! !

!RemoteCanvas methodsFor: 'accessing' stamp: 'RAA 3/3/2001 18:42'!
shadowColor: x

	connection shadowColor: (shadowColor := x).
! !

!RemoteCanvas methodsFor: 'drawing-support' stamp: 'ls 3/26/2000 00:07'!
showAt: pt invalidRects: updateRects
	updateRects do: [ :rect |
		self drawCommand: [ :exec |
			exec forceToScreen: rect ] ]! !

!RemoteCanvas methodsFor: 'drawing-images' stamp: 'ls 3/26/2000 13:25'!
stencil: stencilForm at: aPoint sourceRect: sourceRect color: aColor
	"Flood this canvas with aColor wherever stencilForm has non-zero pixels"
	self drawCommand: [ :executor |
		executor stencil: stencilForm at: aPoint sourceRect: sourceRect color: aColor ]! !

!RemoteCanvas methodsFor: 'Nebraska/embeddedWorlds' stamp: 'RAA 12/11/2000 12:30'!
transform2By: aDisplayTransform clippingTo: aClipRect during: aBlock smoothing: cellSize

	(aDisplayTransform isPureTranslation) ifTrue: [
		^self 
			transformBy: aDisplayTransform 
			clippingTo: aClipRect 
			during: aBlock 
			smoothing: cellSize
	].
	^super 
		transform2By: aDisplayTransform 
		clippingTo: aClipRect 
		during: aBlock 
		smoothing: cellSize
! !

!RemoteCanvas methodsFor: 'drawing-support' stamp: 'RAA 3/3/2001 18:43'!
transformBy: aDisplayTransform clippingTo: aClipRect during: aBlock smoothing: cellSize
	| newCanvas |

	self flag: #bob.		"do tranform and clip work together properly?"
	newCanvas := RemoteCanvas 
		connection: connection 
		clipRect: (aClipRect intersect: outerClipRect)
		transform: (transform composedWith: aDisplayTransform).
	newCanvas privateShadowColor: shadowColor.
	aBlock value: newCanvas.
	connection shadowColor: shadowColor.! !

!RemoteCanvas methodsFor: 'drawing-support' stamp: 'ls 3/26/2000 20:26'!
translateBy: delta during: aBlock
	self transformBy: (MorphicTransform offset: delta negated) clippingTo: outerClipRect during: aBlock smoothing: 1! !

!ByteString methodsFor: '*Nebraska' stamp: 'ul 2/28/2011 05:20'!
getInteger32: location
	"Read a SmallInteger stored in the receiver using big-endian one's complement representation. Avoid creation of LargeIntegers."

	| firstByte |
	(firstByte := self byteAt: location) < 64 ifTrue: [ 
		^(((((
			firstByte bitShift: 8) bitOr:
			(self byteAt: location + 1)) bitShift: 8) bitOr: 
			(self byteAt: location + 2)) bitShift: 8) bitOr: 
			(self byteAt: location + 3) ].
	^(64 - firstByte bitShift: 24) - (((((
		(self byteAt: location + 1)) bitShift: 8) bitOr: 
		(self byteAt: location + 2)) bitShift: 8) bitOr: 
		(self byteAt: location + 3))
! !

!ByteString methodsFor: '*Nebraska' stamp: 'ul 2/28/2011 06:05'!
putInteger32: anInteger at: location
	"Store a SmallInteger in the receiver using big-endian one's complement representation. Avoid creation of LargeIntegers."
	
	| integer firstByte |
	(integer := anInteger) >= 0 
		ifTrue: [ firstByte := (anInteger bitShift: -24) bitAnd: 255 ]
		ifFalse: [
			integer := 0 - integer.
			firstByte := (integer bitShift: -24) + 64 bitAnd: 255 ].
	self 
		byteAt: location put: firstByte;
		byteAt: location + 1 put: ((integer bitShift: -16) bitAnd: 255);
		byteAt: location + 2 put: ((integer bitShift: -8) bitAnd: 255);
		byteAt: location + 3 put: (integer bitAnd: 255)
! !
AudioChatGUI initialize!
EToyListenerMorph initialize!
CanvasDecoder initialize!
